/**
  *  RELOAD Editor - An IMS Metadata, Content Packaging and Learning Design tool.
  *
  *  Copyright (c) 2003 Oleg Liber, Bill Olivier, Phillip Beauvoir
  *
  *  Permission is hereby granted, free of charge, to any person obtaining a copy
  *  of this software and associated documentation files (the "Software"), to deal
  *  in the Software without restriction, including without limitation the rights
  *  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
  *  copies of the Software, and to permit persons to whom the Software is
  *  furnished to do so, subject to the following conditions:
  *
  *  The above copyright notice and this permission notice shall be included in
  *  all copies or substantial portions of the Software.
  *
  *  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  *  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  *  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
  *  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
  *  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
  *  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
  *
  *  Project Management Contact:
  *
  *  Oleg Liber
  *  Bolton Institute of Higher Education
  *  Deane Road
  *  Bolton BL3 5AB
  *  UK
  *
  *  e-mail:   o.liber@bolton.ac.uk
  *
  *
  *  Technical Contact:
  *
  *  Phillip Beauvoir
  *  e-mail:   p.beauvoir@bolton.ac.uk
  *
  *  Web:      http://www.reload.ac.uk
  *
  */

package uk.ac.reload;

import java.beans.*;
import java.io.*;
import java.util.*;

import javax.swing.*;
import javax.swing.event.*;

import uk.ac.reload.menu.*;



/**
 * The Reload Internal Frame Manager.  This manages the workings of the ReloadInternalFrame
 * objects so that we can hide or show them close them down, inform listeners etc.<br>
 *
 * @author Phillip Beauvoir
 * @version 2003-05-19
 */
public class ReloadInternalFrameManager
implements InternalFrameListener
{
    /**
     *  Current X Pos for Window Pos
     */
    protected static int _xpos = -25;

    /**
     *  Current Y Pos for Window Pos
     */
    protected static int _ypos = -25;

    /**
     *  The stagger offset
     */
    protected static final int WINDOWGAP = 25;

    /**
     * The relative size factor
     */
    protected static final double SIZE_FACTOR = 0.9;

    /**
     * The Singleton instance
     */
    private static ReloadInternalFrameManager _instance;

    private ReloadInternalFrame focussedInternalFrame = null;


    private ReloadInternalFrameManager() {

    }

    /**
     * Gets the Singleton Instance of the Manager
     * @return The Singleton Instance of the Manager
     */
    public static ReloadInternalFrameManager getInstance() {
        if(_instance == null) _instance = new ReloadInternalFrameManager();
        return _instance;
    }

    /**
     * Add a new ReloadInternalFrame
     * @param frame The new Frame
     */
    public void addInternalFrame(ReloadInternalFrame frame) {
        // Add us as a frame listener
        frame.addInternalFrameListener(this);

        // Add to main menu
        MainMenu.sharedInstance.addInternalFrame(frame);

        JDesktopPane desktop = ReloadEditor.getInstance().getDesktop();

        // Make it a size relative to the desktop size
        int width = (int)((double)desktop.getWidth() * SIZE_FACTOR);
        if(width < 100) width = 100;
        int height = (int)((double)desktop.getHeight() * SIZE_FACTOR);
        if(height < 100) height = 100;

        // Set the position
        _xpos += WINDOWGAP;
        _ypos += WINDOWGAP;

        if(_xpos + width > desktop.getWidth()  || _ypos + height > desktop.getHeight()) {
            _xpos = 0;
            _ypos = 0;
        }

        frame.setBounds(_xpos, _ypos, width, height);

        // Add the window to the Desktop - it's up to the caller to show it!
        desktop.add(frame, JLayeredPane.DEFAULT_LAYER);
    }

    /**
     * Remove an existing ReloadInternalFrame and select the Previous one
     * @param frame The Frame to remove
     */
    public void removeInternalFrame(ReloadInternalFrame frame) {
        JDesktopPane desktop = ReloadEditor.getInstance().getDesktop();

        // Find the Last frame so we can select it
        JInternalFrame[] frames = desktop.getAllFrames();
        if(frames.length > 0) {
            try {
                frames[frames.length - 1].setSelected(true);
            }
            catch(PropertyVetoException ex) {
                ex.printStackTrace();
            }
        }

        // Remove us as a frame listener
        frame.removeInternalFrameListener(this);

        // Remove from main menu
        MainMenu.sharedInstance.removeInternalFrame(frame);

        // Remove from Tracking
        removeInternalFrameFromWindowMap(frame);
    }

    /**
     * Cascade all the windows
     */
    public void cascade() {
        JDesktopPane desktop = ReloadEditor.getInstance().getDesktop();

        // Make it a size relative to the desktop size
        int width = (int)((double)desktop.getWidth() * SIZE_FACTOR);
        if(width < 100) width = 100;
        int height = (int)((double)desktop.getHeight() * SIZE_FACTOR);
        if(height < 100) height = 100;

        _xpos = -25;
        _ypos = -25;

        JInternalFrame[] frames = desktop.getAllFrames();
        for(int i = 0; i < frames.length; i++) {
            JInternalFrame f = frames[i];

            _xpos += WINDOWGAP;
            _ypos += WINDOWGAP;

            if(_xpos + width > desktop.getWidth()  || _ypos + height > desktop.getHeight()) {
                _xpos = 0;
                _ypos = 0;
            }

            try {
                f.setIcon(false);
                f.setMaximum(false);
                f.setBounds(_xpos, _ypos, width, height);
                f.setSelected(true);
            }
            catch (java.beans.PropertyVetoException e) {}
        }
    }

    /**
     * Tile all the windows vertically
     */
    public void tileVertical() {
        JDesktopPane desktop = ReloadEditor.getInstance().getDesktop();

        JInternalFrame[] frames = desktop.getAllFrames();
        if(frames.length == 0) return;

        int height = desktop.getHeight();
        int width = desktop.getWidth() / frames.length;

        _xpos = 0;
        _ypos = 0;

        for (int i = 0; i < frames.length; i++) {
            JInternalFrame f = frames[i];

            try {
                f.setIcon(false);
                f.setMaximum(false);
                f.setBounds(_xpos, _ypos, width, height);
                f.setSelected(true);
            }
            catch (java.beans.PropertyVetoException e) {}

            _xpos += width;
        }
    }

    /**
     * Tile all the windows horizontally
     */
    public void tileHorizontal() {
        JDesktopPane desktop = ReloadEditor.getInstance().getDesktop();

        JInternalFrame[] frames = desktop.getAllFrames();
        if(frames.length == 0) return;

        int height = desktop.getHeight() / frames.length;
        int width = desktop.getWidth();

        _xpos = 0;
        _ypos = 0;

        for (int i = 0; i < frames.length; i++) {
            JInternalFrame f = frames[i];

            try {
                f.setIcon(false);
                f.setMaximum(false);
                f.setBounds(_xpos, _ypos, width, height);
                f.setSelected(true);
            }
            catch (java.beans.PropertyVetoException e) {}

            _ypos += height;
        }
    }

    public ReloadInternalFrame getFocussedInternalFrame() {

		return this.focussedInternalFrame;
	}

    ////////////////////////////////////////////////////////////////////////////
    // Listeners to InternalFrameListener
    ////////////////////////////////////////////////////////////////////////////

    public void internalFrameOpened(InternalFrameEvent e) {
    }

    public void internalFrameClosing(InternalFrameEvent e) {
    }

    /**
     * The Internal frame is closed so we need to remove it.
     * @param e
     */
    public void internalFrameClosed(InternalFrameEvent e) {
        Object o = e.getSource();
        if(o instanceof ReloadInternalFrame) removeInternalFrame((ReloadInternalFrame)o);
    }

    public void internalFrameIconified(InternalFrameEvent e) {
    }

    public void internalFrameDeiconified(InternalFrameEvent e) {
    }

    public void internalFrameActivated(InternalFrameEvent e) {
        Object o = e.getSource();
        if(o instanceof ReloadInternalFrame) this.focussedInternalFrame = (ReloadInternalFrame)o;
    }

    public void internalFrameDeactivated(InternalFrameEvent e) {
    }


//==============================================================================
//==================== TRACKING OF OPENED FRAMES ==========================
//==============================================================================


    private static Hashtable FileToWindowMAP = new Hashtable();

    /**
     * See if we have the Window open already
     */
    public static ReloadInternalFrame getReloadInternalFrame(File file) {
        return file == null ? null : (ReloadInternalFrame)FileToWindowMAP.get(file);
    }

    /**
     * Add a Frame to the Window map
     */
    public static void addReloadInternalFrameToWindowMap(File file, ReloadInternalFrame frame) {
        // Remove any old one
        if(frame != null) removeInternalFrameFromWindowMap(frame);
        // And add it
        if(file != null && frame != null) FileToWindowMAP.put(file, frame);
    }

    /**
     * This will be called when the window is closed from removeInternalFrame
     * @param frame
     */
    private static void removeInternalFrameFromWindowMap(ReloadInternalFrame frame) {
        if(frame != null && FileToWindowMAP.contains(frame)) {
            Enumeration e = FileToWindowMAP.keys();
            while(e.hasMoreElements()) {
                File file = (File) e.nextElement();
                ReloadInternalFrame f = (ReloadInternalFrame) FileToWindowMAP.get(file);
                if(frame == f) FileToWindowMAP.remove(file);
            }
        }
    }
}