/**
 *  RELOAD Editor - An IMS Metadata, Content Packaging and Learning Design tool.
 *
 *  Copyright (c) 2003 Oleg Liber, Bill Olivier, Phillip Beauvoir
 *
 *  Permission is hereby granted, free of charge, to any person obtaining a copy
 *  of this software and associated documentation files (the "Software"), to deal
 *  in the Software without restriction, including without limitation the rights
 *  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 *  copies of the Software, and to permit persons to whom the Software is
 *  furnished to do so, subject to the following conditions:
 *
 *  The above copyright notice and this permission notice shall be included in
 *  all copies or substantial portions of the Software.
 *
 *  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 *  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 *  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 *  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 *  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 *  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 *
 *  Project Management Contact:
 *
 *  Oleg Liber
 *  Bolton Institute of Higher Education
 *  Deane Road
 *  Bolton BL3 5AB
 *  UK
 *
 *  e-mail:   o.liber@bolton.ac.uk
 *
 *
 *  Technical Contact:
 *
 *  Phillip Beauvoir
 *  e-mail:   p.beauvoir@bolton.ac.uk
 *
 *  Web:      http://www.reload.ac.uk
 *
 */

package uk.ac.reload.contentpackaging;

import java.io.*;
import java.net.*;
import java.util.*;

import org.jdom.*;
import uk.ac.reload.*;
import uk.ac.reload.contentpackaging.resourceview.*;
import uk.ac.reload.dom.*;
import uk.ac.reload.dom.ReloadDocument.*;
import uk.ac.reload.metadata.*;
import uk.ac.reload.schema.*;
import uk.ac.reload.util.*;
import uk.ac.reload.util.htmlparser.*;

/**
 * The Content Package Class
 *
 * @author Phillip Beauvoir
 * @version 2003-07-29
 */
public class ContentPackage
extends AbstractContentPackage
{
    public static final String MANIFEST = "manifest";
    public static final String ORGANIZATIONS = "organizations";
    public static final String RESOURCES = "resources";
    public static final String DEFAULT = "default";
    public static final String ORGANIZATION = "organization";
    public static final String ITEM = "item";
    public static final String PARAMETERS = "parameters";
    public static final String RESOURCE = "resource";
    public static final String BASE = "base";
    public static final String FILE = "file";
    public static final String TYPE = "type";
    public static final String HREF = "href";
    public static final String METADATA = "metadata";
    public static final String IDENTIFIER = "identifier";
    public static final String IDENTIFIERREF = "identifierref";
    public static final String TITLE = "title";
    public static final String DEPENDENCY = "dependency";
    public static final String VERSION = "version";
    public static final String SCHEMA = "schema";
    public static final String SCHEMAVERSION = "schemaversion";

    /**
     * Our unique signature
     */
    static final String[] cp_comments = {
        "This is a Reload version " + ReloadEditor.VERSION + " Content Package document",
        "Spawned from the Reload Content Package Generator - http://www.reload.ac.uk"
    };

    /**
     * Default Constructor
     */
    public ContentPackage() {
        super();
    }

    /**
     * Constructor
     */
    public ContentPackage(File file, Document doc, ContentPackageSchemaController schemaController) {
        super(file, doc, schemaController);
    }

    /**
     * Set up New CP with new project folder, imsmanifest.xml amd Schema files
     */
    public void newContentPackage(File projectFolder, ContentPackageSchemaController cpController, MetadataSchemaController mdController) throws ReloadException {
        // New Default Document
        newDocument(projectFolder, cpController, mdController);

        // Save it
        saveDocument();

        // Clear the dirty flag
        setDirty(false);

        // Copy the Schema Files to the Project Folder
        cpController.copySchemaFilesToFolder(projectFolder);
    }

    /**
     * Setup the JDOM Document with Namespaces set and root element set and some
     * default attributes.
     */
    protected void newDocument(File projectFolder, ContentPackageSchemaController cpController, MetadataSchemaController mdController) throws ReloadException {
        setReloadSchemaController(cpController);

        _doc = new Document();

        // Add our signature
        addComments(_doc);

        // Content Packaging Schema
        ReloadSchema cpSchema = cpController.getReloadSchema();

        // Set Root Element
        Element root = addElementBySchema(this, null, cpSchema.getRootElement(), false);
        _doc.setRootElement(root);

        // Metadata Schema
        ReloadSchema mdSchema = mdController.getReloadSchema();

        // Add Metadata Namespace
        Namespace mdNamespace = Namespace.getNamespace(ReloadDocumentHandler.IMSMD_NAMESPACE_PREFIX, mdSchema.getTargetNamespace());
        root.addNamespaceDeclaration(mdNamespace);

        // Add XSI Schema Instance Namespace
        root.addNamespaceDeclaration(ReloadDocumentHandler.XSI_Namespace);

        // Identifier attribute
        // Note - the JDOM XMLOutputter.printElement() method prints attributes AFTER namespaces
        root.setAttribute(IDENTIFIER,  RandomGUID.getUniqueID("MANIFEST-"));

        // Add Schema Location Attribute which is constructed from Target Namespace
        // and file name of Schema, and also with Metadata Schema location
        StringBuffer schemaLocationURI = new StringBuffer();
        schemaLocationURI.append(cpSchema.getTargetNamespace());
        schemaLocationURI.append(" ");
        schemaLocationURI.append(cpSchema.getSchemaName());
        schemaLocationURI.append(" ");
        schemaLocationURI.append(mdSchema.getTargetNamespace());
        schemaLocationURI.append(" ");
        schemaLocationURI.append(mdSchema.getSchemaName());
        root.setAttribute(ReloadDocumentHandler.XSI_SchemaLocation, schemaLocationURI.toString(), ReloadDocumentHandler.XSI_Namespace);

        // Set as File
        _file = new File(projectFolder, ReloadDocumentHandler.MANIFEST_NAME);
    }


    /**
     * @return the Comments to add to the XML Document
     */
    public String[] getComments() {
        return cp_comments;
    }

    /**
     * Import (aggregate) another Package into this Content Package at manifestElement
     */
    public Element importManifest(File manifestFile, Element manifestElement, String base) throws ReloadException {
        Element importedRootElement = null;

        // Read in imsmanifest.xml
        Document importedDoc = XMLUtils.readXMLFile(manifestFile);
        if(importedDoc != null) {
            // Root Element detach
            importedRootElement = (Element)importedDoc.getRootElement().detach();

            // Check we got the right kind of file
            String rootSource = importedRootElement.getName();
            String rootTarget = getRootElementName();
            if(!rootSource.equals(rootTarget)) throw new ReloadException("Not a valid file");

            // Set Namespace to ours
            Namespace ns = importedRootElement.getNamespace();
            if(ns != null) ReloadDocumentHandler.replaceNamespaces(importedRootElement, ns, getSchemaNamespace());

            // Remove stuff
            importedRootElement.removeAttribute(ReloadDocumentHandler.XSI_SchemaLocation, ReloadDocumentHandler.XSI_Namespace);
            importedRootElement.removeAttribute(ReloadDocumentHandler.XSI_SchemaLocation, ReloadDocumentHandler.XSI_NamespaceOLD);
            importedRootElement.removeNamespaceDeclaration(ReloadDocumentHandler.XSI_Namespace);
            importedRootElement.removeNamespaceDeclaration(ReloadDocumentHandler.XSI_NamespaceOLD);

            // Remove Namespaces
            Vector v = new Vector(); // Use a copy of the list so we don't get a concurrent modification exception
            Iterator it = importedRootElement.getAdditionalNamespaces().listIterator();
            while(it.hasNext()) {
                ns = (Namespace) it.next();
                v.add(ns);
            }
            it = v.iterator();
            while(it.hasNext()) {
                ns = (Namespace) it.next();
                importedRootElement.removeNamespaceDeclaration(ns);
                // Add to root
                Namespace ns2 = getRootElement().getNamespace(ns.getPrefix());
                if(ns2 == null) getRootElement().addNamespaceDeclaration(ns);
            }

            // Create new IDENTIFIERs and map to old IDENTIFIERs
            Hashtable idmap = createNewIDs(importedRootElement);
            // Change each IDENTIFIERREF to new IDENTIFIERs
            createNewIDRefs(importedRootElement, idmap);
            // SCORM
            createNewPrerequisiteRefs(importedRootElement, idmap);

            // Set Base attribute to target folder
            importedRootElement.setAttribute(BASE, base + "/", Namespace.XML_NAMESPACE);

            // Add it
            addElementUndoable(this, manifestElement, importedRootElement, false);
        }

        return importedRootElement;
    }

    /**
     * Export (disagggregate) manifestElement to new Content Package
     */
    public boolean exportManifest(Element manifestElement, File targetFolder) throws ReloadException {
        // If sub-Manifest has valid xml:base property, find sub-Folder of that name in current package
        // If this sub-Folder exists and and imsmanifest.xml exists, copy contents to new folder. Finish.
        String base = getElementBase(manifestElement);
        if(base != null) {
            File sourceFolder = new File(getProjectFolder(), base);
            if(sourceFolder.exists() && targetFolder.exists()) {
                try {
                    // Do we have an imsmanifest.xml file?
                    File imsmanifestFile = new File(sourceFolder, ReloadDocumentHandler.MANIFEST_NAME);
                    if(imsmanifestFile.exists()) {
                        ReloadUtils.copyFolder(sourceFolder, targetFolder, null);
                        return true;
                    }
                }
                catch(IOException ex) {
                    throw new ReloadException(ex);
                }
            }
        }

        // Nope - so try this:-

        // Walk the tree to get all referenced files, and copy over to new Folder
        File[] files = getResourceFiles(manifestElement);
        for(int i = 0; i < files.length; i++) {
            try {
                String path = ReloadUtils.getRelativePath(getProjectFolder(), files[i]);
                File newFile = new File(targetFolder, path);
                newFile.getParentFile().mkdirs();
                ReloadUtils.copyFile(files[i], newFile);
            }
            catch(IOException ex) {
                throw new ReloadException(ex);
            }
        }

        // Create new imsmanifest.xml file from sub-Manifest node
        Document newDoc = new Document();

        // Add comments
        addComments(newDoc);

        // Clone the root
        Element newRoot = (Element)manifestElement.clone();
        newDoc.setRootElement(newRoot);

        // Save it
        File newFile = new File(targetFolder, ReloadDocumentHandler.MANIFEST_NAME);
        XMLUtils.write2XMLFile(newDoc, newFile);

        // Copy the Default Schema Files to the target Folder
        getReloadSchemaController().copySchemaFilesToFolder(targetFolder);

        return true;
    }

    /**
     * @return the Root element name
     */
    public String getRootElementName() {
        return MANIFEST;
    }

    /**
     * @return the project name - the project folder
     */
    public String getProjectName() {
        return getProjectFolder() == null ? null : getProjectFolder().getName();
    }

    /**
     * Get the CP Project Folder which is the containing folder of imsmanifest.xml
     * @return
     */
    public File getProjectFolder() {
        return _file == null ? null : _file.getParentFile();
    }


// ==========================================================================
// ELEMENT MOVING/COPYING/DRAGGING HANDLING
// ==========================================================================

    /**
     * @return whether we can copy an element
     */
    public boolean canCopyElement(Element element) {
        return false;
        //return canDragElement(element, ReloadDocumentClipboard.ACTION_COPY);
    }

    /**
     * @return whether we can cut an element
     */
    public boolean canCutElement(Element element, ReloadSchemaElement schemaElement) {
        return false;
        //return canCopyElement(element) && canDeleteElement(element, schemaElement);
    }

    /**
     * @return whether we can paste an element from the Clipboard to element
     */
    public boolean canPasteFromClipboard(Element element) {
        //Element clipboardElement = ReloadDocumentClipboard.getElement();

        //if(clipboardElement != null && element != null) {
            // Translate a Cut Action into a Move Action??
            //return isAllowedChild(element, clipboardElement);
        //}

        return false;
    }

    /**
     * @return whether we can drag an element around
     */
    public boolean canDragElement(Element element, int action) {
        if(element != null) {
            // Has to be our namespace
            if(!isDocumentNamespace(element)) return false;

            String name = element.getName();
            if(name.equals(RESOURCE) ||
               name.equals(METADATA) ||
               name.equals(ITEM) ||
               name.equals(ORGANIZATION) ||
               name.equals(MANIFEST)) return true;
        }
        return false;
    }

    /**
     * @return whether we can copy/move sourceElement to targetElement
     */
    public boolean acceptElement(Element sourceElement, Element targetElement, int actionCopyMove) {
        if(sourceElement != null && targetElement != null) {
            // Must be our Namespace
            if(!sourceElement.getNamespace().equals(targetElement.getNamespace())) return false;

            // Can't copy/move onto self
            if(sourceElement.equals(targetElement)) return false;

            // General Rules for Moving an Element
            if((actionCopyMove & ReloadDocumentClipboard.ACTION_MOVE) != 0) {
                // Can't move if already there
                if(targetElement.getContent().contains(sourceElement)) return false;
                // Can't move to a child
                if(targetElement.isAncestor(sourceElement)) return false;
            }

            // Get Target & Source Parent Manifests
            Element targetManifest = getParentManifestElement(targetElement);
            Element sourceManifest = getParentManifestElement(sourceElement);

            if(targetManifest == null || sourceManifest == null) return false;

            String sourceName = sourceElement.getName();
            String targetName = targetElement.getName();

            // Different Document?
            boolean differentDoc = !isMemberOfSameDocument(sourceElement, targetElement);
            if(differentDoc) {
                // Only Manifests
                //return sourceName.equals(MANIFEST) && targetName.equals(MANIFEST);
                return false;
            }

            // Source == RESOURCE
            if(sourceName.equals(RESOURCE)) {
                if(targetName.equals(ORGANIZATION) || targetName.equals(ITEM)) {
                    // Same manifest
                    if(targetManifest == sourceManifest) return true;
                    // Target must be higher than resource
                    return sourceManifest.isAncestor(targetManifest);
                }
                // Dragging into RESOURCES Element
                else if(targetName.equals(RESOURCES)) {
                    return !containsResource(sourceElement, targetElement);
                }
            }

            // Source == ITEM
            if(sourceName.equals(ITEM)) {
                if(targetName.equals(ORGANIZATION) || targetName.equals(ITEM)) {
                    // Same manifest
                    if(targetManifest == sourceManifest) return true;
                    // If there is an identifierref, the referenced object's (parent) manifest must be below or same as the target's Parent Manifest
                    Element refElement = getReferencedElement(sourceElement);
                    if(refElement != null) {
                        Element ref_targetManifest = getParentManifestElement(refElement);
                        return ref_targetManifest.isAncestor(targetManifest) || (ref_targetManifest == targetManifest);
                    }
                    else return true;
                }
            }

            // Source == MANIFEST
            else if(sourceName.equals(MANIFEST)) {
                if(targetName.equals(ORGANIZATION) || targetName.equals(ITEM)) {
                    // Target must be higher than manifest
                    return sourceManifest.isAncestor(targetManifest);
                }
                // Dragging into MANIFEST Element
                else if(targetName.equals(MANIFEST)) {
                    return false;
                }
            }

            // Default
            else return isAllowedChild(targetElement, sourceElement);

        }

        return false;
    }

    /**
     * Copy/Move sourceElement to targetElement
     * action is either ACTION_COPY or ACTION MOVE
     */
    public Element shiftElement(Element sourceElement, Element targetElement, int action) {
        if(sourceElement != null && targetElement != null) {

            // Get Target & Source Parent Manifests
            Element targetManifest = getParentManifestElement(targetElement);
            Element sourceManifest = getParentManifestElement(sourceElement);

            if(targetManifest == null || sourceManifest == null) return null;

            boolean isSameDocument = isMemberOfSameDocument(sourceElement, targetElement);

            // If Moving from another Manifest make sure we only copy it
            if(targetManifest != sourceManifest) action = ReloadDocumentClipboard.ACTION_COPY;

            String sourceName = sourceElement.getName();
            String targetName = targetElement.getName();

            // Copying/Moving a RESOURCE
            if(sourceName.equals(RESOURCE)) {
                if(targetName.equals(ORGANIZATION) || targetName.equals(ITEM)) {
                    // Add ITEM that references this RESOURCE
                    return addItemResource(targetElement, sourceElement, true);
                }
                else if(targetName.equals(RESOURCES)) {
                    return _shiftElement(sourceElement, targetElement, action, true);
                }
            }

            // Copying a MANIFEST
            else if(sourceName.equals(MANIFEST)) {
                if(targetName.equals(ORGANIZATION) || targetName.equals(ITEM)) {
                    // Add ITEM that references this MANIFEST
                    return addItemManifest(sourceElement, targetElement, true);
                }
                if(targetName.equals(MANIFEST)) {
                    return _shiftElement(sourceElement, targetElement, action, true);
                }
            }

            // Default
            else if(isAllowedChild(targetElement, sourceElement))
                return _shiftElement(sourceElement, targetElement, action, true);
        }

        return null;
    }

    protected Element _shiftElement(Element sourceElement, Element targetElement, int action, boolean doSelect) {
        // Copy
        if((action & ReloadDocumentClipboard.ACTION_COPY) != 0) {
            return copyElementUndoable(this, sourceElement, targetElement, doSelect);
        }
        // Move
        if((action & ReloadDocumentClipboard.ACTION_MOVE) != 0) {
            return moveElementUndoable(this, sourceElement, targetElement, doSelect);
        }
        return null;
    }


    /**
     * Over-ride this so we can intercept it to do stuff
     */
    public Element addElementBySchema(Object source, Element parentElement, ReloadSchemaElement newSchemaElement, boolean doSelect) {
        Element newElement = super.addElementBySchema(source, parentElement, newSchemaElement, doSelect);

        if(newElement != null) {
            // Here we add a unique identifier if it needs one
            if(newSchemaElement.hasAttribute(IDENTIFIER)) {
                String id = newElement.getAttributeValue(IDENTIFIER);
                if(id == null) {
                    ReloadSchemaAttribute schemaAtt = newSchemaElement.getAttribute(IDENTIFIER);
                    Attribute att = addAttribute(source, newElement, schemaAtt);
                    String uniqueID = generateUniqueID(newElement);
                    att.setValue(uniqueID);
                    fireElementChanged(new ReloadDocumentListenerEvent(source, this, newElement, false));
                }
            }

            String name = newElement.getName();

            if(name.equals(ORGANIZATION)) {
                // If user added first Organization, set it as default
                setDefaultOrganization(newElement);
                // Add default Title
                addTitle(newElement, "Organization");
            }

            else if(name.equals(ITEM)) {
                // Add default Title
                addTitle(newElement, "Item");
            }
        }

        return newElement;
    }

    /**
     * Over-ride this so we can intercept it to do stuff
     */
    public Element copyElementUndoable(Object source, Element element, Element newParent, boolean doSelect) {
        Element newElement = super.copyElementUndoable(source, element, newParent, doSelect);
        _copypasteElement(newElement);
        return newElement;
    }

    /**
     * Over-ride this so we can intercept it to do stuff
     */
    public Element pasteElementUndoable(Object source, Element element, Element newParent, boolean doSelect) {
        Element newElement = super.pasteElementUndoable(source, element, newParent, doSelect);
        _copypasteElement(newElement);
        return newElement;
    }

    /**
     * When an Element is copied over or pasted we need to do a bunch of stuff
     */
    protected void _copypasteElement(Element newElement) {
        if(newElement != null) {
           String name = newElement.getName();

           if(name.equals(ITEM) || name.equals(RESOURCE)) {
               // Create new IDs
               createNewIDs(newElement);
           }

           else if(name.equals(ORGANIZATION)) {
               // Create new IDs
               createNewIDs(newElement);
               // If user added first Organization, set it as default
               setDefaultOrganization(newElement);
               // Check valid IDENTIFIERREFs
               checkIDRefs(newElement);
           }

           else if(name.equals(MANIFEST)) {
               // Create new IDENTIFIERs and map to old IDENTIFIERs
               Hashtable map = createNewIDs(newElement);
               // Change each IDENTIFIERREF to new IDENTIFIERs
               createNewIDRefs(newElement, map);
           }
       }
    }

    /**
     * Add an Item to Item or Organization parent - this references a sub-Manifest
     */
    protected Element addItemManifest(Element manifestElement, Element targetElement, boolean doSelect) {
        // Add it
        DomPath domPath = DomPath.getDomPath(targetElement);
        domPath.appendElementName(ITEM);
        ReloadSchemaElement schemaElement = (ReloadSchemaElement)getReloadSchemaController().getReloadSchemaNode(domPath);
        Element itemElement = addElementBySchemaUndoable(this, targetElement, schemaElement, doSelect);

        // Set IDENTIFIERREF to MANIFEST's IDENTIFIER if specified
        if(itemElement != null) {
            String manifestID = manifestElement.getAttributeValue(IDENTIFIER);
            if(manifestID != null) itemElement.setAttribute(IDENTIFIERREF, manifestID);
        }

        return itemElement;
    }

    /**
     * Add an Item to Item or Organization parent - this references a Resource
     * If resourcesElement is not null set the Item's identifierref to it
     */
    protected Element addItemResource(Element parentElement, Element resourceElement, boolean doSelect) {
        // Add it
        DomPath domPath = DomPath.getDomPath(parentElement);
        domPath.appendElementName(ITEM);
        ReloadSchemaElement schemaElement = (ReloadSchemaElement)getReloadSchemaController().getReloadSchemaNode(domPath);
        Element itemElement = addElementBySchemaUndoable(this, parentElement, schemaElement, doSelect);

        if(itemElement != null && resourceElement != null) {
            // Set IDENTIFIERREF to resource's IDENTIFIER if specified
            String resourceID = resourceElement.getAttributeValue(IDENTIFIER);
            if(resourceID != null) itemElement.setAttribute(IDENTIFIERREF, resourceID);
            // Add Title
            addItemTitle(itemElement, resourceElement);
        }

        return itemElement;
    }

    /**
     * Add an Organization
     */
    protected Element addOrganization(Element parentElement, boolean doSelect) {
        DomPath domPath = DomPath.getDomPath(parentElement);
        domPath.appendElementName(ORGANIZATION);
        ReloadSchemaElement schemaElement = (ReloadSchemaElement)getReloadSchemaController().getReloadSchemaNode(domPath);
        return addElementBySchemaUndoable(this, parentElement, schemaElement, doSelect);
    }

    /**
     * Add a Title element (only one allowed).
     * If a Title Element already exists the text will be replaced by new text
     */
    protected Element addTitle(Element parentElement, String text) {
        Element titleElement = parentElement.getChild(TITLE, parentElement.getNamespace());

        if(titleElement != null) {
            titleElement.setText(text);
        }

        else {
            titleElement = new Element(TITLE, parentElement.getNamespace());
            titleElement.setText(text);
            addElement(this, parentElement, titleElement, false);
        }

        return titleElement;
    }

    /**
     * Add a Title to an Item based on the Resource it References.
     * If it's an html file use the <title> element from that or the file name
     */
    protected Element addItemTitle(Element itemElement, Element resourceElement) {
        Element titleElement = null;

        if(itemElement != null && resourceElement != null) {
            String title = null;
            File file = getResourceFile(resourceElement);
            if(file != null) {
                String fileName = file.getName().toLowerCase();
                if(fileName.endsWith(".html") || fileName.endsWith(".htm")) {
                    HTMLParser parser = new HTMLParser(file);
                    title = parser.getTagText("title");
                }
                else title = ReloadUtils.getFileNameWithoutExtension(file);
            }
            titleElement = addTitle(itemElement, title);
        }

        return titleElement;
    }


// =============================================================================
// ELEMENT FINDING/NAVIGATION HANDLING
// =============================================================================

    /**
     * @return The Element referenced by identifierref in sourceElement.
     * The search digs downward from sourceElement - this is important.
     * This will be either a Resource or a sub-Manifest.
     */
    public Element getReferencedElement(Element sourceElement) {
        String idref = sourceElement.getAttributeValue(IDENTIFIERREF);

        if(idref != null) {
            Element manifest = getParentManifestElement(sourceElement);
            if(manifest != null) {
                // Search the Resources first
                Element[] resources = getElementsInManifest(manifest, RESOURCE, getSchemaNamespace());
                for(int i = 0; i < resources.length; i++) {
                    String id = resources[i].getAttributeValue(IDENTIFIER);
                    if(id != null) {
                        if(idref.equals(id)) return resources[i];
                    }
                }

                // Search for a sub-Manifest
                Element[] submanifests = getElementsInManifest(manifest, MANIFEST, getSchemaNamespace());
                for(int i = 0; i < submanifests.length; i++) {
                    String id = submanifests[i].getAttributeValue(IDENTIFIER);
                    if(id != null && idref.equals(id)) return submanifests[i];
                }
            }
        }

        return null;
    }

    /**
     * @return true if this element has an identifierref attribute
     */
    public boolean isReferencingElement(Element element) {
        String idref = element.getAttributeValue(IDENTIFIERREF);
        return idref != null && !idref.equals("");
    }

    /**
     * @return all available Identifiers from Resources and sub-Manifests
     * that an element can legally reference.  For an Item this will be Resources and sub-manifests
     * and for a Resource Dependency this will be another Resource
     */
    public String[] getReferencedIdentifersAllowed(Element element) {
        Vector v = new Vector();

        Element[] elements = getReferencedElementsAllowed(element);
        for(int i = 0; i < elements.length; i++) {
            String id = elements[i].getAttributeValue(IDENTIFIER);
            if(id != null && !id.equals("")) v.add(id);
        }

        String[] ids = new String[v.size()];
        v.copyInto(ids);
        return ids;
    }


    /**
     * @return all available Elements (Resources and sub-Manifests)
     * that an element can legally reference.  For an Item this will be Resources and sub-manifests
     * and for a Resource Dependency this will be another Resource
     */
    public Element[] getReferencedElementsAllowed(Element element) {
        Vector v = new Vector();

        Element manifest = getParentManifestElement(element);
        if(manifest != null) {
            String elementName = element.getName();

            // Search the Resources first
            Element[] resources = getElementsInManifest(manifest, RESOURCE, getSchemaNamespace());
            for(int i = 0; i < resources.length; i++) {
                String id = resources[i].getAttributeValue(IDENTIFIER);
                if(id != null && !id.equals("")) v.add(resources[i]);
            }

            // Search for sub-Manifests (if an Item)
            if(elementName.equals(ITEM)) {
                Element[] submanifests = getElementsInManifest(manifest, MANIFEST, getSchemaNamespace());
                for(int i = 0; i < submanifests.length; i++) {
                    String id = submanifests[i].getAttributeValue(IDENTIFIER);
                    if(id != null && !id.equals("")) v.add(submanifests[i]);
                }
            }
        }

        Element[] elements = new Element[v.size()];
        v.copyInto(elements);
        return elements;
    }

    /**
     * @return all ORGANIZATION elements in an ORGANIZATIONS element
     */
    public Element[] getOrganizations(Element orgsElement) {
        Vector v = new Vector();

        Iterator it = orgsElement.getChildren(ORGANIZATION, orgsElement.getNamespace()).iterator();
        while(it.hasNext()) {
            Element orgElement = (Element)it.next();
            v.add(orgElement);
        }

        Element[] orgs = new Element[v.size()];
        v.copyInto(orgs);
        return orgs;
    }

    /**
     * @return all ORGANIZATION elements in an ORGANIZATIONS element that can be referenced
     */
    public Element[] getOrganizationsAllowed(Element orgsElement) {
        Vector v = new Vector();

        Element[] orgs = getOrganizations(orgsElement);
        for(int i = 0; i < orgs.length; i++) {
            String id = orgs[i].getAttributeValue(IDENTIFIER);
            if(id != null && !id.equals("")) v.add(orgs[i]);
        }

        Element[] new_orgs = new Element[v.size()];
        v.copyInto(new_orgs);
        return new_orgs;
    }

    /**
     * @return the default Organization for an ORGANIZATIONS element
     * If there is a DEFAULT ref in the ORGANIZATIONS element will return that
     * otherwise retunr first ORGANIZATION Element if found
     */
    public Element getDefaultOrganization(Element orgsElement) {
        Element element = null;

        if(orgsElement != null && orgsElement.getName().equals(ORGANIZATIONS)) {
            // Get ORGANIZATION by default attribute ref
            String defOrg = orgsElement.getAttributeValue(DEFAULT);
            if(defOrg != null) element = getElementByIdentifier(orgsElement, defOrg);
            // Not found so get first ORGANIZATION Element
            if(element == null) element = orgsElement.getChild(ORGANIZATION, orgsElement.getNamespace());
        }

        return element;
    }


    /**
     * @return a nice name to display for an Element depending on element type
     */
    public String getElementDisplayName(Element element) {
        String fname = null;

        if(element != null) {
            String name = element.getName();

            // ITEM & ORGANIZATION - display Title Element
            if(name.equals(ITEM) || name.equals(ORGANIZATION)) {
                Element titleElement = element.getChild(TITLE, element.getNamespace());
                if(titleElement != null) fname = titleElement.getText();
            }

            // DEPENDENCY - display path of referenced Resource
            else if(name.equals(DEPENDENCY)) {
                fname = getElementHREF(element);
            }

            // MANIFEST - display path of referenced Resource
            else if(name.equals(MANIFEST)) {
                fname = element.getAttributeValue(IDENTIFIER);
            }

            // FILE & RESOURCE - display href
            else if(name.equals(FILE) || name.equals(RESOURCE)) {
                fname = element.getAttributeValue(HREF);
            }

            // MD SCHEMA & SCHEMAVERSION
            else if(name.equals(SCHEMA) || name.equals(SCHEMAVERSION)) {
                fname = element.getText();
            }

            // Get Friendly name if possible, or qualified name if not
            if(fname == null || fname.equals("")) {
                fname = getReloadSchemaController().getElementFriendlyName(DomPath.getDomPath(element));
                if(fname == null) fname = element.getQualifiedName();
            }
        }

        return fname;
    }


    /**
     * Find the local parent "manifest" Element for a given Element
     * If element is the manifest, will return that
     */
    public Element getParentManifestElement(Element element) {
        while(!element.getName().equals(MANIFEST)) {
            element = element.getParent();
            if(element == null) return null;
        }
        return element;
    }

    /**
     * @return all sub-Elements of type elementName from a given parent Manifest
     * This does a deep recursive search
     */
    public Element[] getElementsInManifest(Element manifestElement, String elementName, Namespace ns) {
        Vector v = new Vector();
        _getElementsInManifest(manifestElement, v, elementName, ns);
        Element[] elements = new Element[v.size()];
        v.copyInto(elements);
        return elements;
    }

    protected void _getElementsInManifest(Element parent, Vector v, String elementName, Namespace ns) {
        Iterator it = parent.getChildren().iterator();
        while(it.hasNext()) {
            Element child = (Element)it.next();
            if(child.getName().equals(elementName) && child.getNamespace().equals(ns)) {
                v.add(child);
            }
            _getElementsInManifest(child, v, elementName, ns);
        }
    }

    /**
     * @return an Element by its IDENTIFIER attribute starting at Parent element
     * This will only search one-level deep from parent
     */
    public Element getElementByIdentifier(Element parent, String identifier) {
        Iterator it = parent.getChildren().iterator();
        while(it.hasNext()) {
            Element child = (Element)it.next();
            if(isDocumentNamespace(child)) {
                String id = child.getAttributeValue(IDENTIFIER);
                if(id != null) {
                    if(id.equals(identifier)) return child;
                }
            }
        }

        return null;
    }


    /**
     * Get the MD Namespace for a CP Document or null if none declared
     * @return
     */
    public Namespace getMetadataNamespace() {
        Namespace ns = null;

        if(_doc.hasRootElement()) {
            // Do we have one set in the root element?
            Element root = _doc.getRootElement();

            // Find it according to prefix
            ns = root.getNamespace(ReloadDocumentHandler.IMSMD_NAMESPACE_PREFIX);

            // No luck, so iterate thru Namespaces and search for one
            if(ns == null) {
                Iterator it = root.getAdditionalNamespaces().iterator();
                while(it.hasNext()) {
                    Namespace nsTemp = (Namespace) it.next();
                    String nsURI = nsTemp.getURI();
                    if(MetadataSchemaController.isAcceptedNamespaceURI(nsURI)) return nsTemp;
                }
            }
        }

        return ns;
    }

    /**
     * Return whether we have the MD root name of any embedded Metadata "lom"
     */
    public boolean isMetadataRoot(String name) {
        return name.equals(Metadata.ROOT_NAME);
    }

    /**
     * Return whether we have the "metadata" element
     */
    public boolean isMetadataElement(String name) {
        return name.equals(METADATA);
    }

    /**
     * @return The URL string that an Element references
     */
    public String getURL(Element element) {
        String href = getElementHREF(element);
        if(href != null) {
            // Is it a local file for local people?
            try {
                File file = new File(getProjectFolder(), href);
                if(file.exists()) {
                    return ReloadUtils.getLocalFileURL(file);
                }
                else {
                    URL url = new URL(href);
                    return url.toString();
                }
            }
            catch(MalformedURLException ex) {
                try {
                    URL url = new URL("http://" + href);
                    return url.toString();
                }
                catch(MalformedURLException ex1) {
                    return null;
                }
            }
        }

        return null;
    }

    /**
     * @return An actual File reference that an Element references.
     * If it's not a local file or does not exist, return null
     */
    public File getResourceFile(Element element) {
        String href = getElementHREF(element);
        if(href != null) {
            File file = new File(getProjectFolder(), href);
            if(file.exists()) return file;
        }
        return null;
    }


    /**
     * @return All File HREFs for a given Manifest Element taking into account xml:base
     */
    public File[] getResourceFiles(Element manifestElement) {
        Vector v = new Vector();
        _getResourceFiles(manifestElement, v);
        File[] files = new File[v.size()];
        v.copyInto(files);
        return files;
    }

    protected void _getResourceFiles(Element manifestElement, Vector v) {
        // Resources Element
        Element resourcesElement = manifestElement.getChild(RESOURCES, manifestElement.getNamespace());
        if(resourcesElement != null) {
            // Resource Elements
            Iterator it = resourcesElement.getChildren(RESOURCE, resourcesElement.getNamespace()).iterator();
            while(it.hasNext()) {
                Element resourceElement = (Element)it.next();
                File file = getResourceFile(resourceElement);
                if(file != null && !v.contains(file)) v.add(file);
                // File Elements
                Iterator it2 = resourceElement.getChildren(FILE, resourceElement.getNamespace()).iterator();
                while(it2.hasNext()) {
                    Element fileElement = (Element)it2.next();
                    File file2 = getResourceFile(fileElement);
                    if(file2 != null && !v.contains(file2)) v.add(file2);
                }
            }
        }

        // Recurse Sub-Manifests
        Element submanifestElement = manifestElement.getChild(MANIFEST, manifestElement.getNamespace());
        if(submanifestElement != null) {
            _getResourceFiles(submanifestElement, v);
        }
    }


    /**
     * @return An actual HREF reference from an Element's HREF taking into account xml:base
     * and parameters if it's an Item
     */
    public String getElementHREF(Element element) {
        String params = null;
        String name = element.getName();

        // If this is an ITEM OR DEPENDENCY, change element to referenced element
        if(name.equals(ContentPackage.ITEM) || name.equals(ContentPackage.DEPENDENCY)) {
            // Save any parameters
            params = getParameters(element);
            // Get referenced Element
            element = getReferencedElement(element);
            if(element == null) return null;
        }

        // Get the HREF
        String href = element.getAttributeValue(ContentPackage.HREF);
        if(href != null) {
            // Do we have a "base" attribute?
            String base = getElementBase(element);
            if(base != null) href = base + href;
            // Parameters
            if(params != null) href = href + params;
        }

        return href;
    }

    /**
     * @return An actual BASE reference for an Element taking into account xml:base
     */
    public String getElementBase(Element element) {
        String totalBase = "";
        do {
            String base = element.getAttributeValue(BASE, Namespace.XML_NAMESPACE);
            if(base != null) {
                totalBase = base + totalBase;
                if(base.startsWith("/") || base.startsWith("http:")) break; // Stop at absolute URL
            }
            //if(element.getName().equals(MANIFEST)) break; // Stop at first parent Manifest
            element = element.getParent();
        }
        while(element != null);

        return totalBase.equals("") ? null : totalBase;
    }

    /**
     * @return the parameters attribute of an element
     */
    public String getParameters(Element element) {
        String params = element.getAttributeValue(PARAMETERS);

        if(params != null && !params.equals("")) {
            char c = params.charAt(0);
            if((c != '?') && (c != '#')) params = "?" + params;
        }

        return params;
    }


//==============================================================================
// UTILS
//==============================================================================

    /**
     * Create new IDENTIFIERREFs and DEFAULT attributes
     */
    protected void createNewIDRefs(Element element, Hashtable id_map) {
        if(element != null && isDocumentNamespace(element)) {
            String name = element.getName();

            String attrib = null;
            if(name.equals(ITEM)) attrib = IDENTIFIERREF;
            else if(name.equals(ORGANIZATIONS)) attrib = DEFAULT;

            if(attrib != null) {
                String idref = element.getAttributeValue(attrib);
                if(idref != null && !idref.equals("")) {
                    Element ref_element = (Element) id_map.get(idref);
                    if(ref_element != null) {
                        String newID = ref_element.getAttributeValue(IDENTIFIER);
                        if(newID != null) {
                            element.setAttribute(attrib, newID);
                        }
                    }
                }
            }

            // Children
            Iterator it = element.getChildren().iterator();
            while(it.hasNext()) {
                Element child = (Element) it.next();
                createNewIDRefs(child, id_map);
            }
        }
    }

    /**
     * Create new SCORM Prerequisite Refs
     */
    protected void createNewPrerequisiteRefs(Element element, Hashtable id_map) {
        if(element != null && isDocumentNamespace(element)) {
            // SCORM <prerequisite> elements
            Element prereq = element.getChild(SCORM1_2Package.PREREQUISITES, ReloadDocumentHandler.ADLCP_NAMESPACE_12);
            if(prereq != null) {
                String script = prereq.getText();
                if(script != null && !script.equals("")) {
                    // Parse script String for this
                    Enumeration e = id_map.keys();
                    while(e.hasMoreElements()) {
                        String oldID = (String)e.nextElement();
                        int index = script.indexOf(oldID);
                        if(index != -1) {
                            Element ref_element = (Element) id_map.get(oldID);
                            if(ref_element != null) {
                                String newID = ref_element.getAttributeValue(IDENTIFIER);
                                if(newID != null) script = script.replaceAll(oldID, newID);
                            }
                        }
                    }
                    prereq.setText(script);
                }
            }

            // Children
            Iterator it = element.getChildren().iterator();
            while(it.hasNext()) {
                Element child = (Element) it.next();
                createNewPrerequisiteRefs(child, id_map);
            }
        }
    }

    /**
     * Create a new ID for an Element that has an IDENTIFIER attribute and also for its children
     * Only works if the Element already has an IDENTIFIER attribute
     * @return a Hashtable of old IDs mapped to Elements
     */
    protected Hashtable createNewIDs(Element element) {
       Hashtable idmap = new Hashtable();
       _createNewIDs(element, idmap);
       return idmap;
    }

    protected void _createNewIDs(Element element, Hashtable idmap) {
        if(element != null && isDocumentNamespace(element)) {
            String oldID = element.getAttributeValue(IDENTIFIER);
            if(oldID != null) {
                String newID = generateUniqueID(element);
                idmap.put(oldID, element);
                element.setAttribute(IDENTIFIER, newID);
            }
            Iterator it = element.getChildren().iterator();
            while(it.hasNext()) {
                Element child = (Element) it.next();
                _createNewIDs(child, idmap);
            }
        }
    }

    /**
     * If we move or copy an Element that contains an IDENTIFIERREF to another Element
     * we check that the Element referenced is still below us. If it is then OK, if not the
     * reference is invalid so we remove it - also for its children
     */
    protected void checkIDRefs(Element element) {
        if(element != null && isDocumentNamespace(element)) {
            Element ref_element = getReferencedElement(element);
            // Null means it's not found and therefore below us
            if(ref_element == null) {
                Attribute att = element.getAttribute(IDENTIFIERREF);
                if(att != null) element.removeAttribute(att);
            }
            Iterator it = element.getChildren().iterator();
            while(it.hasNext()) {
                Element child = (Element) it.next();
                checkIDRefs(child);
            }
        }
    }

    /**
     * Generate a Unique Identifier for an Element
     */
    protected String generateUniqueID(Element element){
        String prefix;
        String elementName = element.getName();
        if(elementName.equals(MANIFEST)) prefix = "MANIFEST-";
        else if(elementName.equals(ORGANIZATION)) prefix = "ORG-";
        else if(elementName.equals(ITEM)) prefix = "ITEM-";
        else if(elementName.equals(RESOURCE)) prefix = "RES-";
        else prefix = "RLD-";
        return RandomGUID.getUniqueID(prefix);
    }

    /**
     * When an Organization element is added/copied to an Organizations Element
     * and it is the only one, set the Organizations "default" attribute to it
     */
    protected void setDefaultOrganization(Element organization) {
        Element orgs = organization.getParent();
        if(orgs.getChildren(ORGANIZATION, orgs.getNamespace()).size() == 1) {
            String id = organization.getAttributeValue(IDENTIFIER);
            if(id != null) orgs.setAttribute(DEFAULT, id);
        }
    }



// =============================================================================
// RESOURCE HANDLING
// =============================================================================

    /**
     * Add ReloadResource(s) to the Manifest
     */
    public Element addReloadResource(ReloadResource reloadResource, Element parentElement) {
        // We are adding a Folder of Resources
        if(reloadResource.isDirectory()) {

            String name = parentElement.getName();

            // A folder added to ORGANIZATIONS Element so create new Organization
            if(name.equals(ORGANIZATIONS)) {
                // Add new Organization
                parentElement = addOrganization(parentElement, false);
                // Add Title Element - the name of the folder - only here
                addTitle(parentElement, reloadResource.getName());
            }

            // A folder added to ORGANIZATION or ITEM
            else if(name.equals(ORGANIZATION) || name.equals(ITEM)) {
                // Add new empty Item
                parentElement = addItemResource(parentElement, null, false);
                // Add Title Element - the name of the folder - only here
                addTitle(parentElement, reloadResource.getName());
            }

            // Add all resources to Resources Element
            ReloadResource[] resources = reloadResource.getChildren();
            if(resources != null) {
                for(int i = 0; i < resources.length; i++) {
                    // Recurse
                    addReloadResource(resources[i], parentElement);
                }
            }

            return parentElement;
        }

        // A Single Resource
        else return _addReloadResource(reloadResource, parentElement, false);
    }

    /**
     * Add a ReloadResource to a given Element
     */
    protected Element _addReloadResource(ReloadResource reloadResource, Element parentElement, boolean doSelect) {
        // Do we have the Resource?
        Element resourcesElement = getResourcesElement(parentElement);
        Element resourceElement = getResourceByReloadResouce(reloadResource, resourcesElement);
        // Nope - so add to local Resources Element
        if(resourceElement == null) {
            resourceElement = _addReloadResourceToResourcesElement(reloadResource, resourcesElement, doSelect);
        }

        String name = parentElement.getName();

        // We have added it to Organization or Item Element
        if(name.equals(ORGANIZATION) || name.equals(ITEM)) {
            // Add Item that references the Resource
            return addItemResource(parentElement, resourceElement, doSelect);
        }

        return resourceElement;
    }

    /**
     * Add a Resource to the "resources" element
     */
    protected Element _addReloadResourceToResourcesElement(ReloadResource reloadResource, Element resourcesElement, boolean doSelect) {
        DomPath domPath = DomPath.getDomPath(resourcesElement);
        domPath.appendElementName(RESOURCE);
        ReloadSchemaElement schemaElement = (ReloadSchemaElement)getReloadSchemaController().getReloadSchemaNode(domPath);

        Element newElement = addElementBySchema(this, resourcesElement, schemaElement, doSelect);

        if(newElement != null) {
            // HREF
            String href = ReloadUtils.getRelativePath(getProjectFolder(), reloadResource);
            newElement.setAttribute(HREF, href);

            // Add sub-Files
            addFileElements(reloadResource, newElement, schemaElement, doSelect);

            // Undo Event
            if(_undoHandler != null) {
                UndoableAddAction addAction = new UndoableAddAction(resourcesElement, newElement);
                _undoHandler.addUndoableAction(addAction);
            }

        }

        return newElement;
    }

    /**
     * Find out whether this CP contains a ReloadResource
     * @param reloadResource
     * @return
     */
    protected boolean containsReloadResource(ReloadResource reloadResource, Element resourcesElement) {
        // Get a Relative Path
        Element element = getResourceByReloadResouce(reloadResource, resourcesElement);
        return element != null;
    }

    /**
     * Get a Resource in reference to a ReloadResource
     * @param reloadResource
     * @return
     */
    protected Element getResourceByReloadResouce(ReloadResource reloadResource, Element resourcesElement) {
        // Get a Relative Path
        String href = ReloadUtils.getRelativePath(getProjectFolder(), reloadResource);
        return getResource(href, resourcesElement);
    }

    /**
     * Return whether resourcesElement already contains sourceElement as a Resource
     * This searches by the HREF attribute
     */
    public boolean containsResource(Element sourceElement, Element resourcesElement) {
        if(sourceElement != null && resourcesElement != null) {
            if(sourceElement.getName().equals(RESOURCE) && resourcesElement.getName().equals(RESOURCES)) {
                String href = sourceElement.getAttributeValue(HREF);
                //String href = getElementHREF(sourceElement);
                if(href != null) return containsResource(href, resourcesElement);
            }
        }
        return false;
    }

    /**
     * Return whether a Resources Element contains a Resource by HREF
     */
    protected boolean containsResource(String href, Element resourcesElement) {
        Element element = getResource(href, resourcesElement);
        return element != null;
    }

    /**
     * Return a Resource Element given its HREF
     */
    protected Element getResource(String href, Element resourcesElement) {
        Iterator it = resourcesElement.getChildren(RESOURCE, resourcesElement.getNamespace()).iterator();
        while(it.hasNext()) {
            Element resource = (Element) it.next();
            String tmp = resource.getAttributeValue(HREF);
            if(tmp != null) {
                if(tmp.equalsIgnoreCase(href)) return resource;
            }
        }
        return null;
    }

    /**
     * Find the local "resources" Element from a given Element position.  This is because there may
     * be more than one with sub-manifests
     */
    public Element getResourcesElement(Element element) {
        if(element.getName().equals(RESOURCES)) return element;
        Element manifest = getParentManifestElement(element);
        return manifest == null ? null : manifest.getChild(RESOURCES, element.getNamespace());
    }

    /**
     * Return whether targetElement can accept a ReloadResource
     */
    public boolean acceptReloadResource(ReloadResource reloadResource, Element targetElement) {
        // Correct Namespace?
        if(!targetElement.getNamespace().equals(getTargetNamespace())) return false;

        String name = targetElement.getName();
        // In Resources Folder
        if(name.equals(RESOURCES)) {
            // A Folder
            if(reloadResource.isDirectory()) return true;
            // A Single File - true if we don't already have it in the Resources Element
            else {
                Element resourcesElement = getResourcesElement(targetElement);
                return !containsReloadResource(reloadResource, resourcesElement);
            }
        }
        // In Organization and Item Folder
        else if(name.equals(ORGANIZATION) || name.equals(ITEM)) {
            return true;
        }
        // In Organizations Folder only if Resource is a Folder
        else if(name.equals(ORGANIZATIONS)) {
            return reloadResource.isDirectory();
        }

        return false;
    }

    /**
     * Add File dependencies for a ReloadResource
     */
    protected void addFileElements(ReloadResource reloadResource, Element parentElement, ReloadSchemaElement parentSchemaElement, boolean doSelect) {
        ReloadSchemaElement fileSchemaElement = parentSchemaElement.getChild(FILE);

        // Add <file> element for the Resource
        Element fileElement = addElementBySchema(this, parentElement, fileSchemaElement, doSelect);
        if(fileElement != null) {
            String href = ReloadUtils.getRelativePath(getProjectFolder(), reloadResource);
            fileElement.setAttribute(HREF, href);
        }

        // Add the other <file> elements if this is a Parseable resource file
        if(ReloadFileType.isParseableFile(reloadResource)) {
            File[] files = reloadResource.getDependencies(getProjectFolder());
            for(int i = 0; i < files.length; i++) {
                fileElement = addElementBySchema(this, parentElement, fileSchemaElement, doSelect);
                String href = ReloadUtils.getRelativePath(getProjectFolder(), files[i]);
                fileElement.setAttribute(HREF, href);
            }
        }
    }

}
