/**
  *  RELOAD Editor - An IMS Metadata, Content Packaging and Learning Design tool.
  *
  *  Copyright (c) 2003 Oleg Liber, Bill Olivier, Phillip Beauvoir
  *
  *  Permission is hereby granted, free of charge, to any person obtaining a copy
  *  of this software and associated documentation files (the "Software"), to deal
  *  in the Software without restriction, including without limitation the rights
  *  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
  *  copies of the Software, and to permit persons to whom the Software is
  *  furnished to do so, subject to the following conditions:
  *
  *  The above copyright notice and this permission notice shall be included in
  *  all copies or substantial portions of the Software.
  *
  *  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  *  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  *  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
  *  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
  *  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
  *  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
  *
  *  Project Management Contact:
  *
  *  Oleg Liber
  *  Bolton Institute of Higher Education
  *  Deane Road
  *  Bolton BL3 5AB
  *  UK
  *
  *  e-mail:   o.liber@bolton.ac.uk
  *
  *
  *  Technical Contact:
  *
  *  Phillip Beauvoir
  *  e-mail:   p.beauvoir@bolton.ac.uk
  *
  *  Web:      http://www.reload.ac.uk
  *
  */

package uk.ac.reload.contentpackaging.resourceview;

import java.util.*;

import java.awt.*;
import java.awt.dnd.*;
import java.awt.image.*;
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.tree.*;

import uk.ac.reload.contentpackaging.*;
import uk.ac.reload.widgets.tree.*;
import uk.ac.reload.util.*;

/**
 * A Content Package Resources Tree for the CP UI
 *
 * @author Phillip Beauvoir
 * @author Paul Sharples
 * @version 2003-07-01
 */
public class CPResourcesTree
extends ReloadDragTree
implements TreeSelectionListener
{
    /**
     * The Tree Model
     */
    protected CPResourcesTreeModel _treeModel;

    /**
     * Our parent Panel with Menu Actions on
     */
    protected CPResourcesPanel _resourcesPanel;

    /**
     * Constructor
     */
    public CPResourcesTree(CPResourcesPanel resourcesPanel) {
        _resourcesPanel = resourcesPanel;

        _treeModel = new CPResourcesTreeModel();
        setModel(_treeModel);

        // Tree Selection Listener
        addTreeSelectionListener(this);

        setCellRenderer(new CPResourcesTreeRenderer());

        // Tooltips
        ToolTipManager.sharedInstance().registerComponent(this);

        DragSource dragSource = DragSource.getDefaultDragSource();
        dragSource.createDefaultDragGestureRecognizer(this, DnDConstants.ACTION_COPY_OR_MOVE, this);
    }

    /**
     * Set a new CP Project to be displayed
     */
    public void setContentPackage(ContentPackage cp) {
        // Tell Model
        _treeModel.setContentPackage(cp);

        // Select Root Node
        selectNode(_treeModel.getRootNode());
    }

    /**
     * Accessor Method to return which nodes the user has selected
     * @return a vector containing the nodes as ReloadResources
     */
    public ReloadResource[] getSelectedResources(){
        Vector v = new Vector();
        TreePath[] paths = getSelectionPaths();
        for (int i = 0; i < paths.length; i++) {
             CPResourcesTreeNode fileSysEntity =
                 (CPResourcesTreeNode)paths[i].getLastPathComponent();
             // Need to make sure this does not pass the root folder back!
             if (!fileSysEntity.isRoot()){
                 ReloadResource aFileToDelete = (ReloadResource) fileSysEntity.getUserObject();
                 v.add(aFileToDelete);
             }
        }
        ReloadResource[] resources = new ReloadResource[v.size()];
        v.copyInto(resources);
        return resources;
    }

    /**
     * Remove Resources From Tree
     * @param object
     */
    public void removeTreeResource(Object object) {
        _treeModel.removeNodeFromParent(getNode(object));
    }

    /**
     * A node has been selected on the tree.
     * @param event The TreeSelectionEvent fired from us being a TreeSelectionListener
     */
    public void valueChanged(TreeSelectionEvent event) {
        // Update any Menus
        updateMenus(event.getPath());
    }

    /**
     * Update the menus according to the selected node on the tree.
     * This has to be synchronized so different threads don't access it at the same time
     */
    protected synchronized void updateMenus(TreePath selPath) {
        if(selPath == null) return;

        Object selectedNode = selPath.getLastPathComponent();
        if(selectedNode instanceof CPResourcesTreeNode) {
            // Cast it
            CPResourcesTreeNode treeNode = (CPResourcesTreeNode) selectedNode;
            // Get Reload Resource
            ReloadResource resource = treeNode.getReloadResource();
            // Tell Button Panel
            _resourcesPanel.updateMenus(resource);
        }
    }


    // =================== Drag and Drop Source events =========================

    /**
     * A drag gesture has been initiated from this tree
     * <h3>Amendment History</h3>
     * <p>Basic outline code for handling a ghosted drag image.  This works
     * on the Mac, but not on PC.  Without this code on the Mac, a drag event
     * shows the whole JPanel as a ghosted image.<p/>
     */
    public void dragGestureRecognized(DragGestureEvent event) {
        if((event.getDragAction() & DnDConstants.ACTION_COPY_OR_MOVE) == 0) return;

        CPResourcesTreeNode node = (CPResourcesTreeNode)getDragSourceTreeNode(event);
        if(node != null) {
            ReloadResource resource = node.getReloadResource();
            DragTreeObject dragObject = new DragTreeObject(resource, this);

            Point ptDragOrigin = event.getDragOrigin();
            TreePath path = getPathForLocation(ptDragOrigin.x, ptDragOrigin.y);
            if(path == null) return;

            //if(isRootPath(path)) return;	// Ignore user trying to drag the root node

            // Get the Drag Image
            BufferedImage imgGhost = getDragImage(path, ptDragOrigin);

            event.startDrag(DragSource.DefaultCopyNoDrop, imgGhost, new Point(5, 5), dragObject, this);
        }
    }

    /**
     * We do not accept drops
     */
    public boolean isDropOK(DropTargetDragEvent event) {
        return false;
    }

}