/**
 *  RELOAD Editor - An IMS Metadata, Content Packaging and Learning Design tool.
 *
 *  Copyright (c) 2003 Oleg Liber, Bill Olivier, Phillip Beauvoir
 *
 *  Permission is hereby granted, free of charge, to any person obtaining a copy
 *  of this software and associated documentation files (the "Software"), to deal
 *  in the Software without restriction, including without limitation the rights
 *  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 *  copies of the Software, and to permit persons to whom the Software is
 *  furnished to do so, subject to the following conditions:
 *
 *  The above copyright notice and this permission notice shall be included in
 *  all copies or substantial portions of the Software.
 *
 *  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 *  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 *  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 *  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 *  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 *  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 *
 *  Project Management Contact:
 *
 *  Oleg Liber
 *  Bolton Institute of Higher Education
 *  Deane Road
 *  Bolton BL3 5AB
 *  UK
 *
 *  e-mail:   o.liber@bolton.ac.uk
 *
 *
 *  Technical Contact:
 *
 *  Phillip Beauvoir
 *  e-mail:   p.beauvoir@bolton.ac.uk
 *
 *  Web:      http://www.reload.ac.uk
 *
 */

package uk.ac.reload.dom;

import java.io.*;
import java.util.*;
import javax.swing.JOptionPane;

import org.jdom.*;
import uk.ac.reload.*;
import uk.ac.reload.schema.*;
import uk.ac.reload.undo.*;
import uk.ac.reload.util.*;

import uk.ac.reload.metadata.*;
import uk.ac.reload.metadata.formview.*;
import uk.ac.reload.metadata.tableview.*;
import uk.ac.reload.widgets.*;
import uk.ac.reload.vocabulary.*;

/**
 * This encapsulates the JDOM XML Document file and the ReloadSchema associated
 * with it plus helper methods.
 *
 * @author Phillip Beauvoir
 * @version 2003-07-28
 */
public abstract class ReloadDocument {
    /**
     * The JDOM Document of the XML File
     */
    protected Document _doc;

    /**
     * The Schema that will be our template
     */
    protected ReloadSchemaController _controller;

    /**
     * A flag to set if this Document is dirty
     */
    protected boolean dirty;

    /**
     * The File of this Document - this will be null if the Document is not standalone
     */
    protected File _file;

    /**
     * The Undo Handler
     */
    protected ReloadUndoHandler _undoHandler;


    /**
     * Default Constructor
     */
    protected ReloadDocument() {

    }

    /**
     * Constructor
     */
    protected ReloadDocument(File file, Document doc, ReloadSchemaController schemaController) {
        setDocument(file, doc, schemaController);
    }

    /**
     * Set the ReloadSchemaController
     * @param controller
     */
    public void setReloadSchemaController(ReloadSchemaController controller) {
        _controller = controller;
    }

    /**
     * Get the ReloadSchemaController for this Document
     * @return The ReloadSchemaController
     */
    public ReloadSchemaController getReloadSchemaController() {
        return _controller;
    }

    /**
     * Set the Document and Schema Controller
     */
    public void setDocument(File file, Document doc, ReloadSchemaController schemaController) {
        _file = file;
        _doc = doc;
        _controller = schemaController;
    }

    /**
     * This will save the XML IMS file with the existing File ref.
     * @throws ReloadException if an error occurs
     */
    public void saveDocument() throws ReloadException {
        if(_doc != null) {
            XMLUtils.write2XMLFile(_doc, _file);
            dirty = false;
            fireDocumentSaved();
        }
    }

    /**
     * This will save the XML IMS file with a new File ref
     * @param file The File to save as
     * @throws ReloadException if an error occurs
     */
    public void saveAsDocument(File file) throws ReloadException {
        _file = file;
        saveDocument();
    }

    /**
     * Get the JDOM Metadata Document.
     * @return The JDOM Document
     */
    public Document getDocument() {
        return _doc;
    }

    /**
     * @return the Root Element name
     */
    public abstract String getRootElementName();

    /**
     * @return the Comments to add to the XML Document
     */
    public abstract String[] getComments();

    /**
     * @return the Schema Namespace of the Document, or the Target Namespace if there isn't one
     */
    public Namespace getSchemaNamespace() {
        // Use Document's if we have one
        if(_doc.hasRootElement()) {
            Element root = getRootElement();
            return root.getNamespace();
        }
        // Otherwise, from Schema
        else return getTargetNamespace();
    }

    /**
     * @return the Target Schema Namespace of the Document
     */
    public Namespace getTargetNamespace() {
        ReloadSchema reloadSchema = _controller.getReloadSchema();
        return Namespace.getNamespace(reloadSchema.getTargetNamespace());
    }

    /**
     * Get the File of this Document.  This might be null.
     * @return
     */
    public File getFile() {
        return _file;
    }


    /**
     * Get the Root Element of the JDOM Document
     * @return The Root Element of the JDOM Document
     */
    public Element getRootElement() {
        return _doc.getRootElement();
    }


    /**
     * Return the first found Element for the given DOM Path or null if not found
     */
    public Element getElement(ReloadSchemaElement schemaElement) {
        return getElement(schemaElement.getDomPath());
    }

    /**
     * Return the first found Element for the given DOM Path or null if not found
     */
    public Element getElement(DomPath domPath) {
        Element element = _doc.getRootElement();

        String rootName = domPath.getRootPart();

        // Now work thru the path so we can drill into the Schema to find the correct point in the Schema
        StringTokenizer t = domPath.getElements();
        while(t.hasMoreElements()) {
            // Get next Element name
            String name = t.nextToken();

            // Ignore Root Element
            if(name.equals(rootName)) continue;

            element = element.getChild(name, element.getNamespace());
            // Not found
            if(element == null) break;
        }

        return element;
    }

    /**
     * @return true if element1 and element2 belong to the same Document
     */
    public static boolean isMemberOfSameDocument(Element element1, Element element2) {
        Document doc1 = element1.getDocument();
        Document doc2 = element2.getDocument();
        if(doc1 == null || doc2 == null) return false;
        return doc1.equals(doc2);
    }

    /**
     * @return true if element belongs to the Namespace of this Document
     */
    public boolean isDocumentNamespace(Element element) {
        return element.getNamespace() == getSchemaNamespace();
    }

    /**
     * Add our signature comments to the XML Document
     * @param comments An array of String comments to add
     */
    protected void addComments(Document doc) {
        String[] comments = getComments();
        if(comments != null) {
            for(int i = 0; i < comments.length; i++) {
                Comment comment = new Comment(comments[i]);
                doc.addContent(comment);
            }
        }
    }

    /**
     * Whether this Document has been changed in some way
     * @return
     */
    public boolean isDirty() {
        return dirty;
    }

    /**
     * Set whether this Document is dirty or not
     * @param isDirty true or false
     */
    public void setDirty(boolean isDirty) {
        dirty = isDirty;
    }

    /**
     * Cleanup
     */
    public void destroy() {
       _listeners.clear();
       _doc = null;
    }


//========================== ELEMENT OPERATIONS ================================

    /**
     * Add a new Element given the Path in the ReloadSchemaElement.
     * If the Element already exists, it won't be added
     * @param source Who is adding it
     * @param newSchemaElement
     * @return the added Element
     */
    public Element addElementUnique(Object source, ReloadSchemaElement newSchemaElement, boolean doSelect) {

        if(newSchemaElement == null) return null;

        // Get to the Root
        Element element = _doc.getRootElement();
        ReloadSchemaElement schemaElement = newSchemaElement.getReloadSchema().getRootElement();

        // Create a dom Path
        DomPath domPath = newSchemaElement.getDomPath();

        // Make a note of the Root name
        String rootName = domPath.getRootPart();

        StringTokenizer t = domPath.getElements();
        while(t.hasMoreElements()) {
            // Get next Element name
            String name = t.nextToken();

            // Ignore root
            if(name.equals(rootName)) continue;

            Namespace ns;

            // Check for Namespace prefix
            int idx = name.indexOf(':');
            if(idx >= 0) {
                String prefix = name.substring(0, idx);
                name = name.substring(idx + 1);
                ns = element.getNamespace(prefix);
            }
            else ns = element.getNamespace();

            // Get corresponding Child ReloadSchemaElement
            schemaElement = schemaElement.getChild(name, ns);
            if(schemaElement == null) break; // Oops

            // Get Child Element
            Element childElement = element.getChild(name, ns);

            // There is no child, so create one
            if(childElement == null) childElement = addElementBySchemaUndoable(source, element, schemaElement, doSelect);

            element = childElement;
        }

        return element;
    }

    /**
     * Add a new Element to the parent Element based on RELOADSchemaElement and register an Undo event
     * Create new JDOM Element based on childSchemaElement
     * Insert into the correct position!
     * Set any defaults of the Element
     * Add any default child Elements if needed
     * @param source Who is telling us this
     * @param parent The parent Element to add to
     * @param newSchemaElement The Type of Element we want to add
     */
    public Element addElementBySchemaUndoable(Object source, Element parentElement, ReloadSchemaElement newSchemaElement, boolean doSelect) {

        Element newElement = addElementBySchema(source, parentElement, newSchemaElement, doSelect);

        // Get Undoable Action after adding
        if(_undoHandler != null && newElement != null) {
            UndoableAddAction addAction = new UndoableAddAction(parentElement, newElement);
            _undoHandler.addUndoableAction(addAction);
        }

        return newElement;
    }


    /**
     * Add a new Element to the parent Element based on ReloadSchemaElement
     * Create new JDOM Element based on childSchemaElement
     * Insert into the correct position!
     * Set any defaults of the Element
     * Add any default child Elements if needed
     * @param source Who is telling us this
     * @param parent The parent Element to add to
     * @param newSchemaElement The Type of Element we want to add
     */
    public Element addElementBySchema(Object source, Element parentElement, ReloadSchemaElement newSchemaElement, boolean doSelect) {

        if(newSchemaElement == null) return null;

        // Get the Namespace
        Namespace ns;
        // If no parent use Document Namespace or given Namespace
        if(parentElement == null) ns = getSchemaNamespace();
        // Else determine parent or schema
        else {
            // If the prefix is missing it won't work properly, so use parent's NS
            String nsSchemaPrefix = newSchemaElement.getNamespacePrefix();
            if(nsSchemaPrefix.equals("")) ns = parentElement.getNamespace();
            else ns = newSchemaElement.getNamespace();
        }

        // Create Element
        Element newElement = new Element(newSchemaElement.getName(), ns);

        // If we have a default value, add it
        String defaultValue = _controller.getDefaultValue(newSchemaElement);
        if(defaultValue != null)  newElement.setText(defaultValue);

        // Add any default Attributes, if any - they will be added in order as per Schema
        if(newSchemaElement.hasAttributes()) {
            ReloadSchemaAttribute[] atts = newSchemaElement.getAttributes();

            for(int i = 0; i < atts.length; i++) {
                defaultValue = _controller.getDefaultValue(atts[i]);
                if(defaultValue != null) {
                    Attribute att = atts[i].createAttribute();
                    att.setValue(defaultValue);
                    newElement.setAttribute(att);
                }
            }
        }

        // Add to parent at correct position
        // If parent is null, that's OK it's probably the Document
        if(parentElement != null) {
            int index = getInsertPositionOfElement(parentElement, newSchemaElement);

            addElementAtIndex(source, parentElement, newElement, index, doSelect);
        }

        // Add any children that have a minimum occurance of one (i.e. mandatory)
        ReloadSchemaElement[] schemaChildren = newSchemaElement.getChildren();

        for(int i = 0; i < schemaChildren.length; i++) {
            ReloadSchemaElement child = schemaChildren[i];

            if(child.getMinOccurs() == 1) addElementBySchema(source, newElement, child, doSelect);
        }

        return newElement;
    }

    /**
     * Just add a new Element.  Listeners will only be notified once. Undo Event will be fired.
     */
    public Element addElementUndoable(Object source, Element parentElement, Element childElement, boolean doSelect) {

        Element newElement = addElement(source, parentElement, childElement, doSelect);

        // Get Undoable Action after adding
        if(_undoHandler != null && newElement != null) {
            UndoableAddAction addAction = new UndoableAddAction(parentElement, newElement);
            _undoHandler.addUndoableAction(addAction);
        }

        return newElement;
    }

    /**
     * Just add a new Element.  Listeners will only be notified once.
     * If the new Element has children it's up to the listener to handle them
     */
    public Element addElement(Object source, Element parentElement, Element childElement, boolean doSelect) {

        DomPath domPath = DomPath.getDomPath(parentElement);
        domPath.appendElementName(childElement.getName());
        ReloadSchemaElement schemaElement = (ReloadSchemaElement)_controller.getReloadSchemaNode(domPath);

        // Ascertain correct position
        int index = getInsertPositionOfElement(parentElement, schemaElement);
        return addElementAtIndex(source, parentElement, childElement, index, doSelect);
    }

    /**
     * Add an Element at an index Position and Notify listeners
     */
    public synchronized Element addElementAtIndex(Object source, Element parentElement, Element newElement, int index, boolean doSelect) {

        List children = parentElement.getChildren();
        if(index < 0 || index > children.size()) index = 0;
        children.add(index, newElement);

        // Notify
        ReloadDocumentListenerEvent event = new ReloadDocumentListenerEvent(source, this, newElement, doSelect);
        dirty = true;  // this first
        fireElementAdded(event);

        return newElement;
    }

    /**
     * Delete a child Element from its parent and notify any listeners and Undo
     * @param source Who is telling us this
     * @param element The Element to remove
     */
    public synchronized Element deleteElementUndoable(Object source, Element element) {
        Element parent = element.getParent();
        if(parent != null) {
            // Get Undoable Action First so we can get full dom path
            UndoableAction action = null;
            if(_undoHandler != null) {
                action = new UndoableDeleteAction(parent, element);
            }

            // Do it
            removeElement(source, element);

            if(_undoHandler != null && action != null) {
                _undoHandler.addUndoableAction(action);
            }
        }
        return element;
    }

    /**
     * Cut a child Element from its parent and notify any listeners and Undo
     * @param source Who is telling us this
     * @param element The Element to remove
     */
    public synchronized Element cutElementUndoable(Object source, Element element) {
        Element parent = element.getParent();
        if(parent != null) {
            // Get Undoable Action First so we can get full dom path
            UndoableAction action = null;
            if(_undoHandler != null) {
                action = new UndoableCutAction(parent, element);
            }

            // Do it
            removeElement(source, element);

            if(_undoHandler != null && action != null) {
                _undoHandler.addUndoableAction(action);
            }
        }
        return element;
    }

    /**
     * Remove a child Element from its parent and notify any listeners
     * @param source Who is telling us this
     * @param element The Element to remove
     */
    public synchronized Element removeElement(Object source, Element element) {
        Element parent = element.getParent();
        if(parent != null) {
            parent.removeContent(element);
            ReloadDocumentListenerEvent event = new ReloadDocumentListenerEvent(source, this, element, false);
            dirty = true; // this first
            fireElementRemoved(event);
        }
        return element;
    }

    /**
     * Move an Element to a new Parent
     * @return the Element moved
     */
    public Element moveElementUndoable(Object source, Element element, Element newParent, boolean doSelect) {
        Element oldParent = element.getParent();
        if(oldParent != null) {
            // Get Undoable Action First so we can get index position
            UndoableAction action = null;
            if(_undoHandler != null) {
                action = new UndoableMoveAction(oldParent, newParent, element);
            }

            // Do it
            removeElement(source, element);
            addElement(this, newParent, element, doSelect);

            if(_undoHandler != null && action != null) {
                _undoHandler.addUndoableAction(action);
            }
        }
        return element;
    }

    /**
     * Copy an Element to a new Parent
     * @return the Element copied
     */
    public Element copyElementUndoable(Object source, Element element, Element newParent, boolean doSelect) {
        Element newElement = (Element)element.clone();
        addElement(this, newParent, newElement, doSelect);
        if(_undoHandler != null) {
            UndoableAction action = new UndoableCopyAction(newParent, newElement);
            _undoHandler.addUndoableAction(action);
        }
        return newElement;
    }


    /**
     * Paste an Element to a new Parent
     * @return the Element copied
     */
    public Element pasteElementUndoable(Object source, Element element, Element newParent, boolean doSelect) {
        Element newElement = (Element)element.clone();
        addElement(this, newParent, newElement, doSelect);
        if(_undoHandler != null) {
            UndoableAction action = new UndoablePasteAction(newParent, newElement);
            _undoHandler.addUndoableAction(action);
        }
        return newElement;
    }

    /**
     * Move an Element up one place
     * @return the Element moved
     */
    public Element moveElementUp(Object source, Element element, boolean doSelect) {
        Element parent = element.getParent();
        if(parent != null) {
            List children = parent.getChildren();
            if(children.size() > 1) {
                int index = children.indexOf(element);
                if(index > 0) {
                    removeElement(source, element);
                    addElementAtIndex(source, parent, element, index - 1, doSelect);
                }
            }
        }
        return element;
    }

    /**
     * Move an Element down one place
     * @return the Element moved
     */
    public Element moveElementDown(Object source, Element element, boolean doSelect) {
        Element parent = element.getParent();
        if(parent != null) {
            List children = parent.getChildren();
            if(children.size() > 1) {
                int index = children.indexOf(element);
                if(index < children.size() - 1) {
                    removeElement(source, element);
                    addElementAtIndex(source, parent, element, index + 1, doSelect);
                }
            }
        }
        return element;
    }

    /**
     * Somebody has notified us that they have changed an Element in some way.
     * We will notify all listeners.
     * @param source Who is telling us this
     * @param element The Element that changed
     */
    public void changedElement(Object source, Element element) {
        ReloadDocumentListenerEvent event = new ReloadDocumentListenerEvent(source, this, element, false);
        dirty = true; // this first
        fireElementChanged(event);
    }


// ==========================================================================
// ELEMENT MOVING/COPYING/DRAGGING HANDLING
// ==========================================================================

    /**
     * Decide whether a child Element can be added to a parent Element by max amount
     * @param parentElement The parent Element to which we wish to add
     * @param childSchemaElement The SchemaElement of the child to add
     * @return True if we can add it, or false if not
     */
    public boolean canAddElement(Element parentElement, ReloadSchemaElement childSchemaElement) {
        if(childSchemaElement == null) return true;

        // Find out how many we can add
        int max = childSchemaElement.getMaxOccurs();
        // Unlimited
        if(max == -1) return true;
        // Compare with how many we have - we must use the namespace!
        List list = parentElement.getChildren(childSchemaElement.getName(), childSchemaElement.getNamespace());
        return list.size() < max;
    }

    /**
     * @return true if we are allowed (per Schema rules) to add a child
     */
    public boolean isAllowedChild(Element parentElement, Element childElement) {
        // For now, don't allow different namespaces
        if(!parentElement.getNamespace().equals(childElement.getNamespace())) return false;

        DomPath domPath = DomPath.getDomPath(parentElement);
        ReloadSchemaElement parentSchemaElement = (ReloadSchemaElement)getReloadSchemaController().getReloadSchemaNode(domPath);
        if(parentSchemaElement != null) {
            ReloadSchemaElement childSchemaElement = parentSchemaElement.getChild(childElement.getName());
            if(childSchemaElement == null) return false;
            return canAddElement(parentElement, childSchemaElement);
        }

        return false;
    }

    /**
     * Decide whether a Element can be deleted
     * @param element The Element which we wish to delete
     * @param schemaElement The SchemaElement of the Element
     * @return True if we can delete it, or false if not
     */
    public boolean canDeleteElement(Element element, ReloadSchemaElement schemaElement) {
        if(schemaElement == null) return true;

        // Root
        Element parentElement = element.getParent();
        if(parentElement == null) return false;
        // If we have to have a minimum of one
        List list = parentElement.getChildren(schemaElement.getName(), schemaElement.getNamespace());
        return list.size() > schemaElement.getMinOccurs();
    }

    /**
     * @return whether we can copy an element - will need to over-ride this
     */
    public boolean canCopyElement(Element element) {
        return false;
    }

    /**
     * @return whether we can cut an element - will need to over-ride this
     */
    public boolean canCutElement(Element element, ReloadSchemaElement schemaElement) {
        return false;
    }

    /**
     * @return whether we can paste an element - will need to over-ride this
     */
    public boolean canPasteFromClipboard(Element element) {
        return false;
    }

    /**
     * @return whether we can move the Element up one place
     */
    public boolean canMoveElementUp(Element element) {
        Element parent = element.getParent();
        if(parent != null) {
            List children = parent.getChildren();
            if(children.size() > 1) {
                int index = children.indexOf(element);
                if(index > 0) {
                    // Only if same type and namesace
                    Element previousSibling = (Element)children.get(index - 1);
                    if(previousSibling.getName().equals(element.getName()) &&
                       previousSibling.getNamespace().equals(element.getNamespace())) return true;
                }
            }
        }
        return false;
    }

    /**
     * @return whether we can move the Element down one place
     */
    public boolean canMoveElementDown(Element element) {
        Element parent = element.getParent();
        if(parent != null) {
            List children = parent.getChildren();
            if(children.size() > 1) {
                int index = children.indexOf(element);
                if(index < children.size() - 1) {
                    // Only if same type and namesace
                    Element nextSibling = (Element)children.get(index + 1);
                    if(nextSibling.getName().equals(element.getName()) &&
                       nextSibling.getNamespace().equals(element.getNamespace())) return true;
                }
            }
        }
        return false;
    }

//========================== ATTRIBUTE OPERATIONS ==============================

    /**
     * Create and Add a new Attribute to a given Element as per the Schema
     * @return The created attribute or null if it cannot be done
     */
    public Attribute addAttribute(Object source, Element parentElement, ReloadSchemaAttribute newSchemaAttribute) {
        if(parentElement == null || newSchemaAttribute == null) return null;

        Attribute att = newSchemaAttribute.createAttribute();
        if(att != null) {
            // Add it at the correct position
            java.util.List list = parentElement.getAttributes();
            int index = getInsertPositionOfAttribute(parentElement, newSchemaAttribute);
            if(index < 0 || index > list.size()) index = 0;
            list.add(index, att);
        }

        return att;
    }

//============================== SOME USEFUL ROUTINES  ==========================


    /**
     * @return the index position of an Element in relation to its parent
     */
    public static int getIndexOfElement(Element element) {
        int index = 0;
        Element parent = element.getParent();
        if(parent != null) {
            List children = parent.getChildren();
            index = children.indexOf(element);
            if(index == -1) index = 0;
        }
        return index;
    }

    /**
     * @return the correct position to insert an Element
     */
    public static int getInsertPositionOfElement(Element parentElement, ReloadSchemaElement newSchemaElement) {

        if(newSchemaElement == null) return parentElement.getChildren().size();

        // No children
        if(parentElement.getChildren().size() == 0) return 0;

        int index = 0;

        // New Element's name
        String name = newSchemaElement.getName();

        // Get the Parent SchemaElement
        ReloadSchemaElement parentSchemaElement = (ReloadSchemaElement)newSchemaElement.getParent();

        // If it's null we must be at the root
        if(parentSchemaElement == null) parentSchemaElement = newSchemaElement;

        // Get the Parent SchemaElement's children
        ReloadSchemaElement[] schemaChildren = parentSchemaElement.getChildren();

        // Iterate thru the Parent Element's children
        List children = parentElement.getChildren();

        for(int i = 0; i < children.size(); i++) {
            Element element = (Element)children.get(i);

            // Get Position of this Element as per Schema
            String elementName = element.getName();
            int pos = parentSchemaElement.indexofChild(elementName);

            // Reached an element with a different namespace
            //if(!element.getNamespaceURI().equals(newSchemaElement.getNamespaceURI())) pos = -1;

            // Check for -1 - this means an element not in the Schema
            if(pos != -1) {
                for(int j = pos; j < schemaChildren.length; j++) {
                    if(name.equals(schemaChildren[j].getName())) index = i + 1;
                }
            }
        }

        return index;
    }

    /**
     * @return the correct position to insert an Attribute in an Element
     */
    public static int getInsertPositionOfAttribute(Element parentElement, ReloadSchemaAttribute reloadSchemaAttribute) {
        int index = 0;

        // Get Element's Attributes
        java.util.List elementAtts = parentElement.getAttributes();

        // None here
        if(elementAtts.size() == 0) return 0;

        // Attributes's name
        String name = reloadSchemaAttribute.getName();

        // Get the Parent SchemaElement
        ReloadSchemaElement parentSchemaElement = (ReloadSchemaElement)reloadSchemaAttribute.getParent();

        // Get the Parent SchemaElement's schema attributes
        ReloadSchemaAttribute[] schemaAtts = parentSchemaElement.getAttributes();

        // Iterate thru the Parent Element's attributes
        for(int i = 0; i < elementAtts.size(); i++) {
            Attribute att = (Attribute)elementAtts.get(i);

            // Get Position of this Attribute as per Schema
            String attName = att.getName();
            int pos = parentSchemaElement.indexofAttribute(attName);

            // Check for -1 - this means an Attribute not in the Schema
            if(pos != -1) {
                for(int j = pos; j < schemaAtts.length; j++) {
                    if(name.equals(schemaAtts[j].getName())) index = i + 1;
                }
            }
            else {
                index = i + 1;
            }
        }

        return index;
    }

    //============================== LISTENER EVENTS  ==========================

    /**
     * List of Listeners
     */
    private Vector _listeners = new Vector();

    /**
     * Add a ReloadDocumentListener
     * @param listener The ReloadDocumentListener
     */
    public synchronized void addReloadDocumentListener(ReloadDocumentListener listener) {
        if(!_listeners.contains(listener)) _listeners.addElement(listener);
    }

    /**
     * Remove a ReloadDocumentListener
     * @param listener The ReloadDocumentListener
     */
    public synchronized void removeReloadDocumentListener(ReloadDocumentListener listener) {
        _listeners.removeElement(listener);
    }

    /**
     * Tell our listeners that we have added a new Element
     */
    public void fireElementAdded(ReloadDocumentListenerEvent event) {

        for(int i = _listeners.size() - 1; i >= 0; i--) {
            ReloadDocumentListener listener = (ReloadDocumentListener)_listeners.elementAt(i);
            listener.elementAdded(event);
        }
    }

    /**
     * Tell our listeners that we have deleted an Element
     */
    public void fireElementRemoved(ReloadDocumentListenerEvent event) {
        for(int i = _listeners.size() - 1; i >= 0; i--) {
            ReloadDocumentListener listener = (ReloadDocumentListener)_listeners.elementAt(i);
            listener.elementRemoved(event);
        }
    }

    /**
     * Tell our listeners that we have changed an Element
     */
    public void fireElementChanged(ReloadDocumentListenerEvent event) {

        for(int i = _listeners.size() - 1; i >= 0; i--) {
            ReloadDocumentListener listener = (ReloadDocumentListener)_listeners.elementAt(i);
            listener.elementChanged(event);
        }

        // -- This part is only for the Vocabulary Editor when Vdex ProfileType attribute is changed from the ReloadAttributeEditor ---

		if(event.getSource() instanceof ReloadAttributeEditor && event.getElement().getName().equals(Vocabulary.ROOT_NAME)) {

	        ReloadInternalFrame focussedInternalFrame = ReloadInternalFrameManager.getInstance().getFocussedInternalFrame();

			if(focussedInternalFrame instanceof VocabularyEditorFrame) {

				VocabularyEditorFrame vxFrame = (VocabularyEditorFrame)focussedInternalFrame;

				VocabularyEditor vxEditor = vxFrame.getVocabularyEditor();

		        Attribute att = event.getElement().getAttribute("profileType");

		        if(att != null) {

					vxEditor.setProfileType(att.getValue());
				}
			}

		}


        // --- This part is only for the Metadata Editor when using Vdex vocabulary files ---

        if(event.getSource() instanceof MetadataEditorFrame || event.getSource() instanceof MetadataComboBox
           || event.getSource() instanceof MetadataTextField || event.getSource() instanceof MetadataTreeTable
           || event.getSource() instanceof ReloadAttributeEditor || event.getSource() instanceof ReloadElementEditor) {

			// Check if the element that changed is the TaxonPath Source element
        	// lom/classification/taxonpath/source/langstring

        	Element sourceElement = event.getElement();

        	if(sourceElement.getName().equals("langstring") && sourceElement.getParent().getName().equals("source")) {

				if(sourceElement.getParent().getParent().getName().equals("taxonpath")) {

			        // if so then delete the Taxon tree below it after giving user warning
			        // if user presses OK then delete, if CANCEL is pressed then dont delete!
			        // lom/classification/taxonpath/taxon

					Element taxonElement = sourceElement.getParent().getParent().getChild("taxon", sourceElement.getNamespace());

					if(taxonElement != null) {

						int value = JOptionPane.showConfirmDialog(ReloadEditor.getInstance(),"Changing the source will delete any classifications in this taxon path", "Reload Editor",JOptionPane.OK_CANCEL_OPTION);

	    	    		if(value == JOptionPane.YES_OPTION) removeElement(ReloadDocument.this, taxonElement);
					}
				}
				else {

					// for all other 'source' elements get the next 'value' element
					// get the schemaElement corresponding to this 'value' element
					// get the vocabularyList corresponding to this schemaElement and update it
					// add ReloadListBox and MetadataComboBox as listeners to the VocabularyList
					// so when list updates these Boxes should also update their displayed lists!

					// (1) get element DomPath
					// (2) get rvList by calling ReloadVocabulary getVocabularyList(DomPath domPath)
					// (4) rvList.getList(vocabSource);

					String vocabSource = sourceElement.getText();

					Element valueElement = sourceElement.getParent().getParent().getChild("value", sourceElement.getNamespace());

					valueElement = valueElement.getChild("langstring", sourceElement.getNamespace());

					if(valueElement != null) {

						Element checkElement = valueElement;

						String path = "";

						while(!checkElement.isRootElement()) {

							path = "/" + checkElement.getName() + path;	// /general/aggregationlevel/value/langstring

							checkElement = checkElement.getParent();
						}
						path = checkElement.getName() + path; // lom/general/aggregationlevel/value/langstring

						DomPath domPath = new DomPath(path);

						ReloadVocabularyList rvList = getReloadSchemaController().getVocabularyList(domPath);

						if(rvList != null) rvList.setListUsingVocabSource(event.getSource(), vocabSource, valueElement, domPath);
					}
				}
			}
		}
    }

    /**
     * Tell our listeners that we have saved
     */
    public void fireDocumentSaved() {
        for(int i = _listeners.size() - 1; i >= 0; i--) {
            ReloadDocumentListener listener = (ReloadDocumentListener)_listeners.elementAt(i);
            listener.documentSaved(this);
        }
    }


// =============================================================================
// ============================ UNDO HANDLING ==================================
// =============================================================================

    public void setUndoHandler(ReloadUndoHandler undoHandler) {
        _undoHandler = undoHandler;
    }

    public ReloadUndoHandler getUndoHandler() {
        return _undoHandler;
    }

    /**
     * Handles Undo/Redo for Deleting Elements
     */
    protected class UndoableDeleteAction implements UndoableAction {
        Element parent;
        Element element;
        int index;
        String name;

        public UndoableDeleteAction(Element parent, Element element) {
            this.parent = parent;
            this.element = element;
            index = getIndexOfElement(element);
            // Friendly Name
            name = _controller.getElementFriendlyName(DomPath.getDomPath(element));
            if(name == null) name = element.getName();
        }

        public void undo() {
            addElementAtIndex(null, parent, element, index, true);
        }

        public void redo() {
            index = getIndexOfElement(element);
            removeElement(null, element);
        }

        public String getName() {
            return "Delete " + name;
        }
    }

    /**
     * Handles Undo/Redo for Cutting Elements
     */
    protected class UndoableCutAction extends UndoableDeleteAction {

        public UndoableCutAction(Element parent, Element element) {
            super(parent, element);
        }

        public String getName() {
            return "Cut " + name;
        }
    }

    /**
     * Handles Undo/Redo for Adding Elements
     */
    protected class UndoableAddAction implements UndoableAction {
        Element parent;
        Element element;
        int index;
        String name;

        public UndoableAddAction(Element parent, Element element) {
            this.parent = parent;
            this.element = element;
            index = getIndexOfElement(element);
            // Friendly Name
            name = _controller.getElementFriendlyName(DomPath.getDomPath(element));
            if(name == null) name = element.getName();
        }

        public void undo() {
            index = getIndexOfElement(element);
            removeElement(null, element);
        }

        public void redo() {
            addElementAtIndex(null, parent, element, index, true);
        }

        public String getName() {
            return "Add " + name;
        }
    }

    /**
     * Handles Undo/Redo for Copying Elements
     */
    protected class UndoableCopyAction extends UndoableAddAction {
        public UndoableCopyAction(Element parent, Element element) {
            super(parent, element);
        }

        public String getName() {
            return "Copy " + name;
        }
    }

    /**
     * Handles Undo/Redo for Pasting Elements
     */
    protected class UndoablePasteAction extends UndoableAddAction {
        public UndoablePasteAction(Element parent, Element element) {
            super(parent, element);
        }

        public String getName() {
            return "Paste " + name;
        }
    }

    /**
     * Handles Undo/Redo for Moving Elements
     */
    protected class UndoableMoveAction implements UndoableAction {
        Element oldParent;
        Element newParent;
        Element element;
        int oldIndex, newIndex;
        String name;

        public UndoableMoveAction(Element oldParent, Element newParent, Element element) {
            this.oldParent = oldParent;
            this.newParent = newParent;
            this.element = element;
            oldIndex = getIndexOfElement(element);
            // Friendly Name
            name = _controller.getElementFriendlyName(DomPath.getDomPath(element));
            if(name == null) name = element.getName();
        }

        public void undo() {
            newIndex = getIndexOfElement(element);
            removeElement(null, element);
            addElementAtIndex(null, oldParent, element, oldIndex, true);
        }

        public void redo() {
            oldIndex = getIndexOfElement(element);
            removeElement(null, element);
            addElementAtIndex(null, newParent, element, newIndex, true);
        }

        public String getName() {
            return "Move " + name;
        }
    }
}