/**
 *  RELOAD Editor - An IMS Metadata, Content Packaging and Learning Design tool.
 *
 *  Copyright (c) 2003 Oleg Liber, Bill Olivier, Phillip Beauvoir
 *
 *  Permission is hereby granted, free of charge, to any person obtaining a copy
 *  of this software and associated documentation files (the "Software"), to deal
 *  in the Software without restriction, including without limitation the rights
 *  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 *  copies of the Software, and to permit persons to whom the Software is
 *  furnished to do so, subject to the following conditions:
 *
 *  The above copyright notice and this permission notice shall be included in
 *  all copies or substantial portions of the Software.
 *
 *  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 *  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 *  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 *  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 *  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 *  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 *
 *  Project Management Contact:
 *
 *  Oleg Liber
 *  Bolton Institute of Higher Education
 *  Deane Road
 *  Bolton BL3 5AB
 *  UK
 *
 *  e-mail:   o.liber@bolton.ac.uk
 *
 *
 *  Technical Contact:
 *
 *  Phillip Beauvoir
 *  e-mail:   p.beauvoir@bolton.ac.uk
 *
 *  Web:      http://www.reload.ac.uk
 *
 */

package uk.ac.reload.dom;

import java.io.*;

import org.jdom.*;
import uk.ac.reload.schema.*;


/**
 * A convenience class to contain a JDOM Element, its associated ReloadSchemaElement
 * and the ReloadDocument to which the Element belongs.<br>
 * It has to implement Serializable for drag and drop on a Mac
 *
 * @author Phillip Beauvoir
 * @version 2003-08-01
 */
public class ReloadElementBinding implements Serializable {

    /**
     * The JDOM Element
     * This is transient to stop Mac drag and drop wanting serializable members
     */
    protected transient Element _element;

    /**
     * The JDOM Parent Element - useful if you don't know the Element
     * This is transient to stop Mac drag and drop wanting serializable members
     */
    protected transient Element _parent;

    /**
     * The associated ReloadSchemaElement
     * This is transient to stop Mac drag and drop wanting serializable members
     */
    protected transient ReloadSchemaElement _reloadSchemaElement;

    /**
     * The ReloadDocument to which the Element belongs
     * This is transient to stop Mac drag and drop wanting serializable members
     */
    protected transient ReloadDocument _reloadDocument;

    /**
     * Constructor
     * @param element The JDOM Element
     * @param reloadSchemaElement The associated ReloadSchemaElement
     * @param reloadDocument The ReloadDocument to which the Element belongs
     */
    public ReloadElementBinding(Element element, ReloadSchemaElement reloadSchemaElement, ReloadDocument reloadDocument) {
        _element = element;
        _reloadSchemaElement = reloadSchemaElement;
        _reloadDocument = reloadDocument;
    }

    /**
     * @return the JDOM Element
     */
    public Element getElement() {
        return _element;
    }

    /**
     * Set the Parent Element
     */
    public void setParentElement(Element parent) {
        _parent = parent;
    }

    /**
     * Get the ReloadSchemaElement
     * @return The associated ReloadSchemaElement
     */
    public ReloadSchemaElement getSchemaElement() {
        return _reloadSchemaElement;
    }

    /**
     * Could use this method instead
     * @return The associated ReloadSchemaElement
     */
    public ReloadSchemaElement __getSchemaElement() {
        if(_reloadSchemaElement == null && _element != null) {
            DomPath domPath = DomPath.getDomPath(_element);
            _reloadSchemaElement = (ReloadSchemaElement)getReloadDocument().getReloadSchemaController().getReloadSchemaNode(domPath);
        }
        return _reloadSchemaElement;
    }

    /**
     * Get the ReloadDocument
     * @return The ReloadDocument to which the Element belongs
     */
    public ReloadDocument getReloadDocument() {
        return _reloadDocument;
    }

    /**
     * @return The ReloadSchemaController
     */
    public ReloadSchemaController getReloadSchemaController() {
        return _reloadDocument.getReloadSchemaController();
    }

    /**
     * If the Element is null (as it might well be) try to create a new one given
     * that we have the parent Element set
     * @return the newly created Element or null if not successful
     */
    public Element createElement(Object source) {
        if(_parent != null) {
            _element = _reloadDocument.addElementBySchema(source, _parent, getSchemaElement(), false);
        }
        return _element;
    }
}