/**
 *  RELOAD Editor - An IMS Metadata, Content Packaging and Learning Design tool.
 *
 *  Copyright (c) 2003 Oleg Liber, Bill Olivier, Phillip Beauvoir
 *
 *  Permission is hereby granted, free of charge, to any person obtaining a copy
 *  of this software and associated documentation files (the "Software"), to deal
 *  in the Software without restriction, including without limitation the rights
 *  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 *  copies of the Software, and to permit persons to whom the Software is
 *  furnished to do so, subject to the following conditions:
 *
 *  The above copyright notice and this permission notice shall be included in
 *  all copies or substantial portions of the Software.
 *
 *  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 *  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 *  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 *  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 *  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 *  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 *
 *  Project Management Contact:
 *
 *  Oleg Liber
 *  Bolton Institute of Higher Education
 *  Deane Road
 *  Bolton BL3 5AB
 *  UK
 *
 *  e-mail:   o.liber@bolton.ac.uk
 *
 *
 *  Technical Contact:
 *
 *  Phillip Beauvoir
 *  e-mail:   p.beauvoir@bolton.ac.uk
 *
 *  Web:      http://www.reload.ac.uk
 *
 */

package uk.ac.reload.schema;

import java.io.*;
import java.util.*;

import javax.swing.*;

import uk.ac.reload.*;
import uk.ac.reload.dom.*;
import org.jdom.*;

/**
 * The Reload Schema Controller
 *
 * @author Phillip Beauvoir
 * @version 2003-08-01
 */
public abstract class ReloadSchemaController {
    /**
     * The Schema that will be our template
     */
    protected ReloadSchema _reloadSchema;

    /**
     * The Schema Files
     */
    protected File _schemaFile;

    /**
     * The Schema Helper that we will query
     */
    protected ReloadSchemaHelper _schemaHelper;

    /**
     * The Reload Vocabulary
     */
    protected ReloadVocabulary _vocab;

    /**
     * Constructor
     */
    public ReloadSchemaController() {

    }

    /**
     * Constructor
     */
    protected ReloadSchemaController(File fileSchema, File fileHelper, File fileVocab) {

        setSchemaFile(fileSchema, fileHelper);
        setVocabFile(fileVocab);
    }

    /**
     * Set the Schema File
     * @param fileSchema
     */
    public void setSchemaFile(File schemaFile, File fileHelper) {
        try {
            _schemaFile = schemaFile;
            _reloadSchema = ReloadSchema.getReloadSchema(schemaFile, getRootElementName());
            _schemaHelper = ReloadSchemaHelper.getReloadSchemaHelper(fileHelper);
        }
        catch(ReloadException ex) {
            ErrorDialogBox.showWarning("Could not load Schema: ", "Load Schema", ex);
        }
    }

    /**
     * Set the Vocabulary File
     * @param fileVocab
     */
    public void setVocabFile(File fileVocab) {
        try {

            _vocab = ReloadVocabulary.getReloadVocabulary(fileVocab);
        }
        catch(ReloadException ex) {
            ErrorDialogBox.showWarning("Could not load Vocabulary: ", "Load Vocabulary", ex);
        }
    }

    /**
     * Set the Default Language for the Language vocabulary
     * @return
     */
    public void setDefaultLanguage(String lang) {
        if(_vocab != null) _vocab.setDefaultLanguage(lang);
    }

    /**
     * For each JDOM Path there is a corresponding
     * ReloadSchemaNode.  This method finds that corresponding
     * ReloadSchemaNode in this Schema and returns it or null if not found.
     * @param domPath The Path to find the ReloadSchemaElement for.
     * @return The ReloadSchemaNode or null if not found
     */
    public ReloadSchemaNode getReloadSchemaNode(DomPath domPath) {

        String attName = null;

        // Get Attribute & Elements Part if this is an Attribute
        if(domPath.isAttribute()) {

            attName = domPath.getAttributePart();
            domPath = new DomPath(domPath.getElementsPart());
        }

        ReloadSchemaElement schemaElement = _reloadSchema.getRootElement();

        // Now work thru the path so we can drill into the Schema to find the correct point in the Schema
        StringTokenizer t = domPath.getElements();
        while(t.hasMoreElements()) {
            // Get next Element name
            String name = t.nextToken();

            // If schemaElement is not root because we are at the top level
            if(!name.equals(_reloadSchema.getRootElementName())) {
                schemaElement = schemaElement.getChild(name);

                if(schemaElement == null) break; // Oops!
            }
        }

        // We do have an Attribute
        if(attName != null) {
            ReloadSchemaAttribute schemaAttribute = schemaElement.getAttribute(attName);
            if(schemaAttribute != null) return schemaAttribute;
        }

        return schemaElement;
    }

    /**
     * @return a Helper vlaue for a given key
     */
    public String getElementHelperValue(DomPath domPath, String helperKey) {
        return _schemaHelper == null ? null : _schemaHelper.getHelperValue(domPath, helperKey);
    }

    /**
     * @return a Friendly name
     */
    public String getElementFriendlyName(DomPath domPath) {
        return getElementHelperValue(domPath, ReloadSchemaHelper.FNAME);
    }

    /**
     * @return a Tip
     */
    public String getElementTip(DomPath domPath) {
        return getElementHelperValue(domPath, ReloadSchemaHelper.TIP);
    }

    /**
     * @return the widget type or null if not set
     */
    public String getWidgetType(DomPath domPath) {
        return getElementHelperValue(domPath, ReloadSchemaHelper.WIDGET);
    }

    /**
     * @return a Leaf Icon for an Element
     */
    public abstract Icon getLeafIcon(Element element);

    /**
     * Return a Closed Icon for an Element
     */
    public abstract Icon getClosedIcon(Element element);

    /**
     * Return an Open Icon for an Element
     */
    public abstract Icon getOpenIcon(Element element);

    /**
     * @return a Leaf Icon for an Element
     */
    public abstract Icon getLeafIcon(ReloadSchemaElement schemaElement);

    /**
     * Return a Closed Icon for an Element
     */
    public abstract Icon getClosedIcon(ReloadSchemaElement schemaElement);

    /**
     * Return an Open Icon for an Element
     */
    public abstract Icon getOpenIcon(ReloadSchemaElement schemaElement);

    /**
     * @return The root element name
     */
    public abstract String getRootElementName();

    /**
     * Get the Vocabulary list for a given element or attribute.
     * We first of all look to see if we have a Helper Vocabulary defined.
     * If not, look for one as defined in the Schema.
     * @param schemaNode The Element or Attribute for which to get the vocab list
     * @return The Vocabulary List or null if no list present
     */
    public ReloadVocabularyList getVocabularyList(ReloadSchemaNode schemaNode) {
        if(schemaNode == null) return null;

        ReloadVocabularyList rvList = null;

        // Check for Helper list first
        if(_vocab != null) rvList = _vocab.getVocabularyList(schemaNode);

        // Now check for internal Schema
        if(rvList == null) rvList = schemaNode.getVocabularyList();

        return rvList;
    }

    /**
     * Get the Vocabulary list for a given element or attribute.
     * We first of all look to see if we have a Helper Vocabulary defined.
     * If not, look for one as defined in the Schema.
     * @param schemaNode The Element or Attribute for which to get the vocab list
     * @return The Vocabulary List or null if no list present
     */
    public ReloadVocabularyList getVocabularyList(DomPath domPath) {
        if(domPath == null) return null;

        ReloadVocabularyList rvList = null;

        // Check for Helper list first
        if(_vocab != null) rvList = _vocab.getVocabularyList(domPath);

        return rvList;
    }

    /**
     * Get the Vocabulary list for a given element or attribute.
     * We first of all look to see if we have a Helper Vocabulary defined.
     * If not, look for one as defined in the Schema.
     * @param schemaNode The Element or Attribute for which to get the vocab list
     * @return The Vocabulary List or null if no list present
     */
    public ReloadVocabularyList getVocabularyList(ReloadSchemaNode schemaNode, ReloadDocument doc) {
        if(schemaNode == null) return null;

        ReloadVocabularyList rvList = null;

        // Check for Helper list first
        if(_vocab != null) rvList = _vocab.getVocabularyList(schemaNode);

        // Now check for internal Schema
        if(rvList == null) rvList = schemaNode.getVocabularyList();

        return rvList;
    }

    /**
     * Get the default value of a ReloadSchemaNode
     * @param schemaNode
     * @return The default value or null
     */
    public String getDefaultValue(ReloadSchemaNode schemaNode) {
        String def = null;

        // Check for helper value first
        ReloadVocabularyList rvList = getVocabularyList(schemaNode);
        if(rvList != null) def = rvList.getDefaultValue();

        // Not found, so check for Schema
        if(def == null) def = schemaNode.getDefaultValue();

        return def;
    }

    /**
     * @return a Facet value of an Element or attribute.  Will look in the Helper file
     * first and then the Schema.  The Facet has to be a valid Schema type Facet.
     */
    public String getFacetValue(ReloadSchemaNode schemaNode, String facetName) {
        // Try Helper file first
        String value = getElementHelperValue(schemaNode.getDomPath(), facetName);
        // Try Schema
        if(value == null) value = schemaNode.getFacetValue(facetName);

        return value;
    }

    /**
     * Return the ReloadSchema connected with this Controller
     * @return
     */
    public ReloadSchema getReloadSchema() {
        return _reloadSchema;
    }

    /**
     * @return the Schema File for this Schema
     */
    public File getSchemaFile() {
        return _schemaFile;
    }

    /**
     * Get the Schema Helper
     * @return The Schema Helper
     */
    public ReloadSchemaHelper getSchemaHelper() {
        return _schemaHelper;
    }

    /**
     * Copy Schema Files to a given folder - implemetors will over-ride this
     */
    public void copySchemaFilesToFolder(File folder) throws ReloadException {

    }
}