/**
  *  RELOAD Editor - An IMS Metadata, Content Packaging and Learning Design tool.
  *
  *  Copyright (c) 2003 Oleg Liber, Bill Olivier, Phillip Beauvoir
  *
  *  Permission is hereby granted, free of charge, to any person obtaining a copy
  *  of this software and associated documentation files (the "Software"), to deal
  *  in the Software without restriction, including without limitation the rights
  *  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
  *  copies of the Software, and to permit persons to whom the Software is
  *  furnished to do so, subject to the following conditions:
  *
  *  The above copyright notice and this permission notice shall be included in
  *  all copies or substantial portions of the Software.
  *
  *  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  *  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  *  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
  *  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
  *  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
  *  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
  *
  *  Project Management Contact:
  *
  *  Oleg Liber
  *  Bolton Institute of Higher Education
  *  Deane Road
  *  Bolton BL3 5AB
  *  UK
  *
  *  e-mail:   o.liber@bolton.ac.uk
  *
  *
  *  Technical Contact:
  *
  *  Phillip Beauvoir
  *  e-mail:   p.beauvoir@bolton.ac.uk
  *
  *  Web:      http://www.reload.ac.uk
  *
  */

package uk.ac.reload.schema;

import java.io.*;
import java.util.*;

import org.jdom.*;
import uk.ac.reload.*;
import uk.ac.reload.dom.*;
import uk.ac.reload.util.*;

/**
 * The Reload Vocabulary - Defines a Vocabulary mapping between Elements/Attributes
 * and a Vocabulary List.  The actual Vocabulary is an XML File that defines the mappings.
 *
 *
 * @author Phillip Beauvoir
 * @version 2003-05-14
 */
public class ReloadVocabulary {
    /**
     * Static ReloadVocabularies - we only want to create these just once and re-use them
     */
    private static Hashtable reloadVocabulariesTable = new Hashtable();

    /**
     * Factory method for getting a static re-usable ReloadVocabulary
     * @param fileVocab The file of the Vocabulary file to get
     * @return The ReloadVocabulary or null if not found
     * @throws ReloadException
     */
    public static ReloadVocabulary getReloadVocabulary(File fileVocab) throws ReloadException {
        if(fileVocab == null) return null;
        // Do we have it already?
        ReloadVocabulary reloadVocabulary = (ReloadVocabulary)reloadVocabulariesTable.get(fileVocab);
        // No, first time creation, create and store in table
        if(reloadVocabulary == null) {
            // Does it exist?
            if(fileVocab.exists()) {
                reloadVocabulary = new ReloadVocabulary(fileVocab);
                reloadVocabulariesTable.put(fileVocab, reloadVocabulary);
            }
        }
        return reloadVocabulary;
    }

    /**
     * The JDOM Document that is created from the XML File
     */
    protected Document _docVocab;

    /**
     * The XML Vocabulary File.
     */
    protected File _fileVocab;


    /**
     * The Vocabulary lists
     */
    protected Vector _lists;


    /**
     * Constructor
     *
     * @param fileVocab The Vocabulary XML File
     * @throws ReloadException
     */
    private ReloadVocabulary(File fileVocab) throws ReloadException {
        if(fileVocab == null) throw new ReloadException("Vocabulary file cannot be null");
        _fileVocab = fileVocab;

        // Get the vocab document
        _docVocab = loadVocabularyFile(fileVocab);

        // Map the lists
        _lists = mapLists();
    }

    /**
     * Load the XML Vocabualry file into a JDOM Document.
     *
     * @param fileVocab
     * @return The JDOM Document
     * @throws ReloadException If cannot create Document
     */
    private Document loadVocabularyFile(File fileVocab) throws ReloadException {
        return XMLUtils.readXMLFile(fileVocab);
    }

    /**
     * Get the Vocabulary JDOM Document
     *
     * @return The Vocabulary JDOM Document
     */
    public Document getVocabularyDocument() {
        return _docVocab;
    }

    /**
     * Get the name of the vocabulary. e.g. "IMS-LRM"
     * @return The name of the vocabulary.
     */
    public String getVocabularyName() {
        return _docVocab.getRootElement().getAttributeValue("name");
    }


    /**
     * Get a Vocabulary List for a Given Element or Attribute.
     * @param schemaNode The Element or Attribute
     * @return The Vocabulary List or null if there isn't one
     */
    public ReloadVocabularyList getVocabularyList(ReloadSchemaNode schemaNode) {
        if(schemaNode == null) return null;

        // Get the path of this node
        DomPath nodePath = schemaNode.getDomPath();

        // Enumerate through the lists
        Enumeration lists = _lists.elements();
        while(lists.hasMoreElements()) {
            ReloadVocabularyList rvList = (ReloadVocabularyList)lists.nextElement();

            DomPath listPath = rvList.getDomPath();

            // Exact match or Partial match
            if(nodePath.equals(listPath) || nodePath.endsWith(listPath)) {
                // An over-riding none list
                if(rvList.getListName().equals("none")) return null;
                return rvList;
            }
        }

        return null;
    }

    // (3) getVocabularyList(DomPath domPath)
    /**
     * Get a Vocabulary List for a Given Element or Attribute.
     * @param domPath of the element
     * @return The Vocabulary List or null if there isn't one
     */
    public ReloadVocabularyList getVocabularyList(DomPath nodePath) {
        if(nodePath == null) return null;

        // Enumerate through the lists
        Enumeration lists = _lists.elements();
        while(lists.hasMoreElements()) {
            ReloadVocabularyList rvList = (ReloadVocabularyList)lists.nextElement();

            DomPath listPath = rvList.getDomPath();

            // Exact match or Partial match
            if(nodePath.equals(listPath) || nodePath.endsWith(listPath)) {
                // An over-riding none list
                if(rvList.getListName().equals("none")) return null;
                return rvList;
            }
        }

        return null;
    }


    /**
     * Set the default language of format "en", "fr" etc
     * @param language
     */
    public void setDefaultLanguage(String lang) {
        Enumeration lists = _lists.elements();
        while(lists.hasMoreElements()) {
            ReloadVocabularyList rvList = (ReloadVocabularyList)lists.nextElement();
            if(rvList.isLangList()) rvList.setDefaultValue(lang);
        }
    }

    /**
     * Map the Vocabulary Lists in a Vector such that we can get a List from its path
     * @return
     */
    private Vector mapLists() {

        Vector map = new Vector();

        // Get all "element" children
        Iterator elementlist = _docVocab.getRootElement().getChildren("element").iterator();

        // Enumerate through them
        while(elementlist.hasNext()) {
            // Get the Element
            Element childElement = (Element) elementlist.next();

            // Get the Path and List name
            String listPath = childElement.getAttributeValue("path");
            String listName = childElement.getAttributeValue("list");
            String defaultListName = childElement.getAttributeValue("defaultList");

            if(listPath == null || listName == null) continue;

            // New ReloadVocabularyList
            ReloadVocabularyList reloadvocabList = new ReloadVocabularyList(new DomPath(listPath), listName);
            map.addElement(reloadvocabList);

            reloadvocabList.setDefaultListName(defaultListName);

            // Iterate through the vocab lists
            Iterator vocablist = _docVocab.getRootElement().getChildren("vocabList").iterator();
            while(vocablist.hasNext()) {
                Element vocabElement = (Element)vocablist.next();

                // Get the name of the vocab list
                String vocabName = vocabElement.getAttributeValue("name");
                // Oops!
                if(vocabName == null) break;

                // Do a match
                if(listName.equals(vocabName)) {
                    // Get each "item" in the list
                    List items = vocabElement.getChildren("item");

                    // Create an appropriate String array
                    String[] s = new String[items.size()];
                    for(int i = 0; i < items.size(); i++) {
                        Element e = (Element) items.get(i);


                        // Add item to String List
                        s[i] = e.getText();

                        // If default, save it
                        Attribute att = e.getAttribute("default");

                        if(att != null) {

							reloadvocabList.setDefaultValue(e.getText());
						}
                    }
                    reloadvocabList.setList(s);

                    reloadvocabList.setDefaultList(s);

                    break;
                }
            }
        }

        return map;
    }

}