/**
  *  RELOAD Editor - An IMS Metadata, Content Packaging and Learning Design tool.
  *
  *  Copyright (c) 2003 Oleg Liber, Bill Olivier, Phillip Beauvoir
  *
  *  Permission is hereby granted, free of charge, to any person obtaining a copy
  *  of this software and associated documentation files (the "Software"), to deal
  *  in the Software without restriction, including without limitation the rights
  *  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
  *  copies of the Software, and to permit persons to whom the Software is
  *  furnished to do so, subject to the following conditions:
  *
  *  The above copyright notice and this permission notice shall be included in
  *  all copies or substantial portions of the Software.
  *
  *  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  *  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  *  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
  *  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
  *  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
  *  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
  *
  *  Project Management Contact:
  *
  *  Oleg Liber
  *  Bolton Institute of Higher Education
  *  Deane Road
  *  Bolton BL3 5AB
  *  UK
  *
  *  e-mail:   o.liber@bolton.ac.uk
  *
  *
  *  Technical Contact:
  *
  *  Phillip Beauvoir
  *  e-mail:   p.beauvoir@bolton.ac.uk
  *
  *  Web:      http://www.reload.ac.uk
  *
  */

package uk.ac.reload.util.htmlparser;

import java.io.*;
import java.net.*;
import java.util.*;

import javax.swing.text.*;
import javax.swing.text.html.*;
import javax.swing.text.html.parser.*;

import uk.ac.reload.util.*;
import uk.ac.reload.*;

/**
 * Class used to parse a web page and find all dependant links contained
 * within it.  Additionally used to figure out how & where to copy files
 * when a user wishes to import them
 *
 * @author Paul Sharples
 * @version 2003-07-01
 */
public class HTMLParser extends HTMLEditorKit.ParserCallback  {

    // An array of the HTML attributes that may contain references to files
    // HREF is used in hyperlinks
    // BACKGROUND is used in tables, body, etc
    // SRC is used in <object> and <embed>, etc
    // CODE is used for Applets
    // ACTION is used for submitting forms

    // This may need to be moved into ReloadFileType????
    protected HTML.Attribute[] _htmlAttributes = {HTML.Attribute.HREF, HTML.Attribute.BACKGROUND,
        HTML.Attribute.SRC, HTML.Attribute.CODE, HTML.Attribute.ACTION};

    /**
     * The File to Parse
     */
    protected File _htmlFile;

    /**
     * We need to check the first page and see if it refers to local links
     * that are higher up in the folder structure than itself. _firstpage
     * will be used to flag up the first time the code is asked to parse a page.
     */
    protected boolean _firstPage;

    /**
     * The Project Folder
     */
    protected File _projectFolder;

    /**
     * We want to know if a user has imported a page which has a local link
     * higher up the directory tree structure than itself
     */
    protected boolean _higherLocalLinkFound;

    /**
     * We want to know if a user has put an absolute local link in a page
     */
    protected boolean _absoluteLocalLinkFound;

    /**
     * Our main data structure to hold source/destination paths
     */
    protected Hashtable _htmlResourcesTable;

    /**
     * Hold the starting point for the webpage(s)
     */
    protected File _startPath;

    /**
     * All Links
     */
    protected Vector _allLinks;

    /**
     * Basic Constructor. Takes the web page to be parsed as first argument
     */
    public HTMLParser(File htmlFile) {
        _htmlFile = htmlFile;
    }

    /**
     * Constructor. Takes the web page to be parsed as first argument. Takes a
     * folder path as the second argument, used to calculate where the files
     * are to be copied to.
     * @param htmlFile
     * @param CpPath
     */
    public HTMLParser(File htmlFile, File CpPath, boolean userWantsToImport) {
        _htmlFile = htmlFile;
        _projectFolder = CpPath;
        _firstPage = true;
        _higherLocalLinkFound = false;
        _absoluteLocalLinkFound = false;
        _startPath = htmlFile;

        if(userWantsToImport){
            _htmlResourcesTable = new Hashtable();
            // create the first destination file using the CpPath and appending the source filename
            File destination = new File(CpPath.toString() + File.separatorChar + htmlFile.getName().toString());
            _htmlResourcesTable.put(htmlFile, destination);
            // pass the info over to the parsing function
            parseTheHtml(htmlFile, CpPath.toString());
        }
        else{
            _allLinks = new Vector();
            parseTheHtml(htmlFile);
        }
    }


    /**
     * Return all Links
     * @return
     */
    public File[] getAllLinks(){
        File[] files = new File[_allLinks.size()];
        _allLinks.copyInto(files);
        return files;
    }

    /**
     * Accessor method for links found in page
     * @return - hashtable of all source(key) and destination(value) files
     */
    public Hashtable getLinks(){
        return _htmlResourcesTable;
    }

    /**
     * Accessor method to see if any FULL absolute local links were found in the page(s)
     * i.e. 'file:///c:/dreamweaversite/acmeweb/index.htm'
     * @return true/false
     */
    public boolean isAbsoluteLocalLinkFound(){
        return _absoluteLocalLinkFound;
    }

    /**
     * Accessor method to see if any local links were found in the first page
     * that are higher in the directory tree than itself.
     * @return true/false
     */
    public boolean isHigherLocalLinkFound() {
        return _higherLocalLinkFound;
    }


    /**
     * parseTheHtml(File HtmlFile) - used for second pass to find all links
     * @param HtmlFile
     */
    private void parseTheHtml(File HtmlFile){
          final String sourcePath = HtmlFile.getParent() + File.separatorChar;

          if(HtmlFile.toString().toLowerCase().endsWith(".css")) {
              CSSParser aCssFile = new CSSParser(HtmlFile);
              String[] theCssLinks = aCssFile.getLinks();
              for(int i = 0; i < theCssLinks.length; i++) {
                  buildTheResource(theCssLinks[i], sourcePath);
              }
              return;
          }

          HTMLEditorKit.ParserCallback callback = new HTMLEditorKit.ParserCallback (){
              // Instead of having almost identical code for both handleStartTag and
              // handleSimpleTag methods, put the code into subroutines.
              public void handleStartTag(HTML.Tag tag, MutableAttributeSet attrSet, int pos) {
                  buildTheResource(parseAttributes(tag, attrSet, pos, sourcePath), sourcePath);
              }
              public void handleSimpleTag(HTML.Tag tag, MutableAttributeSet attrSet, int pos) {
                  buildTheResource(parseAttributes(tag, attrSet, pos, sourcePath), sourcePath);
              }
          };
          try {
              Reader reader = new FileReader(HtmlFile);
              new ParserDelegator().parse(reader, callback, true);
          }
          catch(Exception e) {
              if(ReloadEditor.DEBUG) e.printStackTrace();
          }
      }// end of parseTheHTML()


      private String parseAttributes(HTML.Tag tag, MutableAttributeSet attrSet, int pos, String sourcePath){
          String returnString=null;
          for (Enumeration attributes = attrSet.getAttributeNames(); attributes.hasMoreElements() ;) {
              Object o = attributes.nextElement();
              // deal with any javascript calls first
              String attributename = o.toString();
              Iterator listElement;
              listElement = ReloadFileType.JAVASCRIPTEVENTS.iterator();
              while (listElement.hasNext()) {
                   if (attributename.toLowerCase().compareTo(listElement.next().toString()) == 0) {
                       String attributevalue = (String) attrSet.getAttribute(o);
                      dealWithJavascriptEvent(attributevalue, sourcePath);
                  }
              }
              // next lets look for specific attributes that may contain file references
              for (int attributeArrayIndex=0; attributeArrayIndex < _htmlAttributes.length; attributeArrayIndex++){
                  if (o == _htmlAttributes[attributeArrayIndex]){
                      String aLink = (String) attrSet.getAttribute(o);
                      if (!(aLink.toLowerCase().startsWith("http:") || aLink.toLowerCase().startsWith("https:")
                          || aLink.toLowerCase().startsWith("mailto:"))) {
                          if (aLink.toLowerCase().startsWith("javascript:") ||
                              aLink.startsWith("#")) {
                              dealWithJavascriptEvent(aLink, sourcePath);
                              returnString = null;
                          }
                          else if (aLink.toLowerCase().startsWith("file:")) {
                              _absoluteLocalLinkFound = true;
                              returnString = null;
                          }
                          else {
                              returnString = aLink;
                          }
                      }
                  }
              }
          }
          return returnString;
      }


      private void dealWithJavascriptEvent(String aLink, String sourcePath){
          // split the string at the ' character and examine
          String[] javascriptBits = aLink.split("'");
          // cycle thru all of the strings that were split
          for (int scriptArrayIndex=0; scriptArrayIndex < javascriptBits.length; scriptArrayIndex++){
              // cycle thru all of the file types defined in _webFiles to see if we
              // can find a link
              Iterator listElement;
              listElement = ReloadFileType.ALL_EXTENSIONS.iterator();
              while (listElement.hasNext()) {
                  // if we find that link...
                  if (javascriptBits[scriptArrayIndex].toLowerCase().endsWith(listElement.next().toString())){
                      // call buildTheResource()
                      buildTheResource(javascriptBits[scriptArrayIndex],sourcePath);
                  }
              }
          }
      }


      private void buildTheResource(String localLink, String sourcePath){
          File destinationCanonical = null;
          if(localLink != null) {
              URLDecoder _decoder = new URLDecoder();
              try {
                  File original = new File(_decoder.decode(sourcePath + localLink, "UTF-8"));
                  if(original.exists()) {
                      try {
                          // we need to get the canonical path (without ../../) and add that to the table
                          File originalCanonical = new File((String) original.getCanonicalPath());
                          // Make sure that the link is not referencing a file or folder higher
                          // than the project folder itself. If the new "destination link" starts
                          // with the project folder, then we are okay, otherwise theres a problem
                          if(originalCanonical.toString().startsWith(_projectFolder.toString())) {
                              // Do we have it already?
                              boolean filePath = false;
                              filePath = _allLinks.contains(originalCanonical);
                              // No, first time creation, store in table
                              if(!filePath) {
                                  if(!originalCanonical.isDirectory())
                                      _allLinks.add(originalCanonical);
                                  Iterator listElement;
                                  listElement = ReloadFileType.EXTENSIONS_TO_PARSE.iterator();
                                  while(listElement.hasNext()) {
                                      if(localLink.toLowerCase().endsWith(listElement.next().toString())) {
                                          _firstPage = false;
                                          parseTheHtml(original);
                                      }
                                  }
                              }
                          }
                          else {
                              _higherLocalLinkFound = true;
                              // do something if the link is higher than the project folder
                          }
                      }
                      catch(IOException e) {
                          if(ReloadEditor.DEBUG) System.out.println("A problem occured when trying to find all links in " + localLink +
                                             " This is probably due to the page having Absolute links to local files on your computer.  Please amend this page\n");
                      }
                  }
              }
              catch(Exception ex) {
                  if(ReloadEditor.DEBUG) {
                      if(ReloadEditor.DEBUG) System.out.println("Encoding exception: ");
                      ex.printStackTrace();
                  }
              }
          }
      }

///





    /**
     * ParsetheHtml() - utilises HTMLEditorKit.ParserCallback which reports on
     * either a simple tag <img> or starttag <a href="something">.  Once the
     * parser has reported it has found a tag, it is passed to parseAttributes()
     * which then looks for attributes that may contain links to other resources
     * Either a string representation of the link is returned or null. This result
     * is then passed to buildTheResource() which checks the file exists and adds
     * it to a local hashmap of pages.  parseTheHtml() is recursively called from
     * buildTheResource() if it finds a link that may contain other links
     * i.e. another web page...
     * @param HtmlFile
     * @param destPath
     */
    private void parseTheHtml(final File HtmlFile, String destPath){
        final String sourcePath = HtmlFile.getParent() + File.separatorChar;
        final String packagePath = destPath.toString() + File.separatorChar;

        if(HtmlFile.toString().toLowerCase().endsWith(".css")){
            CSSParser aCssFile = new CSSParser(HtmlFile);
            String[] theCssLinks = aCssFile.getLinks();
            for(int i = 0; i < theCssLinks.length; i++) {
                buildTheResource(theCssLinks[i], sourcePath, packagePath, HtmlFile);
            }
            return;
        }

        HTMLEditorKit.ParserCallback callback = new HTMLEditorKit.ParserCallback (){
            // Instead of having almost identical code for both handleStartTag and
            // handleSimpleTag methods, put the code into subroutines.
            public void handleStartTag(HTML.Tag tag, MutableAttributeSet attrSet, int pos) {
                buildTheResource(parseAttributes(tag, attrSet, pos, sourcePath, packagePath, HtmlFile),sourcePath, packagePath, HtmlFile);
            }
            public void handleSimpleTag(HTML.Tag tag, MutableAttributeSet attrSet, int pos) {
                buildTheResource(parseAttributes(tag, attrSet, pos, sourcePath, packagePath, HtmlFile),sourcePath, packagePath, HtmlFile);
            }
        };
        try {
            Reader reader = new FileReader(HtmlFile.getAbsolutePath());
            new ParserDelegator().parse(reader, callback, true);
        } catch (Exception e) {
            if(ReloadEditor.DEBUG) System.out.println("Error: Unable to start parsing '"+ e.toString() +"'");
        }
    }// end of parseTheHTML()


    /**
     * dealWithJavascriptEvent - is passed a string which may contain a javascript
     * event (from parseAttributes()).  The string is split by a single quote into
     * an array.  Each element is then checked against _webFiles (an array of file
     * types) to see f it references any other files.  If a filename is found, it
     * is passed to buildtheResource() which tries to establish whether or not the
     * file exists on disk and if the hashmap does not already contain the file
     * reference, it adds it
     * @param aLink
     * @param sourcePath
     * @param packagePath
     */
    private void dealWithJavascriptEvent(String aLink, String sourcePath, String packagePath, File pageFrom){
        // split the string at the ' character and examine
        String[] javascriptBits = aLink.split("'");
        // cycle thru all of the strings that were split
        for (int scriptArrayIndex=0;scriptArrayIndex<javascriptBits.length;scriptArrayIndex++){
            // cycle thru all of the file types defined in _webFiles to see if we
            // can find a link
            Iterator listElement;
            listElement = ReloadFileType.ALL_EXTENSIONS.iterator();
            while (listElement.hasNext()) {
                // if we find that link...
                if (javascriptBits[scriptArrayIndex].toLowerCase().endsWith(listElement.next().toString())){
                    // call buildTheResource()
                    buildTheResource(javascriptBits[scriptArrayIndex],sourcePath, packagePath, pageFrom);
                }
            }
        }
    }

    /**
     * parseAttributes() - parses a HTML tag for any attributes that may contain
     * references to local files.  If it finds any, then it returns the string
     * reference to that file, otherwise null.
     * @param tag
     * @param attrSet
     * @param pos
     * @param sourcePath
     * @param packagePath
     * @return
     */
    private String parseAttributes(HTML.Tag tag, MutableAttributeSet attrSet, int pos, String sourcePath, String packagePath, File pageFrom){
        String returnString=null;
        for (Enumeration attributes = attrSet.getAttributeNames(); attributes.hasMoreElements() ;) {
            Object o = attributes.nextElement();
            // deal with any javascript calls first
            String attributename = o.toString();
            Iterator listElement;
            listElement = ReloadFileType.JAVASCRIPTEVENTS.iterator();
            while (listElement.hasNext()) {
                 if (attributename.toLowerCase().compareTo(listElement.next().toString()) == 0) {
                     String attributevalue = (String) attrSet.getAttribute(o);
                    dealWithJavascriptEvent(attributevalue, sourcePath, packagePath, pageFrom);
                }
            }
            // next lets look for specific attributes that may contain file references
            for (int attributeArrayIndex=0; attributeArrayIndex < _htmlAttributes.length; attributeArrayIndex++){
                if (o == _htmlAttributes[attributeArrayIndex]){
                    String aLink = (String) attrSet.getAttribute(o);
                    if (!(aLink.toLowerCase().startsWith("http:") || aLink.toLowerCase().startsWith("https:")
                        || aLink.toLowerCase().startsWith("mailto:"))) {
                        if (aLink.toLowerCase().startsWith("javascript:") ||
                            aLink.startsWith("#")) {
                            dealWithJavascriptEvent(aLink, sourcePath, packagePath, pageFrom);
                            returnString = null;
                        }
                        else if (aLink.toLowerCase().startsWith("file:")) {
                            _absoluteLocalLinkFound = true;
                            if(ReloadEditor.DEBUG) System.out.println("Warning: The link '" + aLink.toString()
                                               + "' in the file '"+pageFrom.toString()
                                               + "' is an absolute link to a page on your hard drive.");
                            returnString = null;
                        }
                        else {
                            returnString = aLink;
                        }
                    }
                }
            }
        }
        return returnString;
    }

    /**
     * Routine to to check that the source file is indeed a valid file on the
     * filesystem and then add it to our hashtable if it isnt there already,
     * along with the destination path to copy it to.
     *
     * @param localLink
     * @param sourcePath
     * @param packagePath
     */
    private void buildTheResource(String localLink, String sourcePath, String packagePath, File pageFrom){
        File destinationCanonical=null;
        if (localLink!=null){
           URLDecoder _decoder = new URLDecoder();
           try{
               File original = new File(_decoder.decode(sourcePath + localLink, "UTF-8"));
               File destination = new File(_decoder.decode(packagePath + localLink, "UTF-8"));
               if(original.exists()) {
                   try {
                       // we need to get the canonical path (without ../../) and add that to the table
                       File originalCanonical = new File((String) original.getCanonicalPath());
                       destinationCanonical = new File((String) destination.getCanonicalPath());

                       // Make sure that the link is not referencing a file or folder higher
                       // than the project folder itself. If the new "destination link" starts
                       // with the project folder, then we are okay, otherwise theres a problem
                        if(destinationCanonical.toString().startsWith(_projectFolder.toString())){
                            // Do we have it already?
                            File filePath = (File) _htmlResourcesTable.get(originalCanonical);
                            // No, first time creation, store in table
                            if(filePath == null) {
                                if(!originalCanonical.isDirectory()) {
                                    _htmlResourcesTable.put(originalCanonical, destinationCanonical);
                                }

                                Iterator listElement;
                                listElement = ReloadFileType.EXTENSIONS_TO_PARSE.iterator();
                                while(listElement.hasNext()) {
                                    if(localLink.toLowerCase().endsWith(listElement.next().toString())) {
                                        _firstPage = false;
                                        parseTheHtml(original, (String) destinationCanonical.getParent());
                                    }
                                }
                            }
                        }else{
                            _higherLocalLinkFound=true;
                            if(ReloadEditor.DEBUG) System.out.println("Warning: The link '" + destinationCanonical.toString()
                                               + "' in the file '"+pageFrom.toString()
                                               + "' is higher than the project folder, so it cannot be copied across.");
                        }
                   }
                   catch(IOException e) {
                       if(ReloadEditor.DEBUG) System.out.println("Error: A problem occured during parsing '"+ e.toString() +"'");
                   }
               }
           }catch(UnsupportedEncodingException e){if(ReloadEditor.DEBUG) System.out.println("Error: Encoding exception in HTML document '"+ e.toString() + "'");}
       }
   }

//==============================================================================
//================ Find the text of a given html tag ===========================
//==============================================================================

   /**
    * @return The text value of an HTML tag
    */
   public String getTagText(String tagName) {
       String text = null;

       TagFinderParserCallback callback = new TagFinderParserCallback(tagName);

       try {
           Reader reader = new FileReader(_htmlFile);
           new ParserDelegator().parse(reader, callback, true);
           text = callback.text;
       }
       catch(Exception e) {
           if(ReloadEditor.DEBUG) e.printStackTrace();
       }
       //finally {
           return text;
       //}
   }

   /**
    * Simple ParserCallback to find the text of a given tag
    */
   class TagFinderParserCallback extends HTMLEditorKit.ParserCallback {
       boolean tag_found, done;
       String text = null;
       String _tagName;

       TagFinderParserCallback(String tagName) {
           _tagName = tagName;
       }

       public void handleStartTag(HTML.Tag tag, MutableAttributeSet attrSet, int pos) {
           if(!tag_found && tag.toString().equals(_tagName)) tag_found = true;
       }

       public void handleText(char[] data, int pos) {
           if(tag_found && !done) {
               text = new String(data);
               done = true;
           }
       }
   }

}//  end
