/**
  *  RELOAD Editor - An IMS Metadata, Content Packaging and Learning Design tool.
  *
  *  Copyright (c) 2003 Oleg Liber, Bill Olivier, Phillip Beauvoir, Phil Barker, Tina Manoharan
  *
  *  Permission is hereby granted, free of charge, to any person obtaining a copy
  *  of this software and associated documentation files (the "Software"), to deal
  *  in the Software without restriction, including without limitation the rights
  *  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
  *  copies of the Software, and to permit persons to whom the Software is
  *  furnished to do so, subject to the following conditions:
  *
  *  The above copyright notice and this permission notice shall be included in
  *  all copies or substantial portions of the Software.
  *
  *  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  *  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  *  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
  *  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
  *  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
  *  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
  *
  *  Project Management Contact:
  *
  *  Oleg Liber
  *  Bolton Institute of Higher Education
  *  Deane Road
  *  Bolton BL3 5AB
  *  UK
  *
  *  e-mail:   o.liber@bolton.ac.uk
  *
  *  Phil Barker
  *  ICBL, Heriot-Watt University
  *  Riccarton
  *  Edinburgh EH14 4AS
  *  UK
  *
  *  e-mail:   philb@icbl.hw.ac.uk
  *
  *
  *  Technical Contact:
  *
  *  Phillip Beauvoir
  *  e-mail:   p.beauvoir@bolton.ac.uk
  *
  *  Web:      http://www.reload.ac.uk
  *
  *  Tina Manoharan
  *  e-mail:   ceetm2@macs.hw.ac.uk
  *
  */

package uk.ac.reload.vocabulary;

import java.io.*;
import java.util.*;

import org.jdom.*;
import uk.ac.reload.*;
import uk.ac.reload.dom.*;
import uk.ac.reload.util.*;


/**
 * The Vocabulary Profile
 * This reads in a VX User Profile XML File
 *
 */
public class VocabularyProfile {

    /**
     * The JDOM Document of the XML Helper File
     */
    protected Document _docProfile;

    /**
     * The File of the Profile File
     */
    protected File _fileProfile;

    /**
     * The Root Element
     */
    protected VocabularyProfileElement rootElement;

    // Static Profiles - we only want to create these just once and re-use them
    private static Hashtable reloadProfilesTable = new Hashtable();

    /**
     * Factory method for getting a static re-usable VocabularyProfile
     */
    public static VocabularyProfile getVocabularyProfile(File fileProfile) throws ReloadException {
        if(fileProfile == null) return null;
        // Do we have it already?
        VocabularyProfile vxProfile = (VocabularyProfile)reloadProfilesTable.get(fileProfile);
        // No, first time creation, create and store in table
        if(vxProfile == null) {
            // Does it exist?
            if(fileProfile.exists()) {
                vxProfile = new VocabularyProfile(fileProfile);
                reloadProfilesTable.put(fileProfile, vxProfile);
            }
        }
        return vxProfile;
    }

    /**
     * Get all available VocabularyProfiles
     * @return
     */
    public static VocabularyProfile[] getVocabularyProfiles() {
        // Set up filter to only display xml files
        FileFilter filter = new FileFilter() {
            public boolean accept(File file) {
                if(file.isDirectory()) return false;
                if(ReloadUtils.getFileExtension(file).equals("xml")) return true;
                return false;
            }
        };

        // Get the Files
        File[] files = ReloadDocumentHandler.folderVXProfile.listFiles(filter);
        Vector v = new Vector();
        for(int i = 0; i < files.length; i++) {
            try {

                VocabularyProfile profile = getVocabularyProfile(files[i]);

                v.addElement(profile);
            }
            catch(ReloadException ex) {
            }
        }

        VocabularyProfile[] profiles = new VocabularyProfile[v.size()];
        v.copyInto(profiles);

        return profiles;
    }

    /**
     * Clear the profiles table
     */
    public static void clearTable() {
        reloadProfilesTable.clear();
    }

    /**
     * Constructor
     */
    private VocabularyProfile(File fileProfile) throws ReloadException {
        if(fileProfile == null) throw new ReloadException("Profile file cannot be null");
        _fileProfile = fileProfile;
        // Get the helper JDOM Document
        _docProfile = loadProfile(_fileProfile);

        // Get a Root Element
        rootElement = new VocabularyProfileElement(_docProfile.getRootElement());
    }

    /**
     * Load the XML Profile File into a JDOM Document
     *
     * @param fileProfile The XML File to read in
     * @return The JDOM Document
     * @throws ReloadException
     */
    protected Document loadProfile(File fileProfile) throws ReloadException {
        return XMLUtils.readXMLFile(fileProfile);
    }

    /**
     * Get name of the Profile.  This is stored
     * in the XML Profile file against the attribute "name"
     * @return The name of the Profile
     */
    public String getProfileName() {
        Element root = _docProfile.getRootElement();
        Attribute att = root.getAttribute("name");
        return att == null ? null : att.getValue();
    }

    /**
     * Get the file name of the associated Vocabulary file.  This is stored
     * in the XML Profile file against the attribute "vocabfile"
     * @return The name of the Vocabulary file
     */
    public String getVocabularyFileName() {
        Element root = _docProfile.getRootElement();
        Attribute att = root.getAttribute("vocabfile");
        return att == null ? null : att.getValue();
    }


    /**
     * Get the file name of the associated Schema file.  This is stored
     * in the XML Profile file against the attribute "schemafile"
     * @return The name of the Schema file
     */
    public String getSchemaFileName() {
        Element root = _docProfile.getRootElement();
        Attribute att = root.getAttribute("schemafile");
        return att == null ? null : att.getValue();
    }


    /**
     * Get the name of the associated Schema file.  This is stored
     * in the XML Profile file against the attribute "namespace"
     * @return The name of the Schema
     */
    public String getSchemaNamespace() {
        Element root = _docProfile.getRootElement();
        Attribute att = root.getAttribute("namespace");
        return att == null ? null : att.getValue();
    }

    /**
     * Get the VocabularyProfileField root field
     * @return
     */
    public VocabularyProfileElement getRootElement() {
        return rootElement;
    }

    /**
     * String representation
     * @return
     */
    public String toString() {
        return getProfileName();
    }
}