/**
  *  RELOAD Editor - An IMS Metadata, Content Packaging and Learning Design tool.
  *
  *  Copyright (c) 2003 Oleg Liber, Bill Olivier, Phillip Beauvoir, Phil Barker, Tina Manoharan
  *
  *  Permission is hereby granted, free of charge, to any person obtaining a copy
  *  of this software and associated documentation files (the "Software"), to deal
  *  in the Software without restriction, including without limitation the rights
  *  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
  *  copies of the Software, and to permit persons to whom the Software is
  *  furnished to do so, subject to the following conditions:
  *
  *  The above copyright notice and this permission notice shall be included in
  *  all copies or substantial portions of the Software.
  *
  *  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  *  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  *  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
  *  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
  *  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
  *  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
  *
  *  Project Management Contact:
  *
  *  Oleg Liber
  *  Bolton Institute of Higher Education
  *  Deane Road
  *  Bolton BL3 5AB
  *  UK
  *
  *  e-mail:   o.liber@bolton.ac.uk
  *
  *  Phil Barker
  *  ICBL, Heriot-Watt University
  *  Riccarton
  *  Edinburgh EH14 4AS
  *  UK
  *
  *  e-mail:   philb@icbl.hw.ac.uk
  *
  *
  *  Technical Contact:
  *
  *  Phillip Beauvoir
  *  e-mail:   p.beauvoir@bolton.ac.uk
  *
  *  Web:      http://www.reload.ac.uk
  *
  *  Tina Manoharan
  *  e-mail:   ceetm2@macs.hw.ac.uk
  *
  */

package uk.ac.reload.vocabulary;

import java.io.*;
import java.util.*;

import javax.swing.*;

import uk.ac.reload.*;
import uk.ac.reload.schema.*;
import uk.ac.reload.dom.*;
import org.jdom.*;

/**
 * The Vocabulary Schema Controller - this interfaces from the various helpers
 *
 */
public class VocabularySchemaController
extends ReloadSchemaController
{

    /**
     * The current Vocabulary Profile
     */
    protected VocabularyProfile _vxProfile;

    /**
     * The default language hard-coded for now
     */
    protected String defaultLanguage = Locale.getDefault().getLanguage();

    /**
     * Constructor
     */
    public VocabularySchemaController(File fileSchema, File fileHelper, File fileVocab) {
        super(fileSchema, fileHelper, fileVocab);
    }

    /**
     * Constructor
     */
    public VocabularySchemaController(VocabularyProfile vxProfile) {
        setVocabularyProfile(vxProfile);
    }

    /**
     * @return the Default VocabularySchemaController
     */
    public static VocabularySchemaController getDefaultSchemaController() {
        return (VocabularySchemaController)ReloadDocumentHandler.getReloadSchemaController(ReloadDocumentHandler.IMSVX_LAX_100);
    }


    /**
     * Set the Vocabulary Schema
     * @param vxProfile
     */
    public VocabularySchemaController setVocabularySchema(VocabularyProfile vxProfile) {
        _vxProfile = vxProfile;

        // Set Schema
        File schemaFile = new File(new File(ReloadDocumentHandler.SCHEMAMODELFOLDER, ReloadDocumentHandler.IMSVX_100), vxProfile.getSchemaFileName());

        setSchemaFile(schemaFile, ReloadDocumentHandler.fileHelperVX);

		return VocabularySchemaController.this;
    }


    /**
     * Set the Vocabulary Profile
     * @param vxProfile
     */
    public void setVocabularyProfile(VocabularyProfile vxProfile) {
        _vxProfile = vxProfile;

        // Set VocabFile
        File vocabFile = new File(ReloadDocumentHandler.folderVXVocab, vxProfile.getVocabularyFileName());

        setVocabFile(vocabFile);

        // Default Language
        setDefaultLanguage(defaultLanguage);
    }

    /**
     * Get the Vocabulary Profile
     * @return
     */
    public VocabularyProfile getVocabularyProfile() {
        // If we don't have one, set the default one
        if(_vxProfile == null) {
            try {

                // setVocabularyProfile(VocabularyProfile.getVocabularyProfile(ReloadDocumentHandler.defaultVXProfile));

                setVocabularyProfile(VocabularyProfile.getVocabularyProfile(ReloadDocumentHandler.getVXSchemaProfileFile(_schemaFile.getName())));
            }
            catch(ReloadException ex) {
                ErrorDialogBox.showWarning("Could not load Default Vocabulary Profile", "Load Profile", ex);
            }
        }

        return _vxProfile;
    }

    /**
     * Return the Root Element name
     * @return
     */
    public String getRootElementName() {
        return Vocabulary.ROOT_NAME;
    }

    /**
     * @return true if nameSpaceURI is in our list of accepted Namespace URIs
     */
    public static boolean isAcceptedNamespaceURI(String nameSpaceURI) {
        for(int i = 0; i < ReloadDocumentHandler.IMSVX_NAMESPACE_URIS.length; i++) {
            if(nameSpaceURI.equals(ReloadDocumentHandler.IMSVX_NAMESPACE_URIS[i])) {
                return true;
            }
        }
        return false;
    }

    /**
     * Get the (4 in this case) top level VDEX Element Categories
     * @return An Enumeration of ReloadSchemaElement objects
     */
    public ReloadSchemaElement[] getTopLevelElements() {
        return _reloadSchema.getRootElement().getChildren();
    }

    /**
     * Find out if the given Element is a top level one below "vdex"
     * @param schemaElement The Schema Element to query
     * @return true if the parent of schemaElement is the root Element
     */
    public boolean isTopLevelElement(ReloadSchemaElement schemaElement) {
        return schemaElement.getParent() == _reloadSchema.getRootElement();
    }

    /**
     * Get The Leaf Icon
     */
    public Icon getLeafIcon(Element element) {
        String elementName = element.getName();
        if(elementName.equals(Vocabulary.ROOT_NAME)) return ReloadUIManager.getIcon(ReloadUIManager.ICON_VX);
        else return ReloadUIManager.getIcon(ReloadUIManager.ICON_VXLEAF);
    }

    /**
     * Get The Leaf Icon
     */
    public Icon getLeafIcon(ReloadSchemaElement schemaElement) {
        String elementName = schemaElement.getName();
        if(elementName.equals(Vocabulary.ROOT_NAME)) return ReloadUIManager.getIcon(ReloadUIManager.ICON_VX);
        else return ReloadUIManager.getIcon(ReloadUIManager.ICON_VXLEAF);
    }

    /**
     * Get The Open Icon
     */
    public Icon getOpenIcon(Element element) {
        String elementName = element.getName();
        if(elementName.equals(Vocabulary.ROOT_NAME)) return ReloadUIManager.getIcon(ReloadUIManager.ICON_VX);
        else return ReloadUIManager.getIcon(ReloadUIManager.ICON_VXFOLDER);
    }

    /**
     * Get The Open Icon
     */
    public Icon getOpenIcon(ReloadSchemaElement schemaElement) {
        String elementName = schemaElement.getName();
        if(elementName.equals(Vocabulary.ROOT_NAME)) return ReloadUIManager.getIcon(ReloadUIManager.ICON_VX);
        else return ReloadUIManager.getIcon(ReloadUIManager.ICON_VXFOLDER);
    }

    /**
     * Get The Closed Icon
     */
    public Icon getClosedIcon(Element element) {
        return getOpenIcon(element);
    }

    /**
     * Get The Closed Icon
     */
    public Icon getClosedIcon(ReloadSchemaElement schemaElement) {
        return getLeafIcon(schemaElement);
    }
}