/**
  *  RELOAD Editor - An IMS Metadata, Content Packaging and Learning Design tool.
  *
  *  Copyright (c) 2003 Oleg Liber, Bill Olivier, Phillip Beauvoir, Phil Barker, Tina Manoharan
  *
  *  Permission is hereby granted, free of charge, to any person obtaining a copy
  *  of this software and associated documentation files (the "Software"), to deal
  *  in the Software without restriction, including without limitation the rights
  *  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
  *  copies of the Software, and to permit persons to whom the Software is
  *  furnished to do so, subject to the following conditions:
  *
  *  The above copyright notice and this permission notice shall be included in
  *  all copies or substantial portions of the Software.
  *
  *  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  *  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  *  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
  *  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
  *  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
  *  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
  *
  *  Project Management Contact:
  *
  *  Oleg Liber
  *  Bolton Institute of Higher Education
  *  Deane Road
  *  Bolton BL3 5AB
  *  UK
  *
  *  e-mail:   o.liber@bolton.ac.uk
  *
  *  Phil Barker
  *  ICBL, Heriot-Watt University
  *  Riccarton
  *  Edinburgh EH14 4AS
  *  UK
  *
  *  e-mail:   philb@icbl.hw.ac.uk
  *
  *
  *  Technical Contact:
  *
  *  Phillip Beauvoir
  *  e-mail:   p.beauvoir@bolton.ac.uk
  *
  *  Web:      http://www.reload.ac.uk
  *
  *  Tina Manoharan
  *  e-mail:   ceetm2@macs.hw.ac.uk
  *
  */

package uk.ac.reload.vocabulary.tableview;

import java.util.*;

import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.table.*;
import javax.swing.tree.*;

import org.jdom.*;
import uk.ac.reload.dom.*;
import uk.ac.reload.menu.*;
import uk.ac.reload.vocabulary.*;
import uk.ac.reload.schema.*;
import uk.ac.reload.widgets.*;
import uk.ac.reload.widgets.treetable.*;

/**
 * The Vocabulary Tree Table to display the Vocabulary
 *
 */
public class VocabularyTreeTable
extends ReloadTreeTable
implements TreeSelectionListener, ReloadDocumentListener, FocusListener
{
    /**
     * The Element Value/Attribute Edit Panel
     */
    protected ReloadInfoPanel _infoPanel;

    /**
     * Popup Menu on Right-clicks
     */
    protected JPopupMenu _popupMenu;

    /**
     * The Vocabulary Document
     */
    protected Vocabulary _vocabulary;

    /**
     * The Schema Controller
     */
    protected ReloadSchemaController _schemaController;

    /**
     * The Edit menu which we will use
     */
    protected Menu_Edit _editMenu;

    /**
     * Keeping note of the current selected tree node means we don't have to do
     * unneccesary updating
     */
    protected TreePath _currentTreePath;

    /**
     * The Proxy Delete Handler
     */
    protected ProxyDeleteHandler _deleteHandler;

    /**
     * The Proxy Cut Handler
     */
    protected ProxyCutHandler _cutHandler;

    /**
     * The Proxy Copy Handler
     */
    protected ProxyCopyHandler _copyHandler;

    /**
     * The Proxy Paste Handler
     */
    protected ProxyPasteHandler _pasteHandler;

    /**
     * The Proxy Move Up Handler
     */
    protected ProxyMoveUpHandler _moveUpHandler;

    /**
     * The Proxy Move Down Handler
     */
    protected ProxyMoveDownHandler _moveDownHandler;


    /**
     * Constructor
     * @param treeTableModel The TreeTabelModel
     */
    public VocabularyTreeTable(VocabularyTreeTableModel treeTableModel, ReloadInfoPanel infoPanel, Menu_Edit editMenu) {
        super(treeTableModel);

        _infoPanel = infoPanel;

        // DOM Document
        _vocabulary = treeTableModel.getVocabulary();

        // Add ourselves as Element Listener
        _vocabulary.addReloadDocumentListener(this);

        _schemaController = _vocabulary.getReloadSchemaController();

        // Set up the Table
        getTableHeader().setReorderingAllowed(false);

        // Columns
        TableColumnModel tcm = getTableHeader().getColumnModel();
        tcm.getColumn(0).setPreferredWidth(100);
        tcm.getColumn(1).setPreferredWidth(300);
        tcm.getColumn(2).setPreferredWidth(80);
        //tcm.getColumn(2).setMaxWidth(80);

        setShowVerticalLines(true);
        setSelectionMode(ListSelectionModel.SINGLE_SELECTION);

        // Make sure we have a renderer for the Tree
        tree.setCellRenderer(new VocabularyTreeRenderer());

        // We listen for tree selections
        tree.addTreeSelectionListener(this);

        // Popup menu
        _popupMenu = new JPopupMenu();
        // Workaround - when a popupmenu is invoked, focus (temporary type) is lost
        // to the popup.  If user then moves to another window, focus is not lost
        // properly and therefore menu items are not cleared
        //popupMenu.setFocusable(false);

        // Listen to mouse-clicks for the Popup Menu
        addMouseListener(new MouseAdapter() {
           // PC
           public void mouseReleased(MouseEvent e) {
               checkPopupTrigger(e);
           }

           // Mac
           public void mousePressed(MouseEvent e) {
               checkPopupTrigger(e);
           }
        });

        // Focus Listener
        addFocusListener(this);

        // Get the Edit menu which we will use
        _editMenu = editMenu;

        // Remap key bindings
        _editMenu.remapKeyStrokes(this);
    }

    /**
     * We got the focus
     */
    public void focusGained(FocusEvent e) {
        //System.out.println(e.paramString());
        if(!e.isTemporary()) {
            updateMenus(tree.getSelectionPath());
        }
    }

    /**
     * We lost the focus
     */
    public void focusLost(FocusEvent e) {
        //System.out.println(e.paramString());
        if(!e.isTemporary()) {
            clearMenus();
        }
    }

    /**
     * Clean up
     */
    public void destroy() {
        _vocabulary.removeReloadDocumentListener(this);
        clearMenus();
    }

    /**
     * Refresh the view
     */
    public void refresh() {
		updateInfoPanel(tree.getSelectionPath());
    }

    /**
     * A node has been selected on the tree.
     * @param event The TreeSelectionEvent fired from us being a TreeSelectionListener
     */
    public void valueChanged(TreeSelectionEvent event) {
       TreePath selPath = event.getPath();
       if(selPath != _currentTreePath) {
           updateMenus(selPath);
           updateInfoPanel(selPath);
           _currentTreePath = selPath;
       }
   }

    /**
     * Update the Info Panel according to the selected node on the tree.
     * @param selPath The TreePath Element that is selected on the Tree.
     */
    protected synchronized void updateInfoPanel(TreePath selPath) {
        if(_infoPanel != null) {
            if(selPath == null) {
                _infoPanel.clear();
                return;
            }

            Object selectedNode = selPath.getLastPathComponent();
            if(selectedNode instanceof VocabularyTreeNode) {
                // Cast it
                VocabularyTreeNode treeNode = (VocabularyTreeNode) selectedNode;

                // And get the ReloadElementBinding
                ReloadElementBinding eb = treeNode.createReloadElementBinding();

                // Tell the panel
                _infoPanel.setElement(eb);
            }
        }
    }

    /**
     * Update the menus according to the selected node on the tree.
     * This has to be synchronized so different threads don't access it at the same time
     * @param element The JDOM Element that is selected on the Tree.
     * @param schemaElement The ReloadSchemaElement that is selected on the Tree.
     */
    protected synchronized void updateMenus(TreePath selPath) {
        // Clear the deck regardless
        clearMenus();

        if(selPath == null) return;

        Object selectedNode = selPath.getLastPathComponent();
        if(selectedNode instanceof VocabularyTreeNode) {
            // Cast it
            VocabularyTreeNode treeNode = (VocabularyTreeNode) selectedNode;

            // And get our bits
            Element element = treeNode.getElement();
            ReloadSchemaElement schemaElement = treeNode.getSchemaElement();

            // New Cut Handler
            _cutHandler = new ProxyCutHandler(_editMenu.actionCut, element, schemaElement);
            // New Copy Handler
            _copyHandler = new ProxyCopyHandler(_editMenu.actionCopy, element);
            // New Paste Handler
            _pasteHandler = new ProxyPasteHandler(_editMenu.actionPaste, element);
            // New Delete Handler
            _deleteHandler = new ProxyDeleteHandler(_editMenu.actionDelete, element, schemaElement);

            // Add to popup
            _popupMenu.add(_cutHandler.getMenuAction());
            _popupMenu.add(_copyHandler.getMenuAction());
            _popupMenu.add(_pasteHandler.getMenuAction());
            _popupMenu.add(_deleteHandler.getMenuAction());

            _popupMenu.addSeparator();

            // New Move Up Handler
            _moveUpHandler = new ProxyMoveUpHandler(_editMenu.actionMoveUp, element);
            _popupMenu.add(_moveUpHandler.getMenuAction());

            // New Move Down Handler
            _moveDownHandler = new ProxyMoveDownHandler(_editMenu.actionMoveDown, element);
            _popupMenu.add(_moveDownHandler.getMenuAction());


            // If the Schema Element has Children display them in the menu
            if(schemaElement != null && schemaElement.hasChildren()) {
                // Separator
                _editMenu.addAdditionalItem(new JPopupMenu.Separator());
                _popupMenu.addSeparator();

                ReloadSchemaElement[] children = schemaElement.getChildren();
                for(int i = 0; i < children.length; i++) {
                    ReloadSchemaElement childSchemaElement = children[i];
                    if(((VocabularyTreeTableModel)getTreeTableModel()).canAddNode(childSchemaElement)) {
                        Action_AddChildElement actionAddElement = new Action_AddChildElement(element, childSchemaElement);

                        _popupMenu.add(actionAddElement);
                        _editMenu.addAdditionalItem(actionAddElement);
                    }
                }
            }
        }
    }

    /**
     * Clear any popup or other menus
     */
    protected synchronized void clearMenus() {
        // We can remove all from the popup menu
        _popupMenu.removeAll();

        // Remove any menu items we created
        _editMenu.removeAdditionalItems();

        // Clear the Handlers
        if(_cutHandler != null) _cutHandler.clearMenu();
        if(_copyHandler != null) _copyHandler.clearMenu();
        if(_pasteHandler != null) _pasteHandler.clearMenu();
        if(_deleteHandler != null) _deleteHandler.clearMenu();
        if(_moveUpHandler != null) _moveUpHandler.clearMenu();
        if(_moveDownHandler != null) _moveDownHandler.clearMenu();
    }

    /**
     * Check to see if we have triggered the popup menu.
     * @param e The MouseEvent that has been triggered.
     */
    protected void checkPopupTrigger(MouseEvent e) {
       if(e.isPopupTrigger()) {
           TreePath selPath = tree.getPathForLocation(e.getX(), e.getY());
           if(selPath == null) return;
           // Select the node
           tree.setSelectionPath(selPath);
           // Show the popup menu
           _popupMenu.show(this, e.getX(), e.getY());
       }
    }


//////////////////////// ELEMENT LISTENER //////////////////////////////////////

    public void elementAdded(ReloadDocumentListenerEvent event) {
        Element element = event.getElement();

        // Get Parent node
        Element parent = element.getParent();
        DefaultMutableTreeNode parentNode = getNode(parent);

        if(parentNode != null) {
            VocabularyTreeNode newNode = addNode(parentNode, element);
            if(newNode != null && event.doSelect()) selectNode(newNode);
        }
    }

    /**
     * Add an Element
     */
    private VocabularyTreeNode addNode(DefaultMutableTreeNode parentNode, Element element) {
        VocabularyTreeNode newNode = null;

        // Ascertain correct position to insert
        int index = getInsertNodePosition(parentNode, element);

        // If not allowed to insert, don't bother
        if(index != -1) {
            // New node
            newNode = new VocabularyTreeNode(element);
            // Insert into model
            getTreeTableModel().insertNodeInto(newNode, parentNode, index);

            // Children
            Iterator it = element.getChildren().iterator();
            while(it.hasNext()) {
                Element child = (Element) it.next();
                addNode(newNode, child);
            }
        }

        return newNode;
    }

    /**
     * @return where to correctly insert the node for an Element.
     * Some nodes might not be visible and therefore the node to Elements count is wrong
     * Return -1 if we shouldn't insert it
     */
    protected int getInsertNodePosition(DefaultMutableTreeNode parentNode, Element element) {
        // Don't display certain nodes
        if(((VocabularyTreeTableModel)getTreeTableModel()).doShowNode(element) == false) return -1;

        int index = 0;

        // Because we might not display certain nodes we have to compensate
        java.util.List children = element.getParent().getChildren();
        for(int i = 0; i < children.size(); i++) {
            Element child = (Element) children.get(i);
            if(child == element) break;
            if(((VocabularyTreeTableModel) getTreeTableModel()).doShowNode(child)) index++;
        }

        // Make sure we don't exceed the child count
        if(index > parentNode.getChildCount()) index = parentNode.getChildCount();
        if(index < 0) index = 0;

        return index;
    }

    /**
     * An Element was removed from the Document so we must find it tree node and
     * remove that from the Tree.
     */
    public void elementRemoved(ReloadDocumentListenerEvent event) {
        // Find the node
        Element element = event.getElement();

        DefaultMutableTreeNode nodeToDelete = getNode(element);
        if(nodeToDelete != null) {
            // The currently selected node is a descendent of/same as the node being deleted
            if(nodeToDelete.isNodeDescendant(getSelectedNode()))  {
                // Save sibling/parent
                DefaultMutableTreeNode prevNode = nodeToDelete.getPreviousSibling();
                if(prevNode == null) prevNode = (DefaultMutableTreeNode)nodeToDelete.getParent();

                // Delete node
                getTreeTableModel().removeNodeFromParent(nodeToDelete);
                // Select previous node this will take care of updating menus
                selectNode(prevNode);
            }

            // Else we were sat on another node and need to update menus
            else {
                // Delete node
                getTreeTableModel().removeNodeFromParent(nodeToDelete);
                // Update Menus
                updateMenus(tree.getSelectionPath());
            }
        }
    }


    /**
     * An Element was changed in some way (maybe the text or Attribute) so we
     * inform the Tree Table Model
     * @param element The Element that changed.
     */
    public void elementChanged(ReloadDocumentListenerEvent event) {
        //DefaultMutableTreeNode node = getNode(event.getElement());
        //if(node != null) getTreeTableModel().nodeChanged(node);

        final DefaultMutableTreeNode node = getNode(event.getElement());
        if(node != null)
            SwingUtilities.invokeLater(new Runnable() {
                public void run() {
                    getTreeTableModel().nodeChanged(node);
                }
            });
    }

    public void documentSaved(ReloadDocument doc) {}

//==============================================================================
//       ADD HANDLER
//==============================================================================

    /**
     * An Action Class that allows us to add a new Child Element to a Parent Element
     * There will be some validity checks.
     */
    class Action_AddChildElement extends AbstractAction {
        Element parentElement;
        ReloadSchemaElement childSchemaElement;
        String name;

        public Action_AddChildElement(Element parentElement, ReloadSchemaElement childSchemaElement) {
            this.parentElement = parentElement;
            this.childSchemaElement = childSchemaElement;

            // Set up name that will be displayed on the Menu
            name = _schemaController.getElementFriendlyName(childSchemaElement.getDomPath());
            if(name == null) name = childSchemaElement.getName();
            putValue(Action.NAME, "Add " + name);

            putValue(Action.SMALL_ICON, _schemaController.getLeafIcon(childSchemaElement));

            // Set Enabled according to whether we are allowed to add this child
            setEnabled(_vocabulary.canAddElement(parentElement, childSchemaElement));
        }

        public void actionPerformed(ActionEvent e) {
            _vocabulary.addElementBySchemaUndoable(VocabularyTreeTable.this, parentElement, childSchemaElement, true);
        }

    }


//==============================================================================
//       PROXY ACTION HANDLERS
//==============================================================================

    /**
     * Move Up Element Proxy Action
     */
    class ProxyMoveUpHandler extends ProxyAction {
        Element element;

        public ProxyMoveUpHandler(MenuAction proxyMenuAction, Element element) {
            super(proxyMenuAction);
            this.element = element;
            // Set Enabled according to whether we are allowed to move this up
            proxyMenuAction.setEnabled(_vocabulary.canMoveElementUp(element));
        }

        public void menuActionPerformed(ReloadMenuEvent event) {
            _vocabulary.moveElementUp(VocabularyTreeTable.this, element, true);
        }

        public void clearMenu() {
            removeAllListeners();
            setEnabled(false);
        }
    }

    /**
     * Move Down Element Proxy Action
     */
    class ProxyMoveDownHandler extends ProxyAction {
        Element element;

        public ProxyMoveDownHandler(MenuAction proxyMenuAction, Element element) {
            super(proxyMenuAction);
            this.element = element;
            // Set Enabled according to whether we are allowed to move this down
            proxyMenuAction.setEnabled(_vocabulary.canMoveElementDown(element));
        }

        public void menuActionPerformed(ReloadMenuEvent event) {
            _vocabulary.moveElementDown(VocabularyTreeTable.this, element, true);
        }

        public void clearMenu() {
            removeAllListeners();
            setEnabled(false);
        }
    }

    /**
     * Delete Element Proxy Action
     */
    class ProxyDeleteHandler extends ProxyAction {
        Element parent;
        Element element;

        public ProxyDeleteHandler(MenuAction proxyMenuAction, Element element, ReloadSchemaElement schemaElement) {
            super(proxyMenuAction);
            this.element = element;
            // Set Enabled according to whether we are allowed to delete this element
            proxyMenuAction.setEnabled(_vocabulary.canDeleteElement(element, schemaElement));
        }

        public void menuActionPerformed(ReloadMenuEvent event) {
            _vocabulary.deleteElementUndoable(VocabularyTreeTable.this, element);
        }

        public void clearMenu() {
            removeAllListeners();
            setEnabled(false);
        }
    }

    /**
     * Cut Element Proxy Action
     */
    class ProxyCutHandler extends ProxyAction {
        Element element;

        public ProxyCutHandler(MenuAction proxyMenuAction, Element element, ReloadSchemaElement schemaElement) {
            super(proxyMenuAction);
            this.element = element;
            // Set Enabled according to whether we are allowed to cut this element
            proxyMenuAction.setEnabled(_vocabulary.canCutElement(element, schemaElement));
        }

        public void menuActionPerformed(ReloadMenuEvent event) {
            _vocabulary.cutElementUndoable(VocabularyTreeTable.this, element);
            ReloadDocumentClipboard.addCutElement(element);
        }

        public void clearMenu() {
            removeAllListeners();
            setEnabled(false);
        }
    }

    /**
     * Copy Element Proxy Action
     */
    class ProxyCopyHandler extends ProxyAction {
        Element element;

        public ProxyCopyHandler(MenuAction proxyMenuAction, Element element) {
            super(proxyMenuAction);
            this.element = element;
            // Set Enabled according to whether we are allowed to copy this element
            proxyMenuAction.setEnabled(_vocabulary.canCopyElement(element));
        }

        public void menuActionPerformed(ReloadMenuEvent event) {
            ReloadDocumentClipboard.addCopiedElement(element);
            // Update Menus
            updateMenus(tree.getSelectionPath());
        }

        public void clearMenu() {
            removeAllListeners();
            setEnabled(false);
        }
    }

    /**
     * Paste Element Proxy Action
     */
    class ProxyPasteHandler extends ProxyAction {
        Element element;

        public ProxyPasteHandler(MenuAction proxyMenuAction, Element element) {
            super(proxyMenuAction);
            this.element = element;
            // Set Enabled according to whether we are allowed to paste this element
            proxyMenuAction.setEnabled(_vocabulary.canPasteFromClipboard(element));
        }

        public void menuActionPerformed(ReloadMenuEvent event) {
            if(_vocabulary.canPasteFromClipboard(element)) {
                Element clipBoardElement = ReloadDocumentClipboard.getElement();
                _vocabulary.pasteElementUndoable(VocabularyTreeTable.this, clipBoardElement, element, false);
                // Update Menus
                updateMenus(tree.getSelectionPath());
            }
        }

        public void clearMenu() {
            removeAllListeners();
            setEnabled(false);
        }
    }
}
