/**
  *  RELOAD Editor - An IMS Metadata, Content Packaging and Learning Design tool.
  *
  *  Copyright (c) 2003 Oleg Liber, Bill Olivier, Phillip Beauvoir, Phil Barker, Tina Manoharan
  *
  *  Permission is hereby granted, free of charge, to any person obtaining a copy
  *  of this software and associated documentation files (the "Software"), to deal
  *  in the Software without restriction, including without limitation the rights
  *  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
  *  copies of the Software, and to permit persons to whom the Software is
  *  furnished to do so, subject to the following conditions:
  *
  *  The above copyright notice and this permission notice shall be included in
  *  all copies or substantial portions of the Software.
  *
  *  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  *  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  *  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
  *  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
  *  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
  *  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
  *
  *  Project Management Contact:
  *
  *  Oleg Liber
  *  Bolton Institute of Higher Education
  *  Deane Road
  *  Bolton BL3 5AB
  *  UK
  *
  *  e-mail:   o.liber@bolton.ac.uk
  *
  *  Phil Barker
  *  ICBL, Heriot-Watt University
  *  Riccarton
  *  Edinburgh EH14 4AS
  *  UK
  *
  *  e-mail:   philb@icbl.hw.ac.uk
  *
  *
  *  Technical Contact:
  *
  *  Phillip Beauvoir
  *  e-mail:   p.beauvoir@bolton.ac.uk
  *
  *  Web:      http://www.reload.ac.uk
  *
  *  Tina Manoharan
  *  e-mail:   ceetm2@macs.hw.ac.uk
  *
  */

package uk.ac.reload.vocabulary.tableview;

import java.util.*;

import org.jdom.*;
import uk.ac.reload.vocabulary.*;
import uk.ac.reload.widgets.treetable.*;
import uk.ac.reload.schema.*;

/**
 * The Vocabulary Tree Table Model for the VocabularyTreeTable
 *
 */
public class VocabularyTreeTableModel
extends ReloadTreeTableModel {

    /**
     * The Class types for the Columns - this is important because the tree
     * won't display properly without it.
     */
    static protected Class[] cTypes = { TreeTableModel.class, String.class, String.class};

    /**
     * The Column Names
     */
    static protected String[]  cNames = {"Name", "Value", "Attr" };

    /**
     * The Vocabulary DOM Document
     */
    protected Vocabulary _vocabulary;

    /**
     * Constructor
     * @param vocabulary The Vocabulary DOM Document
     */
    public VocabularyTreeTableModel(Vocabulary vocabulary) {
        _vocabulary = vocabulary;
        // Create the root node
        root = new VocabularyTreeNode(vocabulary);
        // Build the child nodes
        buildChildren((VocabularyTreeNode)root);
    }

    /**
     * Build up child nodes
     * @param node The Parent Node
     */
    public void buildChildren(VocabularyTreeNode node) {
        Iterator children = node.getElement().getChildren().iterator();
        while(children.hasNext()) {
            Element child = (Element)children.next();
            if(doShowNode(child)) {
                VocabularyTreeNode newNode = new VocabularyTreeNode(child);
                node.add(newNode);
                buildChildren(newNode);
            }
        }
    }


    /**
     * @return Whether to display an Element on the Tree
     */
    protected boolean doShowNode(Element element) {
        return doShowNode(element.getName(), element.getNamespace());
    }

    /**
     * @return Whether to display an Element on the Tree
     */
    protected boolean doShowNode(ReloadSchemaElement schemaElement) {
        return doShowNode(schemaElement.getName(), schemaElement.getNamespace());
    }

    /**
     * @return Whether to display an Element on the Tree
     */
    protected boolean doShowNode(String elementName, Namespace ns) {
        return true;
    }

    /**
     * @return Whether to allow to add an Element on the Tree
     */
    protected boolean canAddNode(Element element) {
        return canAddNode(element.getName(), element.getNamespace());
    }

    /**
     * @return Whether to allow to add an Element on the Tree
     */
    protected boolean canAddNode(ReloadSchemaElement schemaElement) {
        return canAddNode(schemaElement.getName(), schemaElement.getNamespace());
    }

    /**
     * @return Whether to allow to add an Element on the Tree
     */
    protected boolean canAddNode(String elementName, Namespace ns) {
        return doShowNode(elementName, ns);
    }

    /**
     * @return the Vocabulary
     */
    public Vocabulary getVocabulary() {
        return _vocabulary;
    }

    /**
     * Get the Child count in the Model
     * @param parent The parent node
     * @return The Number of children of parent
     */
    public int getChildCount(Object parent) {
        VocabularyTreeNode elementNode = (VocabularyTreeNode) parent;
        return elementNode.getChildCount();
    }


    /**
     * Get the child node at index point
     * @param parent The parent node
     * @param index The index of the child
     * @return The Child node at the index point
     */
    public Object getChild(Object parent, int index) {
        VocabularyTreeNode elementNode = (VocabularyTreeNode) parent;
        return elementNode.getChildAt(index);
    }

    /**
     * Set the value of a node
     * @param aValue The value to set
     * @param node The node whose value will be set
     * @param column The column of the node
     */
    public void setValueAt(Object aValue, Object node, int column) {

    }

    /**
     * Get the value for a given node and column.
     * This asks for the value at column 1 or greater, column 0 is the JTree
     * @param node The Node to get the value for
     * @param column The column
     * @return The appropriate value.
     */
    public Object getValueAt(Object node, int column) {
        VocabularyTreeNode elementNode = (VocabularyTreeNode)node;
        switch(column) {
            case 1:
                // Return the Text of the JDOM Element
                return elementNode.getElement().getText();
            case 2:
                // Return the Attribute value, if any
                List list = elementNode.getElement().getAttributes();
                if(!list.isEmpty()) {
                    Attribute att = (Attribute)list.get(0);
                    return att.getValue();
                }
                return "";
        }
        return "";
    }

    /**
     * Determine whether a particular cell is editable.
     * @param node The node we are asking.
     * @param column The column to query
     * @return
     */
    public boolean isCellEditable(Object node, int column) {
        VocabularyTreeNode elementNode = (VocabularyTreeNode)node;
        switch(column) {
            // Column 0 (the JTree) has to be true so that the TreeTable expands and contracts
            case 0:
                return true;
            case 1:
                return false;
            case 2:
                return false;
            default:
                return false;
        }
    }

    /**
     * Return the type of class for each column.
     * Column 0 has to be of type TreeTableModel.class in order for it to work
     * @param column The column to query
     * @return The class assocaited with each column
     */
    public Class getColumnClass(int column) {
        return cTypes[column];
    }

    /**
     * Get the number of columns in our model.
     * @return The number of columns in our model.
     */
    public int getColumnCount() {
        return cNames.length;
    }

    /**
     * Get the name for a particular column.
     * @param column The column to ask.
     * @return The name of the column.
     */
    public String getColumnName(int column) {
        return cNames[column];
    }
}

