/**
 *  RELOAD Editor - An IMS Metadata, Content Packaging and Learning Design tool.
 *
 *  Copyright (c) 2003 Oleg Liber, Bill Olivier, Phillip Beauvoir
 *
 *  Permission is hereby granted, free of charge, to any person obtaining a copy
 *  of this software and associated documentation files (the "Software"), to deal
 *  in the Software without restriction, including without limitation the rights
 *  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 *  copies of the Software, and to permit persons to whom the Software is
 *  furnished to do so, subject to the following conditions:
 *
 *  The above copyright notice and this permission notice shall be included in
 *  all copies or substantial portions of the Software.
 *
 *  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 *  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 *  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 *  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 *  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 *  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 *
 *  Project Management Contact:
 *
 *  Oleg Liber
 *  Bolton Institute of Higher Education
 *  Deane Road
 *  Bolton BL3 5AB
 *  UK
 *
 *  e-mail:   o.liber@bolton.ac.uk
 *
 *
 *  Technical Contact:
 *
 *  Phillip Beauvoir
 *  e-mail:   p.beauvoir@bolton.ac.uk
 *
 *  Web:      http://www.reload.ac.uk
 *
 */

package uk.ac.reload.widgets;

import java.awt.*;
import javax.swing.*;
import javax.swing.event.*;

import org.jdom.*;
import uk.ac.reload.dom.*;
import uk.ac.reload.schema.*;


/**
 * A Panel to display and edit a JDOM Element
 *
 * @author Phillip Beauvoir
 * @version 2003-08-01
 */
public class ReloadElementEditor
extends JPanel
implements ReloadDocumentListener, DocumentListener
{
    /**
     * The ElementBinding we are editing
     */
    protected ReloadElementBinding _elementBinding;

    /**
     * The widget
     */
    protected ReloadWidget _reloadWidget;

    /**
     * Padding
     */
    protected JPanel padPanel;

    /**
     * Whether we notify or not
     */
    protected boolean allowNotification = true;

    /**
     * Constructor
     */
    public ReloadElementEditor() {
        setLayout(new BorderLayout());
        setBackground(Color.lightGray);
        padPanel = new JPanel();
    }

    /**
     * Set The Panel to display the data for the Element
     */
    public void setElement(ReloadElementBinding elementBinding) {
        // Remove old Listener
        if(_elementBinding != null) _elementBinding.getReloadDocument().removeReloadDocumentListener(this);

        _elementBinding = elementBinding;

        // Add Listener
        _elementBinding.getReloadDocument().addReloadDocumentListener(this);

        // Set it up
        setupWidget(_elementBinding.getReloadDocument(), _elementBinding.getElement(), _elementBinding.getSchemaElement());
    }

    /**
     * Set The Widget to display the data for the Element
     */
    protected void setupWidget(ReloadDocument doc, Element element, ReloadSchemaElement schemaElement) {
        if(_reloadWidget != null) _reloadWidget.removeDocumentListener(this);

        // Remove all components from the panel
        removeAll();

        // If this Element is merely a container do not show
        boolean enabled = schemaElement == null ? true : schemaElement.isValue();

		/**
		 * This is a patch for Vdex Metadata Element.
		 *
		 * Tina Manoharan
		 * 2003-09-02
		*/
		if(schemaElement != null) {

			if(schemaElement.getName().startsWith("metadata")) enabled = true;
		}

        if(enabled) {

            // Get and display the Widget
            _reloadWidget = ReloadWidgetFactory.getWidget(doc, doc.getReloadSchemaController(), schemaElement);

            if(_reloadWidget.isSingleLine()) {

                add((Component)_reloadWidget, BorderLayout.NORTH);
                add(padPanel, BorderLayout.CENTER);
            }
            else if(_reloadWidget instanceof ReloadTextPane) {

                JScrollPane scrollPane = new JScrollPane((Component) _reloadWidget);
                add(scrollPane, BorderLayout.CENTER);
            }
            else {

                add((Component) _reloadWidget, BorderLayout.CENTER);
            }

            // Set the text value
            setText(element);

            // Listener
            _reloadWidget.addDocumentListener(this);
        }
    }

    /**
     * Set the Text in the widget to that in the Element
     */
    protected void setText(Element element) {
        allowNotification = false;
        if(_reloadWidget != null) _reloadWidget.setTextValue(element == null ? "" : element.getText());
        allowNotification = true;
    }

    /**
     * Insert text in the widget
     */
    public void insertText(String text) {
        if(_reloadWidget != null) _reloadWidget.insertTextValue(text);
    }

    /**
     * Set the focus
     */
    public void setFocus() {
        if(_reloadWidget != null) ((Component)_reloadWidget).requestFocus();
    }

    /**
     * Clean up
     */
    public void destroy() {
        if(_elementBinding != null) _elementBinding.getReloadDocument().removeReloadDocumentListener(this);
        if(_reloadWidget != null) _reloadWidget.removeDocumentListener(this);
    }

    /**
     * Update the Model
     */
    protected synchronized void fireElementChanged(String text) {
        Element element = _elementBinding.getElement();

        // Element was null so create new one
        if(element == null) element = _elementBinding.createElement(this);

        if(element != null) {
            element.setText(text);
            _elementBinding.getReloadDocument().changedElement(this, element);
        }
    }

// =============================================================================
// ===================== These events are received FROM the ReloadDocument =====
// =============================================================================

    public void elementChanged(ReloadDocumentListenerEvent event) {
        // Don't listen to events that we have fired
        if(event.getSource() == this) return;

        // If it's us update
        Element element = _elementBinding.getElement();
        if(element != null) {
            Element changed_element = event.getElement();
            if(element == changed_element) setText(element);
        }
    }

    public void elementRemoved(ReloadDocumentListenerEvent event) {}
    public void elementAdded(ReloadDocumentListenerEvent event) {}
    public void documentSaved(ReloadDocument doc) {}


//==============================================================================
// DocumentListener for changes in the Widget
//==============================================================================

    public void removeUpdate(DocumentEvent evt) {
        if(allowNotification && _reloadWidget != null) fireElementChanged(_reloadWidget.getTextValue());
    }

    public void changedUpdate(DocumentEvent evt) {
        if(allowNotification && _reloadWidget != null) fireElementChanged(_reloadWidget.getTextValue());
    }

    public void insertUpdate(DocumentEvent evt) {
        if(allowNotification && _reloadWidget != null) fireElementChanged(_reloadWidget.getTextValue());
    }
}
