/**
 *  RELOAD Editor - An IMS Metadata, Content Packaging and Learning Design tool.
 *
 *  Copyright (c) 2003 Oleg Liber, Bill Olivier, Phillip Beauvoir
 *
 *  Permission is hereby granted, free of charge, to any person obtaining a copy
 *  of this software and associated documentation files (the "Software"), to deal
 *  in the Software without restriction, including without limitation the rights
 *  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 *  copies of the Software, and to permit persons to whom the Software is
 *  furnished to do so, subject to the following conditions:
 *
 *  The above copyright notice and this permission notice shall be included in
 *  all copies or substantial portions of the Software.
 *
 *  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 *  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 *  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 *  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 *  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 *  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 *
 *  Project Management Contact:
 *
 *  Oleg Liber
 *  Bolton Institute of Higher Education
 *  Deane Road
 *  Bolton BL3 5AB
 *  UK
 *
 *  e-mail:   o.liber@bolton.ac.uk
 *
 *
 *  Technical Contact:
 *
 *  Phillip Beauvoir
 *  e-mail:   p.beauvoir@bolton.ac.uk
 *
 *  Web:      http://www.reload.ac.uk
 *
 */

package uk.ac.reload;

import java.util.*;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;

import uk.ac.reload.menu.*;
import uk.ac.reload.prefs.*;
import uk.ac.reload.util.*;


/**
 * The Reload Editor Frame
 *
 * @author Phillip Beauvoir
 * @version 2003-07-25
 */
public class ReloadEditor
extends JFrame
{

    /**
     * Set this to true for a Release Build, keep false for debugging
     */
    public static boolean ISBUILD = true;

    //public static boolean STATUS_WINDOW = true;        // Send Output to Status Window
    public static boolean STATUS_WINDOW = ISBUILD;       // Send Output to Status Window
    public static boolean DEBUG = !ISBUILD;              // Debug Trace

    // Information Constants
    public static String APP_NAME =           "Reload Editor";
    public static String VERSION =            "1.1";
    public static String BUILD_DATE =         "11 August 2003";
    public static String COPYRIGHT =          "Copyright (c) 2003 Oleg Liber, Bill Olivier, Phillip Beauvoir";
    public static String AUTHORS =            "Phillip Beauvoir, Paul Sharples & Tina Manoharan";
    public static String CONTRIBUTORS =       "Phil Barker, Lorna Campbell, Sarah Currier, " +
                                              "Charles Duncan, Pierre Gorissen, Gerry Graham, Steve Jeyes, " +
                                              "Wilbert Kraan, Boon Low, Colin Milligan, " +
                                              "Sharon Perry, Mark Power, Ben Ryan, " +
                                              "Scott Wilson, Frank Zappa and all the Glasgow code-bashers";
    public static String EMAIL_CONTACT =      "p.beauvoir@bolton.ac.uk";
    public static String WEB_PAGE =           "http://www.reload.ac.uk";

    public static String CR = System.getProperty("line.separator");


    /**
     * The singleton instance of the app
     */
    private static ReloadEditor _instance;

    /**
     * The desktop pane.
     */
    protected JDesktopPane _desktop;

    /**
     * Gets the Singleton Instance of the Editor
     * @return The Singleton Instance of the Editor
     */
    public static ReloadEditor getInstance() {
        return _instance;
    }


    /**
     * Constructor method for new Reload Editor
     */
    public ReloadEditor() {
        _instance = this;

        // UI Defaults
        ReloadUIManager.setUIDefaults();

        // SplashScreen
        SplashScreen splash = new SplashScreen();

        // Get the Status Window going
        ReloadStatusWindow.getInstance();

        // Set Look And Feel from User Prefs file
        String lf = UserPrefs.getSharedInstance().getValue(UserPrefs.GENERAL_PREFS_LF);
        if(lf == null) lf = UIManager.getSystemLookAndFeelClassName();
        //String lf = UIManager.getSystemLookAndFeelClassName();
        ReloadUIManager.setLookAndFeel(lf);

        // Set application title in title bar
        super.setTitle(APP_NAME);

        // Set application Icon
        ImageIcon icon = ReloadUIManager.getIcon(ReloadUIManager.ICON_APP32);
        if(icon != null) setIconImage(icon.getImage());

        // Trap window closing event for our Window listener
        setDefaultCloseOperation(JFrame.DO_NOTHING_ON_CLOSE);

        // Add a listener to Close our window down on exit
        addWindowListener(new WindowAdapter() {
            public void windowClosing(WindowEvent e) {
            	closeApp();
            }
        });

        // Set up desktop
        JPanel panel = new JPanel(new BorderLayout());
        _desktop = new JDesktopPane();
        panel.add(_desktop, BorderLayout.CENTER);
        setContentPane(panel);

        // Create Menu Bar (after setting up the desktop, please!)
        setJMenuBar(MainMenu.sharedInstance);

        // Create Toolbar
        getContentPane().add(MainMenu.sharedInstance.getReloadToolBar(), BorderLayout.NORTH);

        splash.close();

        // Show the main Frame Window, centred
        ReloadUtils.centreWindowProportional(this, 0.9, 0.9);
        setVisible(true);
    }

    ////////////////////////////////////////////////////////////////////////////
    ////////////////////////////////////////////////////////////////////////////
    ////////////////////////////////////////////////////////////////////////////

    /**
     * Main program entry point
     * @param args Command line arguments
     */
    public static void main(String[] args) {

		/**
		* Tina Manoharan
		* 2003-09-09
		*/
        // initialise ReloadPropertiesManager
        ReloadPropertiesManager.setResourceBundle();


        new ReloadEditor();
    }

    /**
     * Quit the application
     * Put any cleanup code here.
     */
    public void closeApp() {
    	// Tell any registered listeners
        boolean result = fireApplicationClosing();
        if(result) {
            // Save some Prefs
            String lf = UIManager.getLookAndFeel().getClass().getName();
            UserPrefs.getSharedInstance().putValue(UserPrefs.GENERAL_PREFS_LF, lf);

            try {
                UserPrefs.getSharedInstance().save();
            }
            catch(ReloadException ex) {
                ErrorDialogBox.showWarning("Could not save Preferences: ", "Preferences", ex);
            }

            // Quit
            System.exit(0);
        }
    }

    /**
     * Set the Title Bar String
     * @param title The Title Bar String
     */
    public void setTitle(String title) {
    	super.setTitle(APP_NAME + " - " + title);
    }


    ////////////////////////////////////////////////////////////////////////////
    //   INTERNAL FRAME MANAGEMENT   ///////////////////////////////////////////
    ////////////////////////////////////////////////////////////////////////////

    public JDesktopPane getDesktop() {
        return _desktop;
    }

    /**
     * Add a new ReloadInternalFrame
     * @param frame The new Frame
     */
    public void addInternalFrame(ReloadInternalFrame frame) {
        ReloadInternalFrameManager.getInstance().addInternalFrame(frame);
    }

    /**
     * Remove a new ReloadInternalFrame
     * @param frame The Frame
     */
    public void removeInternalFrame(ReloadInternalFrame frame) {
        ReloadInternalFrameManager.getInstance().removeInternalFrame(frame);
    }

    ////////////////////////////////////////////////////////////////////////////
    //   APPLICATION LISTENERS  ////////////////////////////////////////////////
    ////////////////////////////////////////////////////////////////////////////

    /**
     * List of Listeners
     */
    private Vector listeners = new Vector();

    /**
     * Add an Application Listener
     * @param listener The Application Listener
     */
    public synchronized void addApplicationListener(ApplicationListener listener) {
        if(!listeners.contains(listener)) listeners.addElement(listener);
    }

    /**
     * Remove an Application Listener
     * @param listener The Application Listener
     */
    public synchronized void removeApplicationListener(ApplicationListener listener) {
        listeners.removeElement(listener);
    }

    /**
     * Tell our listeners that we are closing the App
     * They must reply by answering with true for OK, or false for not OK
     */
    public boolean fireApplicationClosing() {
        boolean ok = true;
        // Go from last to first in case anyone removes themselves
        for(int i = listeners.size() - 1; i >= 0; i--) {
            ApplicationListener listener = (ApplicationListener)listeners.elementAt(i);
            boolean result = listener.applicationClosing();
            if(result == false) ok = false;
        }
        return ok;
    }
}