/**
 *  RELOAD Editor - An IMS Metadata, Content Packaging and Learning Design tool
 *  for the JISC X4L Programme.
 *
 *  Copyright (C) 2003 Oleg Liber, Bill Olivier, Phillip Beauvoir
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *  Project Management Contact:
 *
 *  Oleg Liber
 *  Bolton Institute of Higher Education
 *  Deane Road
 *  Bolton BL3 5AB
 *  UK
 *
 *  e-mail:   o.liber@bolton.ac.uk
 *
 *
 *  Technical Contact:
 *
 *  Phillip Beauvoir
 *  e-mail:   p.beauvoir@bolton.ac.uk
 *
 *  Web:      http://www.reload.ac.uk
 *
 */


package uk.ac.reload.contentpackaging.resourceview;

import java.io.*;
import java.util.*;
import javax.swing.*;
import javax.swing.tree.*;
import uk.ac.reload.*;
import uk.ac.reload.widgets.*;
import uk.ac.reload.util.*;

/**
 * Class to delete selected files from the project folder (resources pool)
 *
 * @author Paul Sharples
 * @version 2003-04-30
 */
public class ResourceRemover {


    /*
    * A vector to hold all of the files that have been selected
    * to be deleted by the user.
    */
    private Vector fileVector = new Vector();

    /*
    * A vector to hold all of the folders that have been selected
    * to be deleted by the user.
    */
    private Vector folderVector = new Vector();

    /**
     * Default constructor
     */
    public ResourceRemover(){};

    /**
     * Method to obtain all currently selected tree nodes
     * and decide if each element is a file reference or
     * a directory
     */
    public void removeFiles(CPResourcesTree theTree) {
        ReloadResource[] resources = theTree.getSelectedResources();
        /*
        * select the root node, just in case the tree decides to try
        * and refresh an open folder icon to a closed icon once
        * the directory is empty...
        */
        theTree.setSelectionRow(0);
        boolean yesToAll = false;
        for(int i = 0; i < resources.length; i++) {
            ReloadResource aFile = resources[i];
            if(!yesToAll) {
                int answer = getUserResponse(aFile);
                if(answer == ReloadYesAllNoDialog.YES_TO_ALL)
                    yesToAll = true;
                if(answer == ReloadYesAllNoDialog.YES || answer == ReloadYesAllNoDialog.YES_TO_ALL) {
                    if(aFile.isDirectory()) {
                        findSubFolders(aFile, theTree);
                    }
                    else {
                        fileVector.add(aFile);
                    }
                }
                else if(answer == ReloadYesAllNoDialog.CANCEL || answer == ReloadYesAllNoDialog.CLOSE) {
                    // If the user cancels or closes the dialog then we will
                    // abandon the delete and clear the file & folder vectors
                    fileVector.clear();
                    folderVector.clear();
                    return;
                }
            }
            else {

                if(aFile.isDirectory()) {
                    findSubFolders(aFile, theTree);
                }
                else {
                    fileVector.add(aFile);
                }
            }
        }
        cleanup(theTree);
    }

    /**
     * Method to build message string, pass it to the YesAllNo dialog
     * and return the users choice
     * @param res - a ReloadResource object
     * @return users choice for yes, yes to all, no, cancel or close
     */
    private int getUserResponse(ReloadResource res){
        ReloadYesAllNoDialog dialog = new ReloadYesAllNoDialog();
        String message, title, fileOrFolder;
        fileOrFolder = res.isDirectory() ? "folder and all its contents" : "file";
        message = res.getAbsoluteFile().toString() + ReloadEditor.CR +
            "Are you sure you want to delete this "
            + fileOrFolder + "?";
        title = "Delete Resources";
        return dialog.getUserResponse(ReloadEditor.getInstance(), message, title);
    }

    /**
     * Method to actually update the tree and delete the selected files
     * This is done last.  The file vector and the folder vector are built up
     * first - in case we try to delete a folder when it isn't empty.
     */
    private void cleanup(CPResourcesTree theTree) {
        // first pass do the files
        Iterator allfiles = fileVector.iterator();
        while(allfiles.hasNext()) {
            ReloadResource aFileResource = (ReloadResource) allfiles.next();
            if (aFileResource.exists())  aFileResource.delete();
            if(!aFileResource.exists() && (theTree.getNode(aFileResource) != null))
                theTree.removeTreeResource(aFileResource);
        }
        // empty the list because we have finished
        fileVector.clear();

        // second pass do the folders
        Iterator allfolders = folderVector.iterator();
        while(allfolders.hasNext()) {
            ReloadResource aFolderResource = (ReloadResource) allfolders.next();
            if (aFolderResource.exists()) aFolderResource.delete();
            if(!aFolderResource.exists() && (theTree.getNode(aFolderResource) != null))
                theTree.removeTreeResource(aFolderResource);
        }
        // empty the list because we have finished
        folderVector.clear();
    }


    /**
     * Method to find all subfolders and files from a folder reference
     * add add them to two vector lists
     * @param afolder -  a folder
     */
    private void findSubFolders(ReloadResource afolder, CPResourcesTree theTree) {
        if(afolder != null) {
            CPResourcesTreeNode folderchild = (CPResourcesTreeNode) theTree.getNode(afolder);
            Enumeration e = folderchild.children();
            while(e.hasMoreElements()) {
                CPResourcesTreeNode node = (CPResourcesTreeNode) e.nextElement();
                ReloadResource aresource = (ReloadResource) node.getUserObject();
                if(aresource != null) {
                    if(aresource.exists() && aresource.isDirectory()) {
                        //recurse
                        findSubFolders(aresource, theTree);
                    }
                    else {
                        // add each file we come across to the file vector
                        fileVector.add(aresource);

                    }
                }
            }
            // finally add the folder to the folder vector
            folderVector.add(afolder);
        }
     }

}
