/**
 *  RELOAD Editor - An IMS Metadata, Content Packaging and Learning Design tool.
 *
 *  Copyright (c) 2003 Oleg Liber, Bill Olivier, Phillip Beauvoir
 *
 *  Permission is hereby granted, free of charge, to any person obtaining a copy
 *  of this software and associated documentation files (the "Software"), to deal
 *  in the Software without restriction, including without limitation the rights
 *  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 *  copies of the Software, and to permit persons to whom the Software is
 *  furnished to do so, subject to the following conditions:
 *
 *  The above copyright notice and this permission notice shall be included in
 *  all copies or substantial portions of the Software.
 *
 *  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 *  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 *  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 *  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 *  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 *  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 *
 *  Project Management Contact:
 *
 *  Oleg Liber
 *  Bolton Institute of Higher Education
 *  Deane Road
 *  Bolton BL3 5AB
 *  UK
 *
 *  e-mail:   o.liber@bolton.ac.uk
 *
 *
 *  Technical Contact:
 *
 *  Phillip Beauvoir
 *  e-mail:   p.beauvoir@bolton.ac.uk
 *
 *  Web:      http://www.reload.ac.uk
 *
 */

package uk.ac.reload.menu;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;

import uk.ac.reload.*;
import uk.ac.reload.dom.*;

/**
 * The Main Menu for the Application.  It extends JMenuBar.  It also takes care
 * of the Toolbar and any main Popup menus.  It can listen to any refresh requests
 * so that for example, a change will set enabled and disabled menu items and buttons.
 * This class is tied in with the MainMenuAction and MenuAction classes.
 *
 * @author Phillip Beauvoir
 * @version 2003-07-31
 */
public class MainMenu
extends JMenuBar
{
    /**
     * The Singleton instance
     */
    public static MainMenu sharedInstance = new MainMenu();

    public MenuAction_Save actionSave = new MenuAction_Save();
    public MenuAction_SaveAs actionSaveAs = new MenuAction_SaveAs();
    public MenuAction_ImportResources actionImport = new MenuAction_ImportResources();
    public MenuAction_EditMetadata actionEditMetadata = new MenuAction_EditMetadata();

	/**
	* Tina Manoharan
	* 2003-08-11
	*/
    public MenuAction_EditVocabulary actionEditVocabulary = new MenuAction_EditVocabulary();

    public MenuAction_EditSCORM actionEditSCORM = new MenuAction_EditSCORM();
    public MenuAction_ZipIt actionZipIt = new MenuAction_ZipIt();
    public MenuAction_ViewFile actionViewFile = new MenuAction_ViewFile();
    public MenuAction_ViewCP actionViewCP = new MenuAction_ViewCP();
    //public MenuAction_Refresh actionRefresh = new MenuAction_Refresh();

    /**
     * The main application toolbar
     */
    private ReloadToolBar toolBar;

    /**
     * The Key Mask will be 'Meta' for Mac and 'Ctrl' for PC/Unix
     */
    private int keyMask = Toolkit.getDefaultToolkit().getMenuShortcutKeyMask();

    /**
     * Constructor
     */
    private MainMenu() {
        // Attach Actions to Menu Items
        createMenuBar();

        // Create the ToolBar
        toolBar = createToolBar();
    }

    /**
     * Add an Internal Frame Window to the "Window" Menu Item.
     * We get the Menu Item from the Frame itself.
     * @param frame The frame to add to the menu
     */
    public void addInternalFrame(ReloadInternalFrame frame) {
        windowMenu.add(frame.getMenuItem());
    }

    /**
     * Remove an Internal Frame Window from the "Window" Menu Item.
     * We get the Menu Item from the Frame itself.
     * @param frame The frame to remove from the menu
     */
    public void removeInternalFrame(ReloadInternalFrame frame) {
        windowMenu.remove(frame.getMenuItem());
    }

    // =========================== MENU BAR ==================================
    /**
     * The File Menu
     */
    public JMenu fileMenu;

    /**
     * The Edit Menu
     */
    public Menu_Edit editMenu;

    /**
     * The Debug Menu
     */
    public JMenu debugMenu;

    /**
     * The Options Menu
     */
    public JMenu optionsMenu;

    /**
     * The View Menu
     */
    public JMenu viewMenu;

    /**
     * The Window Menu
     */
    public JMenu windowMenu;

    /**
     * The Help Menu
     */
    public JMenu helpMenu;

    /**
     * Create the Menu Bar
     */
    private void createMenuBar() {
        // File Menu
        fileMenu = createFileMenu();
        // Edit menu
        editMenu = new Menu_Edit();
        add(editMenu);
        // Debug menu
        if(ReloadEditor.DEBUG) debugMenu = createDebugMenu();
        // Options menu
        optionsMenu = createOptionsMenu();
        // View Menu
        viewMenu = createViewMenu();
        // Window Menu
        windowMenu = createWindowMenu();
        // Help Menu
        helpMenu = createHelpMenu();
    }

    /**
     * Create the File Menu
     * @return The JMenu
     */
    private JMenu createFileMenu() {
        JMenu jmenu = add(new JMenu("File"));
        jmenu.setMnemonic('f');

        // New
        JMenu newMenu = new JMenu("New");
        newMenu.setMnemonic('n');
        newMenu.setIcon(ReloadUIManager.getIcon(ReloadUIManager.ICON_NEW));
        jmenu.add(newMenu);

        	// New metadata
        	JMenuItem item = newMenu.add(new MenuAction_NewMetadata());
        	item.setMnemonic('m');

	    	/**
			 * Tina Manoharan
			 * 2003-08-11
	 		*/
        	// New vocabulary
        	item = newMenu.add(new MenuAction_NewVocabulary());
        	item.setMnemonic('v');


        	// New CP
        	item = newMenu.add(new MenuAction_NewContentPackage());
        	item.setMnemonic('c');

        	// New SCORM CP
        	item = newMenu.add(new MenuAction_NewSCORM1_2Package());
        	item.setMnemonic('p');

        // Open
        item = jmenu.add(new MenuAction_Open());
        item.setMnemonic('o');

        // Import
        JMenu importMenu = new JMenu("Import/Export");
        importMenu.setMnemonic('i');
        importMenu.setIcon(ReloadUIManager.getIcon(ReloadUIManager.ICON_IMPORT));
        jmenu.add(importMenu);

        // Import Resources
        item = importMenu.add(actionImport);
        item.setMnemonic('r');

		/**
		* Tina Manoharan
		* 2003-10-20
		*/
        // Import Zthes into Vocabulary Editor
        item = importMenu.add(new MenuAction_ImportZthes());
        item.setMnemonic('t');

        // Import Others into Vocabulary Editor
        item = importMenu.add(new MenuAction_ImportOtherVocabulary());
        item.setMnemonic('h');

        // Export Zthes from Vocabulary Editor
        item = importMenu.add(new MenuAction_ExportZthes());
        item.setMnemonic('e');

        jmenu.addSeparator();

        // Save
        item = jmenu.add(actionSave);
        item.setMnemonic('s');
        item.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_S, keyMask));

        // Save As
        item = jmenu.add(actionSaveAs);
        item.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_S, Event.SHIFT_MASK + keyMask));

        jmenu.addSeparator();

        // Zip
        item = jmenu.add(actionZipIt);
        item.setMnemonic('z');

        jmenu.addSeparator();

        // Exit
        item = jmenu.add(new MenuAction_Exit());
        item.setMnemonic('x');

        return jmenu;
    }

    /**
     * Create the Debug Menu
     * @return The JMenu
     */
    private JMenu createDebugMenu() {
        JMenu jmenu = add(new JMenu("Debug"));
        jmenu.setMnemonic('d');

        // MD Schemas
        jmenu.add(new JMenuItem(new MenuAction_DebugSchema(ReloadDocumentHandler.IMSMD_122, "IMS Metadata 1.2.1 Schema")));
        jmenu.add(new JMenuItem(new MenuAction_DebugSchema(ReloadDocumentHandler.IMSMD_122, "IMS Metadata 1.2.2 Schema")));
        jmenu.addSeparator();

	    /**
		 * Tina Manoharan
		 * 2003-08-11
	 	*/
        // VX Schemas
        jmenu.add(new JMenuItem(new MenuAction_DebugSchema(ReloadDocumentHandler.IMSVX_LAX_100, "IMS Vdex Lax 1.0.0 Schema")));
        jmenu.add(new JMenuItem(new MenuAction_DebugSchema(ReloadDocumentHandler.IMSVX_FLAT_100, "IMS Vdex Flat 1.0.0 Schema")));
        jmenu.add(new JMenuItem(new MenuAction_DebugSchema(ReloadDocumentHandler.IMSVX_HIERARCHICAL_100, "IMS Vdex Hierarchical 1.0.0 Schema")));
        jmenu.add(new JMenuItem(new MenuAction_DebugSchema(ReloadDocumentHandler.IMSVX_GLOSSARY_100, "IMS Vdex Glossary 1.0.0 Schema")));
        jmenu.add(new JMenuItem(new MenuAction_DebugSchema(ReloadDocumentHandler.IMSVX_THESAURUS_100, "IMS Vdex Thesaurus 1.0.0 Schema")));
        jmenu.addSeparator();

        // CP Schemas
        jmenu.add(new JMenuItem(new MenuAction_DebugSchema(ReloadDocumentHandler.IMSCP_112, "IMS Content Packaging 1.1.2 Schema")));
        jmenu.add(new JMenuItem(new MenuAction_DebugSchema(ReloadDocumentHandler.IMSCP_113, "IMS Content Packaging 1.1.3 Schema")));
        jmenu.addSeparator();

        // SCORM Schemas
        jmenu.add(new JMenuItem(new MenuAction_DebugSchema(ReloadDocumentHandler.ADLCP_120_112, "ADL SCORM 1.2 (CP 1.1.2) Schema")));
        jmenu.add(new JMenuItem(new MenuAction_DebugSchema(ReloadDocumentHandler.ADLCP_120_113, "ADL SCORM 1.2 (CP 1.1.3) Schema")));
        jmenu.addSeparator();

        // Simple Sequencing Schemas
        jmenu.add(new JMenuItem(new MenuAction_DebugSchema(ReloadDocumentHandler.IMSSS_100, "IMS Simple Sequencing 1.0 Schema")));

        return jmenu;
    }

    /**
     * Create the Options Menu
     * @return The JMenu
     */
    private JMenu createOptionsMenu() {
        JMenu jmenu = add(new JMenu("Options"));
        jmenu.setMnemonic('o');

        //jmenu.addSeparator();

        // MD Profile Editor Window
        if(ReloadEditor.DEBUG) {
            JMenuItem item = jmenu.add(new MenuAction_MDProfileEditor());

            item = jmenu.add(new MenuAction_VXProfileEditor());
        }

        JMenuItem item = jmenu.add(new MenuAction_Prefs());

        return jmenu;
    }

    /**
     * Create the View Menu
     * @return The JMenu
     */
    private JMenu createViewMenu() {
        JMenu jmenu = add(new JMenu("View"));
        jmenu.setMnemonic('v');

        // Launch File
        JMenuItem item = jmenu.add(actionViewFile);
        item.setMnemonic('f');

        // View CP
        item = jmenu.add(actionViewCP);
        item.setMnemonic('i');

        // Refresh
        //item = jmenu.add(actionRefresh);
        //item.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_F5, 0));

        jmenu.addSeparator();

        // Look & Feels
        JMenu subMenu = new JMenu("Look & Feel");
        addLookAndFeels(subMenu);
        jmenu.add(subMenu);

        // Status Window
        item = jmenu.add(new MenuAction_StatusWindow());

        return jmenu;
    }

    /**
     * Add installed Look & Feels to the menu
     * @param menu The menu to add the L&Fs to.
     */
    protected void addLookAndFeels(JMenu menu) {
        // Add some Menu Items for each installed L&F
        ButtonGroup bg = new ButtonGroup();

        // Get the look and feels
        UIManager.LookAndFeelInfo[] lfs = UIManager.getInstalledLookAndFeels();

        for(int i = 0; i < lfs.length; i++) {
            UIManager.LookAndFeelInfo lfInfo = lfs[i];

            // Add menu item
            JCheckBoxMenuItem jcb = new JCheckBoxMenuItem(new MenuAction_LookAndFeel(lfInfo));
            JMenuItem item = menu.add(jcb);
            bg.add(item);

            // Select currently set L&F as ticked
            if(UIManager.getLookAndFeel().getName().equals(lfInfo.getName())) jcb.setSelected(true);
        }
    }

    /**
     * Create the Window Menu
     * @return The JMenu
     */
    private JMenu createWindowMenu() {
        JMenu jmenu = add(new JMenu("Window"));
        jmenu.setMnemonic('w');

        // Cascade
        JMenuItem item = jmenu.add(new MenuAction_CascadeWindow());
        jmenu.setMnemonic('c');
        // Tile H
        item = jmenu.add(new MenuAction_TileHorizontalWindow());
        jmenu.setMnemonic('t');
        // Tile V
        item = jmenu.add(new MenuAction_TileVerticalWindow());
        jmenu.setMnemonic('v');

        jmenu.addSeparator();

        return jmenu;
    }

    /**
     * Create the Help Menu
     * @return The JMenu
     */
    private JMenu createHelpMenu() {
        JMenu jmenu = add(new JMenu("Help"));
        jmenu.setMnemonic('h');

        // Help
        JMenuItem item = jmenu.add(new MenuAction_Help());
        item.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_F1, 0));

        jmenu.addSeparator();

        // About
        item = jmenu.add(new MenuAction_About());
        item.setMnemonic('a');

        return jmenu;
    }

    // =========================== TOOLBAR ==================================

    /**
     * Create the Toolbar.
     * @return The Toolbar
     */
    private ReloadToolBar createToolBar() {
        ReloadToolBar tBar = new ReloadToolBar();

        // New
        MenuAction_New actionNew = new MenuAction_New();
        tBar.add(actionNew);

        // Open
        MenuAction_Open actionOpen = new MenuAction_Open();
        tBar.add(actionOpen);

        // Save
        tBar.add(actionSave);

        // Add a separator
        tBar.addSeparator();

        // Undo
        tBar.add(editMenu.actionUndo);

        // Redo
        tBar.add(editMenu.actionRedo);

        // Add a separator
        tBar.addSeparator();

        // Cut
        tBar.add(editMenu.actionCut);

        // Copy
        tBar.add(editMenu.actionCopy);

        // Paste
        tBar.add(editMenu.actionPaste);

        // Delete
        tBar.add(editMenu.actionDelete);

        // Add a separator
        tBar.addSeparator();

        // Move Up
        tBar.add(editMenu.actionMoveUp);

        // Move Down
        tBar.add(editMenu.actionMoveDown);

        // Add a separator
        tBar.addSeparator();

        // Edit Metadata
        tBar.add(actionEditMetadata);

	    /**
		 * Tina Manoharan
		 * 2003-08-11
	 	*/
		// Edit Vocabulary
		tBar.add(actionEditVocabulary);

        // Edit SCORM
        tBar.add(actionEditSCORM);

        // Add a separator
        tBar.addSeparator();

        // Zip
        tBar.add(actionZipIt);

        // View File
        tBar.add(actionViewFile);

        // View CP
        tBar.add(actionViewCP);

        return tBar;
    }

    /**
     * Get the ReloadToolBar.
     * @return The main toolbar
     */
    public ReloadToolBar getReloadToolBar() {
        return toolBar;
    }

}

