/**
  *  RELOAD Editor - An IMS Metadata, Content Packaging and Learning Design tool.
  *
  *  Copyright (c) 2003 Oleg Liber, Bill Olivier, Phillip Beauvoir
  *
  *  Permission is hereby granted, free of charge, to any person obtaining a copy
  *  of this software and associated documentation files (the "Software"), to deal
  *  in the Software without restriction, including without limitation the rights
  *  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
  *  copies of the Software, and to permit persons to whom the Software is
  *  furnished to do so, subject to the following conditions:
  *
  *  The above copyright notice and this permission notice shall be included in
  *  all copies or substantial portions of the Software.
  *
  *  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  *  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  *  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
  *  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
  *  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
  *  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
  *
  *  Project Management Contact:
  *
  *  Oleg Liber
  *  Bolton Institute of Higher Education
  *  Deane Road
  *  Bolton BL3 5AB
  *  UK
  *
  *  e-mail:   o.liber@bolton.ac.uk
  *
  *
  *  Technical Contact:
  *
  *  Phillip Beauvoir
  *  e-mail:   p.beauvoir@bolton.ac.uk
  *
  *  Web:      http://www.reload.ac.uk
  *
  */

package uk.ac.reload.menu;

import uk.ac.reload.undo.*;

/**
 * An Undo Manager for the Menus
 *
 * @author Phillip Beauvoir
 * @version 2003-04-25
 */
public class ReloadUndoMenuManager implements UndoListener, MenuActionListener {
    /**
     * The Undo Handler
     */
    private ReloadUndoHandler undoHandler;

    private boolean hasFocus = false;

    private MenuAction actionUndo;
    private MenuAction actionRedo;

    /**
     * Constructor
     */
    public ReloadUndoMenuManager(Menu_Edit editMenu) {
        undoHandler = new ReloadUndoHandler(this);
        actionUndo = editMenu.actionUndo;
        actionRedo = editMenu.actionRedo;
    }

    /**
     * The Undo or Redo Menu Item was performed
     */
    public void menuActionPerformed(ReloadMenuEvent event) {
        if(hasFocus && event.getSource() == actionUndo) {
            undoLastAction();
        }
        else if(hasFocus && event.getSource() == actionRedo) {
            redoLastAction();
        }
    }

    /**
     * An Undoable Action occurred
     */
    public void undoableActionHappened(UndoableAction action) {
        if(hasFocus) setMenus();
    }

    private void setMenus() {
        UndoableAction undoAction = undoHandler.nextUndoAction();
        if(undoAction == null) {
            actionUndo.setText("Undo");
            actionUndo.setEnabled(false);
        }
        else {
            actionUndo.setText("Undo " + undoAction.getName());
            actionUndo.setEnabled(true);
        }

        UndoableAction redoAction = undoHandler.nextRedoAction();
        if(redoAction == null) {
            actionRedo.setText("Redo");
            actionRedo.setEnabled(false);
        }
        else {
            actionRedo.setText("Redo " + redoAction.getName());
            actionRedo.setEnabled(true);
        }
    }

    /**
     * Undo the last Action in the Stack
     */
    private void undoLastAction() {
        undoHandler.undoLastAction();
        if(hasFocus) setMenus();
    }

    /**
     * Redo the last Action in the Stack
     */
    private void redoLastAction() {
        undoHandler.redoLastAction();
        if(hasFocus) setMenus();
    }

    public void setFocusGained() {
        //System.out.println("Focus Gained");
        hasFocus = true;
        actionUndo.addMenuActionListener(this);
        actionRedo.addMenuActionListener(this);
        setMenus();
    }

    public void setFocusLost() {
        //System.out.println("Focus Lost");
        hasFocus = false;
        actionUndo.removeMenuActionListener(this);
        actionRedo.removeMenuActionListener(this);

        actionUndo.setText("Undo");
        actionUndo.setEnabled(false);
        actionRedo.setText("Redo");
        actionRedo.setEnabled(false);
    }

    /**
     * Clean up
     */
    public void destroy() {
        actionUndo.removeMenuActionListener(this);
        actionRedo.removeMenuActionListener(this);
    }


    /**
     * @return the Undo Handler
     */
    public ReloadUndoHandler getUndoHandler() {
        return undoHandler;
    }
}
