/**
 *  RELOAD Editor - An IMS Metadata, Content Packaging and Learning Design tool.
 *
 *  Copyright (c) 2003 Oleg Liber, Bill Olivier, Phillip Beauvoir
 *
 *  Permission is hereby granted, free of charge, to any person obtaining a copy
 *  of this software and associated documentation files (the "Software"), to deal
 *  in the Software without restriction, including without limitation the rights
 *  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 *  copies of the Software, and to permit persons to whom the Software is
 *  furnished to do so, subject to the following conditions:
 *
 *  The above copyright notice and this permission notice shall be included in
 *  all copies or substantial portions of the Software.
 *
 *  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 *  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 *  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 *  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 *  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 *  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 *
 *  Project Management Contact:
 *
 *  Oleg Liber
 *  Bolton Institute of Higher Education
 *  Deane Road
 *  Bolton BL3 5AB
 *  UK
 *
 *  e-mail:   o.liber@bolton.ac.uk
 *
 *
 *  Technical Contact:
 *
 *  Phillip Beauvoir
 *  e-mail:   p.beauvoir@bolton.ac.uk
 *
 *  Web:      http://www.reload.ac.uk
 *
 */

package uk.ac.reload.metadata;

import java.io.*;

import org.jdom.*;
import uk.ac.reload.*;
import uk.ac.reload.dom.*;
import uk.ac.reload.schema.*;
import uk.ac.reload.util.*;

/**
 * The Metadata Class
 *
 * @author Phillip Beauvoir
 * @version 2003-08-01
 */
public class Metadata
extends AbstractMetadata
{
    public static final String ROOT_NAME = "lom";

    /**
     * Our unique signature
     */
    static final String[] md_comments = {
        "This is a Reload version " + ReloadEditor.VERSION + " Metadata document",
        "Spawned from the Reload Metadata Generator - http://www.reload.ac.uk"
    };

    /**
     * Whether we are a standalone file or attached
     */
    protected boolean _isStandalone;

    /**
     * Constructor
     */
    public Metadata(boolean isStandalone) {
        super();
        _isStandalone = isStandalone;
    }

    /**
     * Setup the JDOM Document with Namespaces and root element
     */
    public void newDocument(MetadataSchemaController controller) {
        _doc = new Document();

        _controller = controller;

        // Add our signature
        if(_isStandalone) addComments(_doc);

        // Add Root Element according to Schema
        ReloadSchema mdSchema = _controller.getReloadSchema();
        Element root = addElementBySchema(this, null, mdSchema.getRootElement(), true);
        _doc.setRootElement(root);

        // Add Root Declarations
        if(_isStandalone) addRootDeclarations(root);

        // Clear the dirty flag
        setDirty(false);
    }

    /**
     * @return the Root element name
     */
    public String getRootElementName() {
        return ROOT_NAME;
    }

    /**
     * @return the Comments to add to the XML Document
     */
    public String[] getComments() {
        return md_comments;
    }

    /**
     * Add namespace and other attribute decalarations to the root element
     * @param root
     */
    private void addRootDeclarations(Element root) {
        ReloadSchema mdSchema = getReloadSchemaController().getReloadSchema();

        // Add XSI Schema Instance Namespace
        root.addNamespaceDeclaration(ReloadDocumentHandler.XSI_Namespace);

        // Add Schema Location Attribute which is constructed from Target Namespace
        // and file name of Schema
        String schemaLocationURI = mdSchema.getTargetNamespace() + " " + mdSchema.getSchemaName();
        root.setAttribute(ReloadDocumentHandler.XSI_SchemaLocation, schemaLocationURI, ReloadDocumentHandler.XSI_Namespace);
    }

    /**
     * Set the Metadata to a LOM Element from a Content Package
     */
    public void setMetadataElement(Element mdElement, MetadataSchemaController controller) {
        _doc = new Document();
        _controller = controller;
        _doc.setRootElement(mdElement);
    }

    /**
     * Set the Metadata to an Imported Metadata Doc
     */
    public void importMetadata(Document doc, MetadataSchemaController controller) throws ReloadException {
        // Get the Root Element
        Element root = (Element)doc.getRootElement();

        // Clear any Attributes
        root.setAttributes(null);

        // Add Namespace prefixes to all Elements
        Namespace ns = root.getNamespace();
        if(ns != null) ReloadDocumentHandler.replaceNamespaces(root, ns, getSchemaNamespaceEmbedded());

        _doc = doc;
        _controller = controller;
    }

    /**
     * Export the MD from a CP node to a standalone file
     * @param file
     * @throws ReloadException
     */
    public void exportMetadata(File file) throws ReloadException {
        if(!isStandalone() && _doc != null) {
            Document newDoc = new Document();
            // Add comments
            addComments(newDoc);

            // Clone
            Element root = (Element)_doc.getRootElement().clone();

            // Remove Namespace prefixes from all Elements
            Namespace ns = root.getNamespace();
            if(ns != null) ReloadDocumentHandler.replaceNamespaces(root, ns, Namespace.getNamespace(ns.getURI()));

            // Set to root
            newDoc.setRootElement(root);

            // Add root declarations
            addRootDeclarations(root);

            XMLUtils.write2XMLFile(newDoc, file);
            newDoc = null;
        }
    }

    /**
     * Return whether we are a standalone file
     * @return
     */
    public boolean isStandalone() {
        return _isStandalone;
    }

    /**
     * Get the appropriate Namespace
     */
    public Namespace getSchemaNamespace() {
        if(_isStandalone) return super.getSchemaNamespace();
        else return getSchemaNamespaceEmbedded();
    }

    /**
     * Get the standalone Namespace
     */
    public Namespace getSchemaNamespaceStandalone() {
        return super.getSchemaNamespace();
    }

    /**
     * Get the embedded Namespace
     */
    public Namespace getSchemaNamespaceEmbedded() {
        return Namespace.getNamespace(ReloadDocumentHandler.IMSMD_NAMESPACE_PREFIX, getSchemaNamespaceStandalone().getURI());
    }

// ==========================================================================
// ELEMENT MOVING/COPYING/DRAGGING HANDLING
// ==========================================================================

    /**
     * @return whether we can copy an element
     */
    public boolean canCopyElement(Element element) {
        return !element.isRootElement();
    }

    /**
     * @return whether we can cut an element
     */
    public boolean canCutElement(Element element, ReloadSchemaElement schemaElement) {
        return canDeleteElement(element, schemaElement);
    }

    /**
     * @return whether we can paste an element
     */
    public boolean canPasteFromClipboard(Element element) {
        Element clipboardElement = ReloadDocumentClipboard.getElement();
        return clipboardElement != null && isAllowedChild(element, clipboardElement);
    }

}