/**
  *  RELOAD Editor - An IMS Metadata, Content Packaging and Learning Design tool.
  *
  *  Copyright (c) 2003 Oleg Liber, Bill Olivier, Phillip Beauvoir
  *
  *  Permission is hereby granted, free of charge, to any person obtaining a copy
  *  of this software and associated documentation files (the "Software"), to deal
  *  in the Software without restriction, including without limitation the rights
  *  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
  *  copies of the Software, and to permit persons to whom the Software is
  *  furnished to do so, subject to the following conditions:
  *
  *  The above copyright notice and this permission notice shall be included in
  *  all copies or substantial portions of the Software.
  *
  *  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  *  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  *  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
  *  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
  *  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
  *  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
  *
  *  Project Management Contact:
  *
  *  Oleg Liber
  *  Bolton Institute of Higher Education
  *  Deane Road
  *  Bolton BL3 5AB
  *  UK
  *
  *  e-mail:   o.liber@bolton.ac.uk
  *
  *
  *  Technical Contact:
  *
  *  Phillip Beauvoir
  *  e-mail:   p.beauvoir@bolton.ac.uk
  *
  *  Web:      http://www.reload.ac.uk
  *
  */

package uk.ac.reload.metadata;

import java.io.*;
import java.util.*;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;

import org.jdom.*;
import uk.ac.reload.*;
import uk.ac.reload.menu.*;
import uk.ac.reload.metadata.formview.*;
import uk.ac.reload.metadata.tableview.*;
import uk.ac.reload.widgets.*;

/**
 * The Metadata Editor
 *
 *
 * @author Phillip Beauvoir
 * @version 2003-07-25
 */
public class MetadataEditor
extends JPanel
{

    /**
     * The Metadata Document
     */
    protected Metadata _metadata;

    /**
     * The Prefs panel
     */
    protected MetadataPrefsPanel _prefsPanel;

    /**
     * The Form View
     */
    protected MetadataFormPanel _formPanel;

    /**
     * The Table View
     */
    protected MetadataTablePanel _tablePanel;

    /**
     * The Tab Pane
     */
    protected JTabbedPane _tabPane;

    /**
     * The Edit Menu
     */
    protected Menu_Edit _editMenu;

    /**
     * The Undo Menu Manager for this Window
     */
    protected ReloadUndoMenuManager _undoMenuManager;

    /**
     * Constructor
     */
    public MetadataEditor(Menu_Edit editMenu) {
        _editMenu = editMenu;

        setLayout(new BorderLayout());

        // Get a new Form View
        _formPanel = new MetadataFormPanel(this);

        // Get a new Table View
        _tablePanel = new MetadataTablePanel(editMenu);

        // Get a Prefs panel
        _prefsPanel = new MetadataPrefsPanel();

        // Add Prefs Panel and Form View to another Panel
        JPanel p = new JPanel(new BorderLayout());
        p.add(_prefsPanel, BorderLayout.NORTH);
        p.add(new JScrollPane(_formPanel), BorderLayout.CENTER);

        // Tab Pane
        _tabPane = new JTabbedPane();
        _tabPane.addTab("Form", null, p, "Form View");
        _tabPane.addTab("Full", null, _tablePanel, "Full View");
        add(_tabPane, BorderLayout.CENTER);
    }

    /**
     * Get the Metadata Document
     * @return
     */
    public Metadata getMetadata() {
        return _metadata;
    }

    /**
     * Load a new Metadata JDOM Document
     * @param metadata The Metadata JDOM Document
     * @throws ReloadException
     */
    public void newDocument(boolean standalone, MetadataSchemaController schemaController) throws ReloadException {
        _metadata = new Metadata(standalone);
        _metadata.newDocument(schemaController);
        init(_metadata);
    }

    /**
     * Set a Metadata JDOM Document - this will be standalone
     * @throws ReloadException
     */
    public void setDocument(File file, Document doc, MetadataSchemaController schemaController) throws ReloadException {
        _metadata = new Metadata(true);
        _metadata.setDocument(file, doc, schemaController);
        init(_metadata);
    }

    /**
     * Set the Metadata to a MD Element from a CP
     * @param mdElement
     */
    public void setMetadataElement(Element mdElement, MetadataSchemaController schemaController) {
        _metadata = new Metadata(false);
        _metadata.setMetadataElement(mdElement, schemaController);
        init(_metadata);
    }

    /**
     * Set the Metadata to an Imported MD Doc for a CP
     * @param mdElement
     */
    public void importMetadata(Document doc, MetadataSchemaController schemaController) throws ReloadException {
        //_metadata = new Metadata(false);
        _metadata.importMetadata(doc, schemaController);
        _metadata.setDirty(true);
        init(_metadata);
    }

    /**
     * Export the Metadata to a File
     * @param mdElement
     */
    public void exportMetadata(File file) throws ReloadException {
        if(_metadata != null) _metadata.exportMetadata(file);
    }

    /**
     * Initialise some stuff
     */
    private void init(Metadata metadata) {
        // Tell Prefs Panel
        _prefsPanel.init(metadata);

        _formPanel.setDocument(metadata);
        _tablePanel.setDocument(metadata);

        // New Undo Menu Manager
        _undoMenuManager = new ReloadUndoMenuManager(_editMenu);
        metadata.setUndoHandler(_undoMenuManager.getUndoHandler());
    }

    /**
     * Over-ride this so we can set view stuff
     */
    public void initView() {
        _tablePanel.initView();
    }

    /**
     * @return the UndoManager
     */
    public ReloadUndoMenuManager getUndoManager() {
        return _undoMenuManager;
    }

    /**
     * Dynamically load a Metadata Profile
     */
    public void loadProfile(MetadataProfile mdProfile) {
        // Tell Schema Controller
        ((MetadataSchemaController)getMetadata().getReloadSchemaController()).setMetadataProfile(mdProfile);

        // Tell Panels
        if(_metadata != null) {
            SwingUtilities.invokeLater(new Runnable() {
                public void run() {
                    _formPanel.setDocument(_metadata);
                    _tablePanel.refresh();
                }
            });
        }
    }

    /**
     * Set the Default Language
     * @param language en, fr, de etc
     */
    public void setDefaultLanguage(String lang) {
       ((MetadataSchemaController)getMetadata().getReloadSchemaController()).setDefaultLanguage(lang);
    }

    /**
     * Dispose of this and clean up
     */
    public void destroy() {
        try {
            if(_undoMenuManager != null) _undoMenuManager.destroy();
            _formPanel.destroy();
            _tablePanel.destroy();
            _metadata.destroy();
        }
        catch(Exception ex) {
            ex.printStackTrace();
        }
    }

    /**
     * The Preferences Panel
     */
    class MetadataPrefsPanel extends JPanel {

        /**
         * Profile Combo box
         */
        ReloadComboBox profileCombobox;

        /**
         * Language Combo box
         */
        ReloadComboBox languageCombobox;

        /**
         * Constructor
         */
        MetadataPrefsPanel() {
            setLayout(new FlowLayout(FlowLayout.LEFT));

            // PROFILE COMBO BOX
            add(new JLabel("Profile: "));
            profileCombobox = new ReloadComboBox(false);
            add(profileCombobox);

            // Populate it
            MetadataProfile[] profiles = MetadataProfile.getMetadataProfiles();
            profileCombobox.setItems(profiles);

            // LANGUAGE COMBO BOX
            add(new JLabel("Default Language: "));
            languageCombobox = new ReloadComboBox(Locale.getISOLanguages());
            languageCombobox.setSelectedItem(Locale.getDefault().getLanguage());
            add(languageCombobox);
        }

        /**
         * Initialise
         */
        public void init(Metadata metadata) {
            MetadataProfile mdProfile = ((MetadataSchemaController)metadata.getReloadSchemaController()).getMetadataProfile();
            profileCombobox.setSelectedItem(mdProfile);

            // Listen to changes on the Profile Combo
            profileCombobox.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    MetadataProfile profile = (MetadataProfile)profileCombobox.getSelectedItem();
                    loadProfile(profile);
                    setDefaultLanguage(languageCombobox.getSelectedItem().toString());
                }
            });

            // Listen to changes on the Language Combo
            languageCombobox.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    setDefaultLanguage(languageCombobox.getSelectedItem().toString());
                }
            });
        }
    }
}