/**
  *  RELOAD Editor - An IMS Metadata, Content Packaging and Learning Design tool.
  *
  *  Copyright (c) 2003 Oleg Liber, Bill Olivier, Phillip Beauvoir
  *
  *  Permission is hereby granted, free of charge, to any person obtaining a copy
  *  of this software and associated documentation files (the "Software"), to deal
  *  in the Software without restriction, including without limitation the rights
  *  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
  *  copies of the Software, and to permit persons to whom the Software is
  *  furnished to do so, subject to the following conditions:
  *
  *  The above copyright notice and this permission notice shall be included in
  *  all copies or substantial portions of the Software.
  *
  *  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  *  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  *  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
  *  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
  *  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
  *  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
  *
  *  Project Management Contact:
  *
  *  Oleg Liber
  *  Bolton Institute of Higher Education
  *  Deane Road
  *  Bolton BL3 5AB
  *  UK
  *
  *  e-mail:   o.liber@bolton.ac.uk
  *
  *
  *  Technical Contact:
  *
  *  Phillip Beauvoir
  *  e-mail:   p.beauvoir@bolton.ac.uk
  *
  *  Web:      http://www.reload.ac.uk
  *
  */

package uk.ac.reload.metadata;

import java.io.*;
import java.util.*;

import javax.swing.*;

import uk.ac.reload.*;
import uk.ac.reload.schema.*;
import uk.ac.reload.dom.*;
import org.jdom.*;

/**
 * The Metadata Schema Controller - this interfaces from the various helpers
 *
 * @author Phillip Beauvoir
 * @version 2003-08-04
 */
public class MetadataSchemaController
extends ReloadSchemaController
{

    /**
     * The current Metadata Profile
     */
    protected MetadataProfile _mdProfile;

    /**
     * The default language hard-coded for now
     */
    protected String defaultLanguage = Locale.getDefault().getLanguage();

    /**
     * Constructor
     */
    public MetadataSchemaController(File fileSchema, File fileHelper, File fileVocab) {
        super(fileSchema, fileHelper, fileVocab);
    }

    /**
     * Constructor
     */
    public MetadataSchemaController(MetadataProfile mdProfile) {
        setMetadataProfile(mdProfile);
    }

    /**
     * @return the Default MetadataSchemaController
     */
    public static MetadataSchemaController getDefaultSchemaController() {
        return (MetadataSchemaController)ReloadDocumentHandler.getReloadSchemaController(ReloadDocumentHandler.IMSMD_122);
    }

    /**
     * Set the Metadata Profile
     * @param mdProfile
     */
    public void setMetadataProfile(MetadataProfile mdProfile) {
        _mdProfile = mdProfile;

        // Set Schema
        //File schemaFile = new File(folderMDSchema, mdProfile.getSchemaName());
        //setSchemaFile(schemaFile, fileHelperMD);

        // Set VocabFile
        File vocabFile = new File(ReloadDocumentHandler.folderMDVocab, mdProfile.getVocabularyFileName());
        setVocabFile(vocabFile);

        // Default Language
        setDefaultLanguage(defaultLanguage);
    }

    /**
     * Get the Metadata Profile
     * @return
     */
    public MetadataProfile getMetadataProfile() {
        // If we don't have one, set the default one
        if(_mdProfile == null) {
            try {
                setMetadataProfile(MetadataProfile.getMetadataProfile(ReloadDocumentHandler.defaultMDProfile));
            }
            catch(ReloadException ex) {
                ErrorDialogBox.showWarning("Could not load Default Metadata Profile", "Load Profile", ex);
            }
        }

        return _mdProfile;
    }

    /**
     * Return the Root Element name
     * @return
     */
    public String getRootElementName() {
        return Metadata.ROOT_NAME;
    }

    /**
     * @return true if nameSpaceURI is in our list of accepted Namespace URIs
     */
    public static boolean isAcceptedNamespaceURI(String nameSpaceURI) {
        for(int i = 0; i < ReloadDocumentHandler.IMSMD_NAMESPACE_URIS.length; i++) {
            if(nameSpaceURI.equals(ReloadDocumentHandler.IMSMD_NAMESPACE_URIS[i])) {
                return true;
            }
        }
        return false;
    }

    /**
     * Get the (9 in this case) top level IMS Element Categories
     * @return An Enumeration of ReloadSchemaElement objects
     */
    public ReloadSchemaElement[] getTopLevelElements() {
        return _reloadSchema.getRootElement().getChildren();
    }

    /**
     * Find out if the given Element is a top level one below "lom"
     * @param schemaElement The Schema Element to query
     * @return true if the parent of schemaElement is the root Element
     */
    public boolean isTopLevelElement(ReloadSchemaElement schemaElement) {
        return schemaElement.getParent() == _reloadSchema.getRootElement();
    }

    /**
     * Get The Leaf Icon
     */
    public Icon getLeafIcon(Element element) {
        String elementName = element.getName();
        if(elementName.equals(Metadata.ROOT_NAME)) return ReloadUIManager.getIcon(ReloadUIManager.ICON_MD);
        else return ReloadUIManager.getIcon(ReloadUIManager.ICON_MDLEAF);
    }

    /**
     * Get The Leaf Icon
     */
    public Icon getLeafIcon(ReloadSchemaElement schemaElement) {
        String elementName = schemaElement.getName();
        if(elementName.equals(Metadata.ROOT_NAME)) return ReloadUIManager.getIcon(ReloadUIManager.ICON_MD);
        else return ReloadUIManager.getIcon(ReloadUIManager.ICON_MDLEAF);
    }

    /**
     * Get The Open Icon
     */
    public Icon getOpenIcon(Element element) {
        String elementName = element.getName();
        if(elementName.equals(Metadata.ROOT_NAME)) return ReloadUIManager.getIcon(ReloadUIManager.ICON_MD);
        else return ReloadUIManager.getIcon(ReloadUIManager.ICON_MDFOLDER);
    }

    /**
     * Get The Open Icon
     */
    public Icon getOpenIcon(ReloadSchemaElement schemaElement) {
        String elementName = schemaElement.getName();
        if(elementName.equals(Metadata.ROOT_NAME)) return ReloadUIManager.getIcon(ReloadUIManager.ICON_MD);
        else return ReloadUIManager.getIcon(ReloadUIManager.ICON_MDFOLDER);
    }

    /**
     * Get The Closed Icon
     */
    public Icon getClosedIcon(Element element) {
        return getOpenIcon(element);
    }

    /**
     * Get The Closed Icon
     */
    public Icon getClosedIcon(ReloadSchemaElement schemaElement) {
        return getLeafIcon(schemaElement);
    }
}