/**
 *  RELOAD Editor - An IMS Metadata, Content Packaging and Learning Design tool.
 *
 *  Copyright (c) 2003 Oleg Liber, Bill Olivier, Phillip Beauvoir
 *
 *  Permission is hereby granted, free of charge, to any person obtaining a copy
 *  of this software and associated documentation files (the "Software"), to deal
 *  in the Software without restriction, including without limitation the rights
 *  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 *  copies of the Software, and to permit persons to whom the Software is
 *  furnished to do so, subject to the following conditions:
 *
 *  The above copyright notice and this permission notice shall be included in
 *  all copies or substantial portions of the Software.
 *
 *  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 *  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 *  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 *  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 *  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 *  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 *
 *  Project Management Contact:
 *
 *  Oleg Liber
 *  Bolton Institute of Higher Education
 *  Deane Road
 *  Bolton BL3 5AB
 *  UK
 *
 *  e-mail:   o.liber@bolton.ac.uk
 *
 *
 *  Technical Contact:
 *
 *  Phillip Beauvoir
 *  e-mail:   p.beauvoir@bolton.ac.uk
 *
 *  Web:      http://www.reload.ac.uk
 *
 */

package uk.ac.reload.prefs;

import java.io.*;

import org.jdom.*;
import uk.ac.reload.*;
import uk.ac.reload.util.*;

/**
 * The User Prefs file
 *
 * @author Phillip Beauvoir
 * @version 2003-07-31
 */
public class UserPrefs {
    // GENERAL KEYS

    // The default Open folder
    public static final String GENERAL_PREFS_DEFAULT_FOLDER = "default_folder";
    // Look and Feel
    public static final String GENERAL_PREFS_LF = "look_and_feel";

    // CP KEYS

    // Hide Resources Element
    public static final String CP_HIDE_RESOURCES = "hide_resources";

    // Hide SCORMS Elements
    public static final String CP_HIDE_SCORM = "hide_scorm";

    /**
     * The Reload Prefs Filename
     */
    public static final String USER_PREFS_FILENAME = "reload_prefs.xml";

    /**
     * The Reload Prefs main element name
     */
    public static final String USER_PREFS_ELEMENTNAME = "reload_prefs";

    /**
     * The JDOM Document
     */
    protected Document _doc;

    /**
     * The Shared Instance
     */
    private static UserPrefs _sharedInstance;

    /**
     * Private Constructor
     */
    private UserPrefs() {
        // Load it
        _doc = load();
        // Set defaults
        setDefaultValues();
    }

    /**
     * @return The Shared Instance of the UserPrefs
     */
    public static UserPrefs getSharedInstance() {
        if(_sharedInstance == null) _sharedInstance = new UserPrefs();
        return _sharedInstance;
    }

    /**
     * Load the Preferences File
     * @return the JDOM Document
     */
    protected Document load() {
        Document doc = null;

        // Get Prefs file
        File prefsFile = getPrefsFile();

        // If the file exists read it in
        if(prefsFile.exists()) {
            try {
                doc = XMLUtils.readXMLFile(prefsFile);
            }
            catch(ReloadException ex) {
                // Oops, so create a new one
                doc = createNewPrefsFile();
            }

            // Check it's ours
            if(doc != null && doc.hasRootElement()) {
                Element root = doc.getRootElement();
                // If not ours, create new one
                String rootName = root.getName();
                if(!rootName.equals(USER_PREFS_ELEMENTNAME)) {
                    doc = createNewPrefsFile();
                }
            }
        }

        // Else create a new one
        if(doc == null) doc = createNewPrefsFile();

        return doc;
    }

    /**
     * Save the Prefs file
     */
    public void save() throws ReloadException {
        File file = getPrefsFile();
        XMLUtils.write2XMLFile(_doc, file);
    }

    /**
     * Create a new JDOM Prefs Document with defaults
     * @return the JDOM Document
     */
    protected Document createNewPrefsFile() {
        Document doc = new Document();
        doc.setRootElement(new Element(USER_PREFS_ELEMENTNAME));
        return doc;
    }

    /**
     * Set some default values
     */
    protected void setDefaultValues() {
        String value = getValue(CP_HIDE_RESOURCES);
        if(value == null) putBooleanValue(CP_HIDE_RESOURCES, false);

        value = getValue(CP_HIDE_SCORM);
        if(value == null) putBooleanValue(CP_HIDE_SCORM, true);
    }

    /**
     * @return a value string given a key
     */
    public String getValue(String key) {
        String value = null;
        if(_doc != null && _doc.hasRootElement()) {
            Element element = _doc.getRootElement().getChild(key);
            if(element != null) return element.getText();
        }
        return null;
    }

    /**
     * @return a boolean value given a key, if it doesn't exist the default is false
     */
    public boolean getBooleanValue(String key) {
        String value = getValue(key);
        return value == null ? false : value.equals("true");
    }

    /**
     * Add a Boolean value
     */
    public void putBooleanValue(String key, boolean value) {
        putValue(key, value ? "true" : "false");
    }

    /**
     * Put a key and value
     */
    public void putValue(String key, String value) {
        if(_doc != null) {
            // Ensure we have a root element
            Element root = _doc.getRootElement();
            if(root == null) {
                root = new Element(USER_PREFS_ELEMENTNAME);
                _doc.setRootElement(root);
            }
            // Do we have it already?
            Element element = root.getChild(key);
            if(element != null) element.setText(value);
            // New one
            else {
                element = new Element(key);
                element.setText(value);
                root.addContent(element);
            }
        }
    }

    /**
     * Remove a value
     */
    public void removeValue(String key) {
        if(_doc != null) {
            Element root = _doc.getRootElement();
            if(root != null) {
                root.removeChild(key);
            }
        }
    }

    /**
     * @return the Preferences File
     */
    public File getPrefsFile() {
        File reloadHome = new File(System.getProperty("user.home"), "reload");
        reloadHome.mkdirs();
        return new File(reloadHome, USER_PREFS_FILENAME);
    }
}