/**
  *  RELOAD Editor - An IMS Metadata, Content Packaging and Learning Design tool.
  *
  *  Copyright (c) 2003 Oleg Liber, Bill Olivier, Phillip Beauvoir
  *
  *  Permission is hereby granted, free of charge, to any person obtaining a copy
  *  of this software and associated documentation files (the "Software"), to deal
  *  in the Software without restriction, including without limitation the rights
  *  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
  *  copies of the Software, and to permit persons to whom the Software is
  *  furnished to do so, subject to the following conditions:
  *
  *  The above copyright notice and this permission notice shall be included in
  *  all copies or substantial portions of the Software.
  *
  *  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  *  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  *  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
  *  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
  *  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
  *  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
  *
  *  Project Management Contact:
  *
  *  Oleg Liber
  *  Bolton Institute of Higher Education
  *  Deane Road
  *  Bolton BL3 5AB
  *  UK
  *
  *  e-mail:   o.liber@bolton.ac.uk
  *
  *
  *  Technical Contact:
  *
  *  Phillip Beauvoir
  *  e-mail:   p.beauvoir@bolton.ac.uk
  *
  *  Web:      http://www.reload.ac.uk
  *
  */

package uk.ac.reload.util;

import java.io.*;
import java.util.*;

import javax.swing.*;
import javax.xml.transform.*;
import javax.xml.transform.stream.*;

//import com.icl.saxon.*;
//import com.icl.saxon.jdom.*;
//import com.icl.saxon.expr.*;
//import com.icl.saxon.om.*;

import org.jdom.*;
import org.jdom.input.*;
import org.jdom.output.*;
import org.jdom.transform.*;

import uk.ac.reload.*;
import uk.ac.reload.widgets.*;
import uk.ac.reload.dom.*;

/**
 * Some useful XML Utilities that leverage the JDOM Package<br>
 *
 * @author Phillip Beauvoir
 * @version 2002-06-02
 */
public final class XMLUtils {

    /**
     * The default Sax Parser that JDOM will use.  This can be replaced by something else.
     */
    public static String DEFAULT_SAX_DRIVER_CLASS = "org.apache.xerces.parsers.SAXParser";


    /**
     * Writes a JDOM Document to file
     * @param doc The JDOM Document to write
     * @param file The file to write to
     * @throws ReloadException If there is any problem
     */
    public static void write2XMLFile(Document doc, File file) throws ReloadException {
        try {
            XMLOutputter outputter = new XMLOutputter("  ", true);
            outputter.setTextNormalize(true);  // This gets rid of junk characters
            FileOutputStream out = new FileOutputStream(file);
            outputter.output(doc, out);
            out.close();
        }
        catch(Exception ex) {
            throw new ReloadException(ex);
        }
    }

    /**
     * Reads and returns a JDOM Document from file with Schema validation
     * @param file The XML File
     * @param schemaNamespace The Schema Target Namespace
     * @param schemaLocation The Schema Location
     * @return The JDOM Document or null if not found
     * @throws ReloadException If there is any problem
     */
    public static Document readXMLFile(File file, String schemaNamespace, String schemaLocation) throws ReloadException {
        Document doc = null;

        try {
            SAXBuilder builder = new SAXBuilder(DEFAULT_SAX_DRIVER_CLASS, true);
            builder.setFeature("http://apache.org/xml/features/validation/schema", true);
            builder.setProperty("http://apache.org/xml/properties/schema/external-schemaLocation",
                                schemaNamespace + " " + schemaLocation);
            // changed - this allows UNC mapped locations to load
            // doc = builder.build(file);
            doc = builder.build(new FileInputStream(file));
        }
        catch(Exception ex) {
            throw new ReloadException(ex);
        }

        return doc;
    }

    /**
     * Reads and returns a JDOM Document from file without Schema Validation
     * @param file The XML File
     * @return The JDOM Document or null if not found
     * @throws ReloadException If there is any problem
     */
    public static Document readXMLFile(File file) throws ReloadException {
        Document doc = null;

        try {
            SAXBuilder builder = new SAXBuilder(DEFAULT_SAX_DRIVER_CLASS);
            // changed - this allows UNC mapped locations to load
            // doc = builder.build(file);
            doc = builder.build(new FileInputStream(file));
        }
        catch(Exception ex) {
            throw new ReloadException(ex);
        }

        return doc;
    }

    /**
     * Ask the user for a File name to save this Document as
     * @return File or null if cancelled
     */
    public static File askFileNameSave(String title, javax.swing.filechooser.FileFilter filter) {
        // Ask for the File Name
        ReloadFileChooser chooser = new ReloadFileChooser();
        chooser.setDialogTitle(title);
        chooser.setFileFilter(filter);
        int returnVal = chooser.showSaveDialog(ReloadEditor.getInstance());

        // User Cancelled
        if(returnVal != JFileChooser.APPROVE_OPTION) return null;

        // Get the chosen File
        File file = chooser.getSelectedFileAndStore();

        // Ensure we have an extension
        String fileName = file.getPath();
        if(fileName.indexOf(".") == -1) file = new File(fileName + ".xml");

        return file;
    }

    /**
     * Ask the user for a File name to open
     * @return File or null if cancelled
     */
    public static File askFileNameOpen(String title, javax.swing.filechooser.FileFilter filter) {
        // Ask for the File Name
        ReloadFileChooser chooser = new ReloadFileChooser();
        chooser.setDialogTitle(title);
        chooser.setFileFilter(filter);
        int returnVal = chooser.showOpenDialog(ReloadEditor.getInstance());

        // User Cancelled
        if(returnVal != JFileChooser.APPROVE_OPTION) return null;

        // Get the chosen File
        return chooser.getSelectedFileAndStore();
    }


    /**
     * Ask the user for a File name to open, the name of the current dir is given
     * @return File or null if cancelled
     */
    public static File askFileNameOpenUsingDirName(String title, javax.swing.filechooser.FileFilter filter, File currentDir) {
        // Ask for the File Name
        ReloadFileChooser chooser = new ReloadFileChooser();
        chooser.setDialogTitle(title);
        chooser.setFileFilter(filter);
        chooser.setCurrentDirectory(currentDir);
        int returnVal = chooser.showOpenDialog(ReloadEditor.getInstance());

        // User Cancelled
        if(returnVal != JFileChooser.APPROVE_OPTION) return null;

        // Get the chosen File
        return chooser.getSelectedFileAndStore();
    }


	/**
	* Tina Manoharan
	* 2003-09-09
	*/
    /**
     * Gets the value of an element from its name
     * @return value of element or null if element not found
     */
    public static String getElementValueByName(String rootName, DomPath domPath, File xmlFile) {

		Document doc;

		Element element;

		try {

			doc = readXMLFile(xmlFile);

			element = doc.getRootElement();

			StringTokenizer st = domPath.getElements();

			// Now work thru the path
			while(st.hasMoreTokens()) {

				// Get next Element name
				String name = st.nextToken();

				// igonore Root element
				if(name.equals(rootName)) continue;

				element = element.getChild(name, element.getNamespace());

				// Not found
				if(element == null) break;
			}

			if(element != null) return element.getTextTrim();
		}
		catch(ReloadException re) {

			if(ReloadEditor.DEBUG) System.out.println("Exception when getting value of element: " + domPath.getPath());
		}

		return null;
    }

    /**
     * Gets the an element using domPath and its value
     * @return true or false depending on whether element was found or not
     */
    public static boolean getElementByNameValuePair(String rootName, DomPath domPath, File xmlFile, String value) {

		Document doc;

		Element element;

		try {

			doc = readXMLFile(xmlFile);

			element = doc.getRootElement();

			StringTokenizer st = domPath.getElements();

			// Now work thru the path
			while(st.hasMoreTokens()) {

				// Get next Element name
				String name = st.nextToken();

				// igonore Root element
				if(name.equals(rootName)) continue;

				element = element.getChild(name, element.getNamespace());

				// Not found
				if(element == null) break;
			}

			if(element != null && element.getText().equals(value)) return true;
		}
		catch(ReloadException re) {

			if(ReloadEditor.DEBUG) System.out.println("Exception when getting an element using domPath and value: " + domPath.getPath());
		}

		return false;
    }


    /**
     * Identifies the file using domPath and its value
     * @return the true/false depending on whether element was found or not
     */
    public static boolean getFileByNameValuePair(String rootName, DomPath domPath, File xmlFile, String value) {

		Document doc;

		Element element;

		try {

			doc = readXMLFile(xmlFile);

			element = doc.getRootElement();

			StringTokenizer st = domPath.getElements();

			// Now work thru the path
			while(st.hasMoreTokens()) {

				// Get next Element name
				String name = st.nextToken();

				// igonore Root element
				if(name.equals(rootName)) continue;

				element = element.getChild(name, element.getNamespace());

				// Not found
				if(element == null) break;
			}

			if(element != null && element.getText().equals(value)) return true;
		}
		catch(ReloadException re) {

			if(ReloadEditor.DEBUG) System.out.println("Exception when getting the document using domPath and value: " + domPath.getPath());
		}

		return false;
    }


    /**
     * Gets all values of a particular element using its name from the file
     * @return all values of the element
     */
    public static ArrayList getElementValuesFromFileByName(File source, File stylesheet) {

		ArrayList result = new ArrayList();

		try {

			Document sourceDoc = readXMLFile(source);

	    	// Now do a transformation

			Transformer transformer = TransformerFactory.newInstance().newTransformer(new StreamSource(stylesheet));


			// create result by performing transformation using stylesheet

			JDOMSource sourceJdom = new JDOMSource(sourceDoc);

			JDOMResult resultJdom = new JDOMResult();

			transformer.transform(sourceJdom, resultJdom);

			Document resultDoc = resultJdom.getDocument();


    		if(resultDoc != null) {

				Element root = resultDoc.getRootElement();

				List children = root.getChildren("item");

				for(int i = 0; i < children.size(); i++) {

					Element element = (Element) children.get(i);

					String item = element.getTextTrim();

					if(item != null && !item.equals("") && !result.contains(item)) result.add(item);
				}
			}

		}
        catch(ReloadException rex) {
            if(ReloadEditor.DEBUG) rex.printStackTrace();
            ErrorDialogBox.showWarning("Import Zthes Vocabulary as Vdex", "Error importing", rex);
        }
        catch(TransformerConfigurationException tex) {
            if(ReloadEditor.DEBUG) tex.printStackTrace();
        }
        catch(TransformerException tex2) {
            if(ReloadEditor.DEBUG) tex2.printStackTrace();
        }

		return result;
    }

	/*
    public static ArrayList getElementValuesFromFileByName(File source, String stylesheet) {

		Document doc;

		ArrayList result = new ArrayList();

		try {

			doc = readXMLFile(source);

			// Give it a Saxon wrapper
	    	DocumentWrapper saxonDocWrap = new DocumentWrapper(doc, source.getAbsolutePath());
	    	NamePool pool = NamePool.getDefaultNamePool();
	    	saxonDocWrap.setNamePool(pool);

	    	// Now do a transformation

	    	System.setProperty("javax.xml.transform.TransformerFactory", "com.icl.saxon.TransformerFactoryImpl");
	    	TransformerFactory tfactory = TransformerFactory.newInstance();

	    	Templates templates = tfactory.newTemplates(new StreamSource(stylesheet));
	    	Transformer transformer = templates.newTransformer();

			JDOMResult tempOutput = new JDOMResult();

	    	transformer.transform(saxonDocWrap, tempOutput);

	    	doc = tempOutput.getDocument();

	    	if(doc != null) {

				Element root = doc.getRootElement();

				List children = root.getChildren("item");

				for(int i = 0; i < children.size(); i++) {

					Element element = (Element) children.get(i);

					String item = element.getTextTrim();

					if(item != null && !item.equals("") && !result.contains(item)) result.add(item);
				}
			}
		}
        catch(ReloadException rex) {
            if(ReloadEditor.DEBUG) rex.printStackTrace();
            ErrorDialogBox.showWarning("Import Zthes Vocabulary as Vdex", "Error importing", rex);
        }
        catch(TransformerConfigurationException tex) {
            if(ReloadEditor.DEBUG) tex.printStackTrace();
        }
        catch(TransformerException tex2) {
            if(ReloadEditor.DEBUG) tex2.printStackTrace();
        }

		return result;
    }

	*/
}


