/**
  *  RELOAD Editor - An IMS Metadata, Content Packaging and Learning Design tool.
  *
  *  Copyright (c) 2003 Oleg Liber, Bill Olivier, Phillip Beauvoir
  *
  *  Permission is hereby granted, free of charge, to any person obtaining a copy
  *  of this software and associated documentation files (the "Software"), to deal
  *  in the Software without restriction, including without limitation the rights
  *  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
  *  copies of the Software, and to permit persons to whom the Software is
  *  furnished to do so, subject to the following conditions:
  *
  *  The above copyright notice and this permission notice shall be included in
  *  all copies or substantial portions of the Software.
  *
  *  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  *  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  *  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
  *  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
  *  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
  *  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
  *
  *  Project Management Contact:
  *
  *  Oleg Liber
  *  Bolton Institute of Higher Education
  *  Deane Road
  *  Bolton BL3 5AB
  *  UK
  *
  *  e-mail:   o.liber@bolton.ac.uk
  *
  *
  *  Technical Contact:
  *
  *  Phillip Beauvoir
  *  e-mail:   p.beauvoir@bolton.ac.uk
  *
  *  Web:      http://www.reload.ac.uk
  *
  */

package uk.ac.reload.util;

import java.io.*;
import java.util.*;
import java.util.zip.*;

import javax.swing.*;

import uk.ac.reload.*;
import uk.ac.reload.contentpackaging.*;
import uk.ac.reload.widgets.*;

/**
 * Some useful Zip Utilities
 *
 * @author Phillip Beauvoir
 * @author Paul Sharples
 * @version 2002-07-24
 */
public final class ZipUtils {

    /**
     * Ask the user for a File name to save this Zip as
     * @return File or null if cancelled
     */
    public static File askFileNameSave(String title, javax.swing.filechooser.FileFilter filter) {
        // Ask for the File Name
        ReloadFileChooser chooser = new ReloadFileChooser();
        chooser.setDialogTitle(title);
        chooser.setFileFilter(filter);
        int returnVal = chooser.showSaveDialog(ReloadEditor.getInstance());

        // User Cancelled
        if(returnVal != JFileChooser.APPROVE_OPTION) return null;

        // Get the chosen File
        File file = chooser.getSelectedFileAndStore();

        // Ensure we have an extension
        String fileName = file.getPath();
        if(fileName.indexOf(".") == -1) file = new File(fileName + ".zip");

        return file;
    }

    /**
     * Save a CP as a Zip file
     */
    public static void zipContentPackage(ContentPackage cp, File zipFile, ReloadProgressMonitor progressMonitor) throws ReloadException {
        try {
            BufferedOutputStream out = new BufferedOutputStream(new FileOutputStream(zipFile));
            ZipOutputStream zOut = new ZipOutputStream(out);

            addFolderToZip(cp.getProjectFolder(), cp.getProjectFolder(), zOut, zipFile);

            zOut.flush();
            zOut.close();
        }
        catch(Exception ex) {
            throw new ReloadException("Could not zip");
        }
    }

    /**
     * Add all files and sub-files to the Zip
     */
    public static void addFolderToZip(File rootFolder, File folder, ZipOutputStream zOut, File zipFile) throws ReloadException {
        File[] files = folder.listFiles();
        for(int i = 0; i < files.length; i++) {
            // Sub-folder
            if(files[i].isDirectory()) addFolderToZip(rootFolder, files[i], zOut, zipFile);
            // Careful not to add the zip file to the zip file!
            else if(!files[i].equals(zipFile)) {
                String entryName = ReloadUtils.getRelativePath(rootFolder, files[i]);
                addFileToZip(files[i], entryName, zOut);
                //if(File.separatorChar != '/') entryName = entryName.replace(File.separatorChar, '/');
            }
        }
    }

    /**
     * Add all .xsd (Schema) files to zip file
     */
/*
    private static void addSchemaFilesToZip(File rootFolder, File folder, ZipOutputStream zOut) throws ReloadException {
        File[] files = folder.listFiles();
        for(int i = 0; i < files.length; i++) {
            if(files[i].isDirectory()) addSchemaFilesToZip(rootFolder, files[i], zOut);
            else if(ReloadUtils.getFileExtension(files[i]).equals("xsd")) {
                String entryName = ReloadUtils.getRelativePath(rootFolder, files[i]);
                addFileToZip(files[i], entryName, zOut);
            }
        }
    }
*/


    /**
     * Adds a file to the Zip file
     */
    public static boolean addFileToZip(File file, String entryName, ZipOutputStream zOut) throws ReloadException {
        if(file == null || !file.exists()) return false;

        int bytesRead;
        final int bufSize = 1024;
        byte buf[] = new byte[bufSize];

        try {
            ZipEntry zipEntry = new ZipEntry(entryName);
            // Set time stamp to file's
            zipEntry.setTime(file.lastModified());
            zOut.putNextEntry(zipEntry);
            BufferedInputStream in = new BufferedInputStream(new FileInputStream(file), bufSize);
            while((bytesRead = in.read(buf)) != -1) {
                zOut.write(buf, 0, bytesRead);
            }
            zOut.closeEntry();
            in.close();
        }
        catch(Exception ex) {
            if(ReloadEditor.DEBUG) ex.printStackTrace();
            throw new ReloadException("Could not add file to zip");
        }
        return true;
    }

    /**
     * Adds a String as a field entry to the Zip file
     */
    public static boolean addStringToZip(String text, String entryName, ZipOutputStream zOut) throws ReloadException {
        try {
            BufferedReader reader = new BufferedReader(new StringReader(text));
            ZipEntry zipEntry = new ZipEntry(entryName);
            zOut.putNextEntry(zipEntry);
            int i;
            while((i = reader.read()) != -1) {
                zOut.write(i);
            }
            zOut.closeEntry();
        }
        catch(Exception ex) {
            throw new ReloadException("Could not add String to zip");
        }
        return true;
    }

    /**
     * Extracts a named entry out of the zip file and returns the entry as a String
     * Returns null if weirdness happens
     */
    public static String extractZipEntry(File zipFile, String entryName) throws ReloadException {
        if(zipFile == null || !zipFile.exists()) return null;

        ZipEntry zipEntry;
        ZipInputStream zIn;
        int bit;
        boolean foundEntry = false;
        StringBuffer sb;

        try {
            BufferedInputStream in = new BufferedInputStream(new FileInputStream(zipFile));
            zIn = new ZipInputStream(in);

            // Get zip entry
            while((zipEntry = zIn.getNextEntry()) != null) {
                String zipEntryName = zipEntry.getName();
                if(zipEntryName.equalsIgnoreCase(entryName)) {
                    foundEntry = true;
                    break;
                }
                zIn.closeEntry();
            }
        }
        catch(Exception ex) {
            ex.printStackTrace();
            throw new ReloadException("Could not extract zip entry");
        }

        // If we didn't get it return
        if(foundEntry == false) {
            try{
                zIn.close();
            }
            catch(IOException ex) {}
            return null;
        }

        sb = new StringBuffer();

        // Extract it
        try {
            while((bit = zIn.read()) != -1) {
                sb.append((char)bit);
            }

            zIn.close();
        }
        catch(Exception ex) {
            if(ReloadEditor.DEBUG) ex.printStackTrace();
            throw new ReloadException("Could not extract zip entry");
        }


        return sb.toString();
    }


    /**
     * Extracts a named entry out of the zip file to the specified file
     * Returns the File ref if OK or null if weirdness happens
     */
    public static File extractZipEntry(File zipFile, String entryName, File outFile) throws ReloadException {
        if(zipFile == null || !zipFile.exists()) return null;

        ZipInputStream zIn;
        ZipEntry zipEntry;
        int bytesRead;
        final int bufSize = 1024;
        byte buf[] = new byte[bufSize];
        boolean foundEntry = false;

        try {
            // Ensure that the parent Folder exists
            if(!outFile.getParentFile().exists()) {
                outFile.getParentFile().mkdirs();
            }

            BufferedInputStream in = new BufferedInputStream(new FileInputStream(zipFile), bufSize);
            zIn = new ZipInputStream(in);

            // Find the entry
            while((zipEntry = zIn.getNextEntry()) != null) {
                String zipEntryName = zipEntry.getName();
                if(zipEntryName.equalsIgnoreCase(entryName)) {
                    foundEntry = true;
                    break;
                }
                zIn.closeEntry();
            }
        }
        catch(Exception ex) {
            throw new ReloadException("Could not extract zip entry");
        }

        // If we didn't get it return
        if(foundEntry == false) return null;

        // Extract it and save to outFile
        try {
            BufferedOutputStream out = new BufferedOutputStream(new FileOutputStream(outFile), bufSize);

            while((bytesRead = zIn.read(buf)) != -1) {
                out.write(buf, 0, bytesRead);
                // Allow other things to happen
                Thread.sleep(2);
            }

            out.flush();
            out.close();
            zIn.close();

            // Restore time stamp
            outFile.setLastModified(zipEntry.getTime());
        }
        catch(Exception ex) {
            if(ReloadEditor.DEBUG) ex.printStackTrace();
            throw new ReloadException("Could not save zip entry");
        }

        return outFile;
    }

    /**
     * Gets all file names out of a zip file
     * Returns an array of String names or null if none
    */
    public static String[] getZipFileEntryNames(File zipFile) throws ReloadException {
        if(zipFile == null || !zipFile.exists()) return null;
        ZipInputStream zIn = null;
        ZipEntry zipEntry;
        final int bufSize = 1024;
        Vector fileList = new Vector();

        try {
            BufferedInputStream in = new BufferedInputStream(new FileInputStream(zipFile), bufSize);
            zIn = new ZipInputStream(in);
            while((zipEntry = zIn.getNextEntry()) != null) {
                // Don't add directories
                if(!zipEntry.isDirectory()) {
                    String zipEntryName = zipEntry.getName();
                    fileList.addElement(zipEntryName);
                }
                zIn.closeEntry();
            }
        }
        catch(Exception ex) {
            throw new ReloadException("Could not locate zip entry");
        }
        finally {
            try {
                if(zIn != null) zIn.close();
            }
            catch(IOException ex) { }
        }

        String[] names = new String[fileList.size()];
        fileList.copyInto(names);
        return names;
    }

    /**
     * Extracts all entries out of the zip file to the specified folder
     * Returns true or false to indicate if the progress cancel was pressed
     */
    public static boolean unpackZip(File zipFile, File folder, ReloadProgressMonitor progressMonitor) throws ReloadException {
       if(zipFile == null || !zipFile.exists()) return false;

       folder.mkdirs();

       BufferedInputStream in = null;
       BufferedOutputStream out = null;
       ZipInputStream zIn = null;
       ZipEntry zipEntry;
       int bytesRead;
       final int bufSize = 512;
       byte buf[] = new byte[bufSize];

       try {
           in = new BufferedInputStream(new FileInputStream(zipFile), bufSize);
           zIn = new ZipInputStream(in);

           while((zipEntry = zIn.getNextEntry()) != null) {
               // Don't add directories
               if(!zipEntry.isDirectory()) {
                   File outFile = new File(folder, zipEntry.getName());
                   // Ensure that the parent Folder exists
                   if(!outFile.getParentFile().exists()) outFile.getParentFile().mkdirs();

                   out = new BufferedOutputStream(new FileOutputStream(outFile), bufSize);

                   // If we have a Progress Monitor
                   if(progressMonitor != null) progressMonitor.setNote(zipEntry.getName());

                   int sleep_count = 0;

                   while((bytesRead = zIn.read(buf)) != -1) {
                       out.write(buf, 0, bytesRead);
                       // Allow other things to happen every 40 chunks
                       if(sleep_count >= 40) {
                           Thread.sleep(2);
                           sleep_count = 0;
                       }
                       sleep_count++;
                       // If we have a Progress Monitor
                       if(progressMonitor != null && progressMonitor.isCanceled()) {
                           out.flush();
                           out.close();
                           zIn.close();
                           return false;
                       }
                   }

                   // Restore time stamp
                   outFile.setLastModified(zipEntry.getTime());

                   // Close File
                   out.flush();
                   out.close();
               }

               zIn.closeEntry();
           }

           zIn.close();
       }
       catch(Exception ex) {
           try {
               if(zIn != null) zIn.close();
               if(out != null) {
                   out.flush();
                   out.close();
               }
           }
           catch(IOException ex1) { }
           if(ReloadEditor.DEBUG) ex.printStackTrace();
           throw new ReloadException("Could not save zip entry");
       }

       return true;
   }
}