/**
  *  RELOAD Editor - An IMS Metadata, Content Packaging and Learning Design tool.
  *
  *  Copyright (c) 2003 Oleg Liber, Bill Olivier, Phillip Beauvoir, Phil Barker, Tina Manoharan
  *
  *  Permission is hereby granted, free of charge, to any person obtaining a copy
  *  of this software and associated documentation files (the "Software"), to deal
  *  in the Software without restriction, including without limitation the rights
  *  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
  *  copies of the Software, and to permit persons to whom the Software is
  *  furnished to do so, subject to the following conditions:
  *
  *  The above copyright notice and this permission notice shall be included in
  *  all copies or substantial portions of the Software.
  *
  *  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  *  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  *  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
  *  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
  *  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
  *  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
  *
  *  Project Management Contact:
  *
  *  Oleg Liber
  *  Bolton Institute of Higher Education
  *  Deane Road
  *  Bolton BL3 5AB
  *  UK
  *
  *  e-mail:   o.liber@bolton.ac.uk
  *
  *  Phil Barker
  *  ICBL, Heriot-Watt University
  *  Riccarton
  *  Edinburgh EH14 4AS
  *  UK
  *
  *  e-mail:   philb@icbl.hw.ac.uk
  *
  *
  *  Technical Contact:
  *
  *  Phillip Beauvoir
  *  e-mail:   p.beauvoir@bolton.ac.uk
  *
  *  Web:      http://www.reload.ac.uk
  *
  *  Tina Manoharan
  *  e-mail:   ceetm2@macs.hw.ac.uk
  *
  */

package uk.ac.reload.vocabulary;

import java.io.*;
import java.util.*;

import org.jdom.*;
import uk.ac.reload.*;
import uk.ac.reload.dom.*;
import uk.ac.reload.schema.*;
import uk.ac.reload.util.*;

/**
 * The Vocabulary Class
 *
 */
public class Vocabulary
extends AbstractVocabulary
{
    public static final String ROOT_NAME = "vdex";

    /**
     * Our unique signature
     */
    static final String[] vx_comments = {
        "This is a Reload version " + ReloadEditor.VERSION + " Vocabulary document",
        "Spawned from the Reload Vocabulary Generator - http://www.reload.ac.uk"
    };

    /**
     * Whether we are a standalone file or attached
     */
    protected boolean _isStandalone;

    /**
     * Constructor
     */
    public Vocabulary(boolean isStandalone) {
        super();
        _isStandalone = isStandalone;
    }

    /**
     * Setup the JDOM Document with Namespaces and root element
     */
    public void newDocument(VocabularySchemaController controller) {

        _doc = new Document();

        _controller = controller;

        // Add our signature
        if(_isStandalone) addComments(_doc);

        // Add Root Element according to Schema
        ReloadSchema vxSchema = _controller.getReloadSchema();

        Element root = addElementBySchema(this, null, vxSchema.getRootElement(), true);
        _doc.setRootElement(root);

        // Add Root Declarations
        if(_isStandalone) addRootDeclarations(root);

        // Clear the dirty flag
        setDirty(false);
    }


    /**
	* Update the JDOM Document with the new Schema Namespaces and root element
	*/
	public void updateDocument(VocabularySchemaController controller) {

	    _controller = controller;

	    // Add Root Declarations
	    if(_isStandalone) addRootDeclarations(getRootElement());

	    // remove elements from document which are not in the schema
	    removeElementsAndAttributesBySchema(getRootElement());

	    // Set the dirty flag
	    setDirty(true);
    }

    /**
    * Remove elements/attributes from the JDOM document which are not in the ReloadSchema
	*/
	public void removeElementsAndAttributesBySchema(Object obj) {

		Element element;

		Attribute attribute;

		DomPath domPath;

		ReloadSchemaNode schemaNode;

		if(obj instanceof Element) {

			element = (Element) obj;

			domPath = DomPath.getDomPath(element);

			if(domPath != null) {

				schemaNode = _controller.getReloadSchemaNode(domPath);

				if(schemaNode != null) {

					if(schemaNode instanceof ReloadSchemaElement) {

						ReloadSchemaElement schemaElement = (ReloadSchemaElement) schemaNode;

						//if(!element.isRootElement()) {

							// checking all document attributes if they are in the schema
							List attributeList = element.getAttributes();

							Attribute[] attributes = (Attribute[]) attributeList.toArray(new Attribute[attributeList.size()]);

							for(int i = 0; i < attributes.length; i++) {

								Attribute attrib = attributes[i];

								removeElementsAndAttributesBySchema(attrib);
							}
						//}


						// Checking all ducument elements if they are in the schema
						List childrenList = element.getChildren();

						Element[] children = (Element[]) childrenList.toArray(new Element[childrenList.size()]);

						for(int i = 0; i < children.length; i++) {

							Element child = children[i];

							removeElementsAndAttributesBySchema(child);
						}
					}
				}
				else {

					// remove the element and inform all document listeners
					// removeElement(Vocabulary.this, element);
					deleteElementUndoable(Vocabulary.this, element);
				}
			}
		}
		else if(obj instanceof Attribute) {

			attribute = (Attribute) obj;

			domPath = DomPath.getDomPath(attribute);

			if(domPath != null) {

				schemaNode = _controller.getReloadSchemaNode(domPath);

				if(schemaNode != null) {

					if(schemaNode instanceof ReloadSchemaAttribute) {

						ReloadSchemaAttribute schemaAttribute = (ReloadSchemaAttribute) schemaNode;

						// get the attribute's vocabulary list

						ReloadVocabularyList rvList = schemaAttribute.getVocabularyList();

						if(rvList != null) {

							String[] vocabs = rvList.getList();

							ArrayList vocabList = new ArrayList(Arrays.asList(vocabs));

							if(attribute.getValue() != null) {

								// if the current value is not in the list then get the default value and set it

								if(!vocabList.contains(attribute.getValue())) { // if attribute values is not in the vocab list

									if(schemaAttribute.getDefaultValue() != null) {

										attribute.setValue(schemaAttribute.getDefaultValue());
									}
									else if(rvList.getDefaultValue() != null) {

										attribute.setValue(rvList.getDefaultValue());
									}
									else attribute.detach();
								}
								else { // if attribute value is in the vocab list

									if(schemaAttribute.getDefaultValue() != null) {

										attribute.setValue(schemaAttribute.getDefaultValue());
									}
								}
							}
							else attribute.detach();
						}
					}
				}
				else attribute.detach();
			}
		}
	}


    /**
     * @return the Root element name
     */
    public String getRootElementName() {
        return ROOT_NAME;
    }

    /**
     * @return the Comments to add to the XML Document
     */
    public String[] getComments() {
        return vx_comments;
    }

	/**
	* Tina Manoharan
	* 2003-08-11
	*/
    /**
     * Add namespace and other attribute decalarations to the root element
     * @param root
     */
    public void addRootDeclarations(Element root) {
        ReloadSchema vxSchema = getReloadSchemaController().getReloadSchema();

        // Add XSI Schema Instance Namespace
        root.addNamespaceDeclaration(ReloadDocumentHandler.XSI_Namespace);

        // Add Schema Location Attribute which is constructed from Target Namespace
        // and file name of Schema
        String schemaLocationURI = vxSchema.getTargetNamespace() + " " + vxSchema.getSchemaName();

        root.setAttribute(ReloadDocumentHandler.XSI_SchemaLocation, schemaLocationURI, ReloadDocumentHandler.XSI_Namespace);
    }

    /**
     * Set the Vocabulary to a VDEX Element from a Content Package
     */
    public void setVocabularyElement(Element vxElement, VocabularySchemaController controller) {
        _doc = new Document();
        _controller = controller;
        _doc.setRootElement(vxElement);
    }

    /**
     * Set the Vocabulary to an Imported Vocabulary Doc
     */
    public void importVocabulary(Document doc, VocabularySchemaController controller) throws ReloadException {
        // Get the Root Element
        Element root = (Element)doc.getRootElement();

        // Clear any Attributes
        root.setAttributes(null);

        // Add Namespace prefixes to all Elements
        Namespace ns = root.getNamespace();
        if(ns != null) ReloadDocumentHandler.replaceNamespaces(root, ns, getSchemaNamespaceEmbedded());

        _doc = doc;
        _controller = controller;
    }

    /**
     * Export the VX from a CP node to a standalone file
     * @param file
     * @throws ReloadException
     */
    public void exportVocabulary(File file) throws ReloadException {
        if(!isStandalone() && _doc != null) {
            Document newDoc = new Document();
            // Add comments
            addComments(newDoc);

            // Clone
            Element root = (Element)_doc.getRootElement().clone();

            // Remove Namespace prefixes from all Elements
            Namespace ns = root.getNamespace();
            if(ns != null) ReloadDocumentHandler.replaceNamespaces(root, ns, Namespace.getNamespace(ns.getURI()));

            // Set to root
            newDoc.setRootElement(root);

            // Add root declarations
            addRootDeclarations(root);

            XMLUtils.write2XMLFile(newDoc, file);
            newDoc = null;
        }
    }

    /**
     * Return whether we are a standalone file
     * @return
     */
    public boolean isStandalone() {
        return _isStandalone;
    }

    /**
     * Get the appropriate Namespace
     */
    public Namespace getSchemaNamespace() {
        if(_isStandalone) return super.getSchemaNamespace();
        else return getSchemaNamespaceEmbedded();
    }

    /**
     * Get the standalone Namespace
     */
    public Namespace getSchemaNamespaceStandalone() {
        return super.getSchemaNamespace();
    }

    /**
     * Get the embedded Namespace
     */
    public Namespace getSchemaNamespaceEmbedded() {
        return Namespace.getNamespace(ReloadDocumentHandler.IMSVX_NAMESPACE_PREFIX, getSchemaNamespaceStandalone().getURI());
    }

// ==========================================================================
// ELEMENT MOVING/COPYING/DRAGGING HANDLING
// ==========================================================================

    /**
     * @return whether we can copy an element
     */
    public boolean canCopyElement(Element element) {
        return !element.isRootElement();
    }

    /**
     * @return whether we can cut an element
     */
    public boolean canCutElement(Element element, ReloadSchemaElement schemaElement) {
        return canDeleteElement(element, schemaElement);
    }

    /**
     * @return whether we can paste an element
     */
    public boolean canPasteFromClipboard(Element element) {
        Element clipboardElement = ReloadDocumentClipboard.getElement();
        return clipboardElement != null && isAllowedChild(element, clipboardElement);
    }

}