/**
  *  RELOAD Editor - An IMS Metadata, Content Packaging and Learning Design tool.
  *
  *  Copyright (c) 2003 Oleg Liber, Bill Olivier, Phillip Beauvoir, Phil Barker, Tina Manoharan
  *
  *  Permission is hereby granted, free of charge, to any person obtaining a copy
  *  of this software and associated documentation files (the "Software"), to deal
  *  in the Software without restriction, including without limitation the rights
  *  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
  *  copies of the Software, and to permit persons to whom the Software is
  *  furnished to do so, subject to the following conditions:
  *
  *  The above copyright notice and this permission notice shall be included in
  *  all copies or substantial portions of the Software.
  *
  *  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  *  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  *  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
  *  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
  *  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
  *  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
  *
  *  Project Management Contact:
  *
  *  Oleg Liber
  *  Bolton Institute of Higher Education
  *  Deane Road
  *  Bolton BL3 5AB
  *  UK
  *
  *  e-mail:   o.liber@bolton.ac.uk
  *
  *  Phil Barker
  *  ICBL, Heriot-Watt University
  *  Riccarton
  *  Edinburgh EH14 4AS
  *  UK
  *
  *  e-mail:   philb@icbl.hw.ac.uk
  *
  *
  *  Technical Contact:
  *
  *  Phillip Beauvoir
  *  e-mail:   p.beauvoir@bolton.ac.uk
  *
  *  Web:      http://www.reload.ac.uk
  *
  *  Tina Manoharan
  *  e-mail:   ceetm2@macs.hw.ac.uk
  *
  */

package uk.ac.reload.vocabulary;

import java.io.*;
import java.util.*;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;

import org.jdom.*;
import uk.ac.reload.*;
import uk.ac.reload.menu.*;
import uk.ac.reload.vocabulary.formview.*;
import uk.ac.reload.vocabulary.tableview.*;
import uk.ac.reload.widgets.*;

/**
 * The Vocabulary Editor
 *
 */
public class VocabularyEditor
extends JPanel
{

    /**
     * The Vocabulary Document
     */
    protected Vocabulary _vocabulary;

    /**
     * The Prefs panel
     */
    protected VocabularyPrefsPanel _prefsPanel;

    /**
     * The Form View
     */
    protected VocabularyFormPanel _formPanel;

    /**
     * The Table View
     */
    protected VocabularyTablePanel _tablePanel;

    /**
     * The Tab Pane
     */
    protected JTabbedPane _tabPane;

    /**
     * The Edit Menu
     */
    protected Menu_Edit _editMenu;

    /**
     * The Undo Menu Manager for this Window
     */
    protected ReloadUndoMenuManager _undoMenuManager;



    /**
     * Constructor
     */
    public VocabularyEditor(Menu_Edit editMenu) {
        _editMenu = editMenu;

        setLayout(new BorderLayout());

        // Get a new Form View
        _formPanel = new VocabularyFormPanel(this);

        // Get a new Table View
        _tablePanel = new VocabularyTablePanel(editMenu);

        // Get a Prefs panel
        _prefsPanel = new VocabularyPrefsPanel();

        // Add Prefs Panel and Form View to another Panel
        JPanel p = new JPanel(new BorderLayout());
        p.add(_prefsPanel, BorderLayout.NORTH);
        p.add(new JScrollPane(_formPanel), BorderLayout.CENTER);

        // Tab Pane
        _tabPane = new JTabbedPane();
        _tabPane.addTab("Form", null, p, "Form View");
        _tabPane.addTab("Full", null, _tablePanel, "Full View");
        add(_tabPane, BorderLayout.CENTER);
    }

    /**
     * Get the Vocabulary Document
     * @return
     */
    public Vocabulary getVocabulary() {
        return _vocabulary;
    }

    public void setProfileType(String profileType) {

		_prefsPanel.setProfileType(profileType);
	}

	public String getProfileType() {

		return _prefsPanel.getProfileType();
	}

    /**
     * Load a new Vocabulary JDOM Document
     * @param vocabulary The Vocabulary JDOM Document
     * @throws ReloadException
     */
    public void newDocument(boolean standalone, VocabularySchemaController schemaController) throws ReloadException {

        _vocabulary = new Vocabulary(standalone);
        _vocabulary.newDocument(schemaController);

        init(_vocabulary);
    }

    /**
     * Set a Vocabulary JDOM Document - this will be standalone
     * @throws ReloadException
     */
    public void setDocument(File file, Document doc, VocabularySchemaController schemaController) throws ReloadException {
        _vocabulary = new Vocabulary(true);
        _vocabulary.setDocument(file, doc, schemaController);
        init(_vocabulary);
    }

    /**
     * Set the Vocabulary to a VX Element from a CP
     * @param vxElement
     */
    public void setVocabularyElement(Element vxElement, VocabularySchemaController schemaController) {
        _vocabulary = new Vocabulary(false);
        _vocabulary.setVocabularyElement(vxElement, schemaController);
        init(_vocabulary);
    }

    /**
     * Set the Vocabulary to an Imported VX Doc for a CP
     * @param vxElement
     */
    public void importVocabulary(Document doc, VocabularySchemaController schemaController) throws ReloadException {
        //_vocabulary = new Vocabulary(false);
        _vocabulary.importVocabulary(doc, schemaController);
        _vocabulary.setDirty(true);
        init(_vocabulary);
    }

    /**
     * Export the Vocabulary to a File
     * @param vxElement
     */
    public void exportVocabulary(File file) throws ReloadException {
        if(_vocabulary != null) _vocabulary.exportVocabulary(file);
    }

    /**
     * Initialise some stuff
     */
    private void init(Vocabulary vocabulary) {

        // Tell Prefs Panel
        _prefsPanel.init(vocabulary);

        _formPanel.setDocument(vocabulary);
        _tablePanel.setDocument(vocabulary);

        // New Undo Menu Manager
        _undoMenuManager = new ReloadUndoMenuManager(_editMenu);
        vocabulary.setUndoHandler(_undoMenuManager.getUndoHandler());
    }

    /**
     * Over-ride this so we can set view stuff
     */
    public void initView() {
        _tablePanel.initView();
    }

    /**
     * @return the UndoManager
     */
    public ReloadUndoMenuManager getUndoManager() {
        return _undoMenuManager;
    }

    /**
     * Dynamically load a Vocabulary Schema
     */
    public void loadSchema(VocabularyProfile vxProfile) {
        // Tell Schema Controller
        VocabularySchemaController vocabSchemaController = ((VocabularySchemaController)getVocabulary().getReloadSchemaController()).setVocabularySchema(vxProfile);

        _vocabulary.updateDocument(vocabSchemaController);
    }


    /**
     * Dynamically load a Vocabulary Profile
     */
    public void loadProfile(VocabularyProfile vxProfile) {
        // Tell Schema Controller
        ((VocabularySchemaController)getVocabulary().getReloadSchemaController()).setVocabularyProfile(vxProfile);

        // Tell Panels
        if(_vocabulary != null) {
            SwingUtilities.invokeLater(new Runnable() {
                public void run() {

                    _formPanel.setDocument(_vocabulary);

                    _tablePanel.setDocument(_vocabulary);

                    _tablePanel.refresh();
                }
            });
        }
    }

    /**
     * Set the Default Language
     * @param language en, fr, de etc
     */
    public void setDefaultLanguage(String lang) {
       ((VocabularySchemaController)getVocabulary().getReloadSchemaController()).setDefaultLanguage(lang);
    }

    /**
     * Dispose of this and clean up
     */
    public void destroy() {
        try {
            if(_undoMenuManager != null) _undoMenuManager.destroy();
            _formPanel.destroy();
            _tablePanel.destroy();
            _vocabulary.destroy();
        }
        catch(Exception ex) {
            ex.printStackTrace();
        }
    }

    /**
     * The Preferences Panel
     */
    class VocabularyPrefsPanel extends JPanel {

        /**
         * Profile Combo box
         */
        ReloadComboBox profileCombobox;

        /**
         * Language Combo box
         */
        ReloadComboBox languageCombobox;

        String profileType;

        /**
         * Constructor
         */
        VocabularyPrefsPanel() {
            setLayout(new FlowLayout(FlowLayout.LEFT));

            // PROFILE COMBO BOX
            add(new JLabel("Profile: "));
            profileCombobox = new ReloadComboBox(false);
            add(profileCombobox);

            // Populate it
            VocabularyProfile[] profiles = VocabularyProfile.getVocabularyProfiles();
            profileCombobox.setItems(profiles);

            // LANGUAGE COMBO BOX
            add(new JLabel("Default Language: "));
            languageCombobox = new ReloadComboBox(Locale.getISOLanguages());
            languageCombobox.setSelectedItem(Locale.getDefault().getLanguage());
            add(languageCombobox);
        }

        /**
         * Initialise
         */
        public void init(Vocabulary vocabulary) {

            VocabularyProfile vxProfile = ((VocabularySchemaController)vocabulary.getReloadSchemaController()).getVocabularyProfile();

            profileCombobox.setSelectedItem(vxProfile);
            profileType = vxProfile.getProfileName();

            // Listen to changes on the Profile Combo
            profileCombobox.addActionListener(new ActionListener() {

                public void actionPerformed(ActionEvent e) {

               	    VocabularyProfile profile = (VocabularyProfile)profileCombobox.getSelectedItem();

					if(!profile.getProfileName().equals(profileType)) {

			        	JOptionPane.showMessageDialog(ReloadEditor.getInstance(),
                    	                  "Changing the profile type will delete or add elements according to what is permitted and required for that profile type",
                    	                  "Reload Editor",
                    	                  JOptionPane.WARNING_MESSAGE);

                	    profileType = profile.getProfileName();

                	    loadSchema(profile);
                	    loadProfile(profile);
                	    setDefaultLanguage(languageCombobox.getSelectedItem().toString());
					}
                }
            });

            // Listen to changes on the Language Combo
            languageCombobox.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    setDefaultLanguage(languageCombobox.getSelectedItem().toString());
                }
            });
        }

        /**
         * set profile type
         */
		public void setProfileType(String pType) {

			for(int i = 0; i < profileCombobox.getItemCount(); i++) {

				VocabularyProfile vp = (VocabularyProfile) profileCombobox.getItemAt(i);

				if(vp.getProfileName().equals(pType)) {

					profileCombobox.setSelectedItem(vp);

					break;
				}
			}
		}


        /**
         * get profile type
         */
		public String getProfileType() {

			return profileType;
		}

    }

}