/**
  *  RELOAD Editor - An IMS Metadata, Content Packaging and Learning Design tool.
  *
  *  Copyright (c) 2003 Oleg Liber, Bill Olivier, Phillip Beauvoir, Phil Barker, Tina Manoharan
  *
  *  Permission is hereby granted, free of charge, to any person obtaining a copy
  *  of this software and associated documentation files (the "Software"), to deal
  *  in the Software without restriction, including without limitation the rights
  *  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
  *  copies of the Software, and to permit persons to whom the Software is
  *  furnished to do so, subject to the following conditions:
  *
  *  The above copyright notice and this permission notice shall be included in
  *  all copies or substantial portions of the Software.
  *
  *  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  *  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  *  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
  *  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
  *  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
  *  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
  *
  *  Project Management Contact:
  *
  *  Oleg Liber
  *  Bolton Institute of Higher Education
  *  Deane Road
  *  Bolton BL3 5AB
  *  UK
  *
  *  e-mail:   o.liber@bolton.ac.uk
  *
  *  Phil Barker
  *  ICBL, Heriot-Watt University
  *  Riccarton
  *  Edinburgh EH14 4AS
  *  UK
  *
  *  e-mail:   philb@icbl.hw.ac.uk
  *
  *
  *  Technical Contact:
  *
  *  Phillip Beauvoir
  *  e-mail:   p.beauvoir@bolton.ac.uk
  *
  *  Web:      http://www.reload.ac.uk
  *
  *  Tina Manoharan
  *  e-mail:   ceetm2@macs.hw.ac.uk
  *
  */

package uk.ac.reload.vocabulary;

import java.io.*;

import javax.swing.*;
import javax.swing.event.*;

import org.jdom.*;
import uk.ac.reload.*;
import uk.ac.reload.dom.*;
import uk.ac.reload.menu.*;
import uk.ac.reload.util.*;


/**
 * The Vocabulary Editor Frame
 *
 */
public class VocabularyEditorFrame
extends ReloadInternalFrame
implements ApplicationListener, ReloadDocumentListener
{
    /**
     * The Vocabulary Editor
     */
    protected VocabularyEditor _vxEditor;

    /**
     * Handles Save and SaveAs Events
     */
    protected ProxySaveHandler _saveHandler, _saveAsHandler;


    /**
     * Constructor
     */
    public VocabularyEditorFrame() {
        super("Vocabulary", ReloadUIManager.ICON_VX);

        // Ensure we handle closing
        setDefaultCloseOperation(JInternalFrame.DO_NOTHING_ON_CLOSE);

        // Tell the application that we want to know if it's closing
        ReloadEditor.getInstance().addApplicationListener(this);

        // We listen for what this frame is doing
        addInternalFrameListener(new InternalFrameAdapter() {
            // The Frame is closing
            public void internalFrameClosing(InternalFrameEvent e) {
                // Make sure button stuff is completed
                SwingUtilities.invokeLater(new Runnable() {
                    public void run() {
                        checkNeedsSaving();
                    }
                });
            }

            // The Frame is activated
            public void internalFrameActivated(InternalFrameEvent e) {
                setFocusGained();
            }

            // The Frame is deactivated
            public void internalFrameDeactivated(InternalFrameEvent e) {
                setFocusLost();
            }
        });

        // New VocabularyEditor Panel
        _vxEditor = new VocabularyEditor(MainMenu.sharedInstance.editMenu);
        getContentPane().add(_vxEditor);
    }


    public VocabularyEditor getVocabularyEditor() {

		return _vxEditor;
	}

    /**
     * Create a new Vocabulary JDOM Document
     * @param vocabulary The Vocabulary JDOM Document
     * @throws ReloadException
     */
    public void newDocument(VocabularySchemaController schemaController) throws ReloadException {

        setTitle("Vocabulary - untitled");
        _vxEditor.newDocument(true, schemaController);
        init(_vxEditor.getVocabulary());
    }

    /**
     * Set a Vocabulary JDOM Document - this will be standalone
     * @throws ReloadException
     */
    public void setDocument(File file, Document doc, VocabularySchemaController schemaController) throws ReloadException {
        setTitle("Vocabulary - " + file.getPath());
        // Load
        _vxEditor.setDocument(file, doc, schemaController);
        // Add to file->frame map
        ReloadInternalFrameManager.addReloadInternalFrameToWindowMap(file, this);
        // Init
        init(_vxEditor.getVocabulary());
    }

    /**
     * Initialise some stuff
     */
    private void init(Vocabulary vocabulary) {
        // Document listener
        vocabulary.addReloadDocumentListener(this);

        // Proxy Save Handlers
        _saveHandler = new ProxySaveHandler(MainMenu.sharedInstance.actionSave);
        _saveAsHandler = new ProxySaveHandler(MainMenu.sharedInstance.actionSaveAs);
    }

    /**
     * Over-ride this so we can set view stuff
     */
     public void show() {
        super.show();
        _vxEditor.initView();
    }


// =============================================================================
// ReloadDocument Listener Events so we know when to set Save Menu
// =============================================================================

    public void elementAdded(ReloadDocumentListenerEvent e) {
        if(isSelected()) _saveHandler.setEnabled(_vxEditor.getVocabulary().isDirty());
    }

    public void elementChanged(ReloadDocumentListenerEvent e) {
        if(isSelected()) _saveHandler.setEnabled(_vxEditor.getVocabulary().isDirty());
    }

    public void elementRemoved(ReloadDocumentListenerEvent e) {
        if(isSelected()) _saveHandler.setEnabled(_vxEditor.getVocabulary().isDirty());
    }

    public void documentSaved(ReloadDocument doc) {
        if(isSelected()) _saveHandler.setEnabled(_vxEditor.getVocabulary().isDirty());
    }

// =============================================================================
// Focus Handler
// =============================================================================

    /**
     * We got the focus
     */
    protected void setFocusGained() {
        if(_vxEditor.getVocabulary() != null) {
            _vxEditor.getUndoManager().setFocusGained();

            _saveHandler.setEnabled(_vxEditor.getVocabulary().isDirty());
            _saveHandler.addListener();

            _saveAsHandler.setEnabled(true);
            _saveAsHandler.addListener();
        }
    }

    /**
     * We lost the focus
     */
    protected void setFocusLost() {
        _vxEditor.getUndoManager().setFocusLost();

        _saveHandler.setEnabled(false);
        _saveHandler.removeListener();

        _saveAsHandler.setEnabled(false);
        _saveAsHandler.removeListener();
    }

// =============================================================================
// Window Closing, Cleanup and Saving Handlers
// =============================================================================

    /**
     * We have been told by the Application that it is closing.
     * Here we can save if need be.
     * @return true if all is OK, false if not.
     */
    public boolean applicationClosing() {
        return checkNeedsSaving();
    }

    /**
     * We can check if the File needs saving
     * @return true if all is OK, false if not.
     */
    protected boolean checkNeedsSaving() {
        boolean isOK = true;

        if(_vxEditor.getVocabulary() != null && _vxEditor.getVocabulary().isDirty()) {
            int doSave = JOptionPane.showConfirmDialog(this,
                "File has been modified. Do you wish to save changes?",
                "Save Vocabulary File",
                JOptionPane.YES_NO_CANCEL_OPTION);

            if(doSave == JOptionPane.YES_OPTION) isOK = saveDocument();
            else if(doSave == JOptionPane.CANCEL_OPTION) isOK = false;
        }

        if(isOK) dispose();

        return isOK;
    }

    /**
     * Dispose of this Window and clean up
     */
    public void dispose() {
        try {
            ReloadEditor.getInstance().removeApplicationListener(this);

            _saveHandler.removeListener();
            _saveAsHandler.removeListener();

            _vxEditor.getVocabulary().removeReloadDocumentListener(this);
            _vxEditor.destroy();

            super.dispose();
        }
        catch(Exception ex) {
            ex.printStackTrace();
        }
    }

    /**
     * If the Vocabulary file has been edited, ask whether we should save it
     * @return true if OK, false if not
     */
    protected boolean saveDocument() {
        // If we don't have a file name, ask for one
        if(_vxEditor.getVocabulary().getFile() == null) return saveAsDocument();

        try {
            _vxEditor.getVocabulary().saveDocument();
        }
        catch(ReloadException ex) {
            ErrorDialogBox.showWarning(this, "Could not save file: ", "Save Vocabulary", ex);
            return false;
        }
        return true;
    }

    /**
     * Save As...
     * @return
     */
    protected boolean saveAsDocument() {
        // Ask for name
        ReloadFileFilter filter = new ReloadFileFilter(new String[] {"xml"}, "xml files");
        File file = XMLUtils.askFileNameSave("Save Vocabulary", filter);
        // User cancelled
        if(file == null)  return false;

        // Set Title bar
        setTitle("Vocabulary - " + file.getPath());

        try {
            _vxEditor.getVocabulary().saveAsDocument(file);

            // Put it in the opened list
            ReloadInternalFrameManager.addReloadInternalFrameToWindowMap(file, this);
        }
        catch(ReloadException ex) {
            ErrorDialogBox.showWarning(this, "Could not save file: ", "Save Vocabulary", ex);
            return false;
        }
        return true;
    }


    /**
     * Deals with Save Menu Events
     */
    class ProxySaveHandler extends ProxyAction {
        public ProxySaveHandler(MenuAction menuAction) {
            super(menuAction);
        }

        public void menuActionPerformed(ReloadMenuEvent event) {
            if(isSelected() && event.getSource() == MainMenu.sharedInstance.actionSave) saveDocument();
            if(isSelected() && event.getSource() == MainMenu.sharedInstance.actionSaveAs) saveAsDocument();
        }
    }

}