/**
 *  RELOAD Editor - An IMS Metadata, Content Packaging and Learning Design tool.
 *
 *  Copyright (c) 2003 Oleg Liber, Bill Olivier, Phillip Beauvoir
 *
 *  Permission is hereby granted, free of charge, to any person obtaining a copy
 *  of this software and associated documentation files (the "Software"), to deal
 *  in the Software without restriction, including without limitation the rights
 *  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 *  copies of the Software, and to permit persons to whom the Software is
 *  furnished to do so, subject to the following conditions:
 *
 *  The above copyright notice and this permission notice shall be included in
 *  all copies or substantial portions of the Software.
 *
 *  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 *  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 *  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 *  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 *  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 *  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 *
 *  Project Management Contact:
 *
 *  Oleg Liber
 *  Bolton Institute of Higher Education
 *  Deane Road
 *  Bolton BL3 5AB
 *  UK
 *
 *  e-mail:   o.liber@bolton.ac.uk
 *
 *
 *  Technical Contact:
 *
 *  Phillip Beauvoir
 *  e-mail:   p.beauvoir@bolton.ac.uk
 *
 *  Web:      http://www.reload.ac.uk
 *
 */


package uk.ac.reload.widgets;

import org.jdom.*;
import uk.ac.reload.dom.*;
import uk.ac.reload.schema.*;

/**
 * A Widget factory for a view.
 *
 * @author Phillip Beauvoir
 * @version 2003-08-01
 */
public final class ReloadWidgetFactory {

    /**
     * Constructor
     */
    private ReloadWidgetFactory() {
    }

    /**
     * @return A Text Widget suitable for the Value Panel display.
     * This actually needs to be mapped in a Helper File
     */
    public static ReloadWidget getWidget(ReloadDocument doc, ReloadSchemaController schemaController, ReloadSchemaElement schemaElement) {
        // Default editor for unknown type
        if(schemaElement == null) {

			return getReloadTextField();
		}

        ReloadWidget component = null;

        // Try for a Vocabulary first
        ReloadVocabularyList vocab = schemaController.getVocabularyList(schemaElement);
        if(vocab != null) {

            component = getReloadListBox();
            ((ReloadListBox) component).setListData(vocab.getList());
            ((ReloadListBox) component).setList(vocab);
            return component;
        }

        String name = schemaElement.getName();

        String widget_type = schemaController.getWidgetType(schemaElement.getDomPath());
        if(widget_type != null) {
            if(widget_type.equals("textpane")) {

				component = getReloadTextPane();
			}
            else if(widget_type.equals("numberfield")) {

				component = getReloadNumberField();
			}
            else if(widget_type.equals("filechooser")) {

				component = getReloadFileTextField();
			}
			else if(widget_type.equals("buttontextfield")) {

				component = getReloadButtonTextField(doc);
				((ReloadButtonTextField)component).setColumns(30);

				// Set Vdex Source path
				String vdexSourcePath = schemaController.getFacetValue(schemaElement, "vdexSourcePath");
				if(vdexSourcePath != null) ((ReloadButtonTextField)component).setVdexSourcePath(vdexSourcePath);

				// Set Element DomPath
				((ReloadButtonTextField)component).setSchemaElement(schemaElement);
			}
			else if(widget_type.equals("buttontextpane")) {

				component = getReloadButtonTextPane(doc);

				// Set Vdex Source path
				String vdexSourcePath = schemaController.getFacetValue(schemaElement, "vdexSourcePath");
				if(vdexSourcePath != null) ((ReloadButtonTextPane)component).setVdexSourcePath(vdexSourcePath);

				// Set Element DomPath
				((ReloadButtonTextPane)component).setSchemaElement(schemaElement);
			}
            else {

				component = getReloadTextField();
			}
        }
        // Default
        else {

			component = getReloadTextField();
		}

        // Set Max String length
        int maxLength = -1;
        String value = schemaController.getFacetValue(schemaElement, "maxLength");
        if(value != null) maxLength = Integer.parseInt(value);
        component.setMaxLength(maxLength);

        return component;
    }

    /**
     * @return the Widget
     */
    public static ReloadTextField getReloadTextField() {
        return new ReloadTextField();
    }

    /**
     * @return the Widget
     */
    public static ReloadListBox getReloadListBox() {
        return new ReloadListBox();
    }

    /**
     * @return the Widget
     */
    public static ReloadTextPane getReloadTextPane() {
        return new ReloadTextPane();
    }

    /**
     * @return the Widget
     */
    public static ReloadNumberField getReloadNumberField() {
        return new ReloadNumberField();
    }

    /**
     * @return a File chooser
     */
    public static ReloadFileTextField getReloadFileTextField() {
        return new ReloadFileTextField(ReloadFileTextField.FILE_TYPE, "Select File");
    }

    /**
     * @return a TextField and Button
     */
    public static ReloadButtonTextField getReloadButtonTextField(ReloadDocument doc) {
        return new ReloadButtonTextField(doc, ReloadButtonTextField.VDEX_BROWSE_TYPE, "Browse");
    }

    /**
     * @return a TextPane and Button
     */
    public static ReloadButtonTextPane getReloadButtonTextPane(ReloadDocument doc) {
        return new ReloadButtonTextPane(doc, ReloadButtonTextPane.VDEX_BROWSE_TYPE, "Browse");
    }
}