/**
  *  RELOAD Editor - An IMS Metadata, Content Packaging and Learning Design tool.
  *
  *  Copyright (c) 2003 Oleg Liber, Bill Olivier, Phillip Beauvoir
  *
  *  Permission is hereby granted, free of charge, to any person obtaining a copy
  *  of this software and associated documentation files (the "Software"), to deal
  *  in the Software without restriction, including without limitation the rights
  *  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
  *  copies of the Software, and to permit persons to whom the Software is
  *  furnished to do so, subject to the following conditions:
  *
  *  The above copyright notice and this permission notice shall be included in
  *  all copies or substantial portions of the Software.
  *
  *  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  *  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  *  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
  *  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
  *  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
  *  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
  *
  *  Project Management Contact:
  *
  *  Oleg Liber
  *  Bolton Institute of Higher Education
  *  Deane Road
  *  Bolton BL3 5AB
  *  UK
  *
  *  e-mail:   o.liber@bolton.ac.uk
  *
  *
  *  Technical Contact:
  *
  *  Phillip Beauvoir
  *  e-mail:   p.beauvoir@bolton.ac.uk
  *
  *  Web:      http://www.reload.ac.uk
  *
  */

package uk.ac.reload.widgets.tree;

import java.awt.*;
import java.awt.dnd.*;
import javax.swing.tree.*;
import javax.swing.*;
import java.awt.event.*;
import java.awt.image.*;
import uk.ac.reload.util.*;

/**
 * A Tree based on ReloadTree with support for Drag and Drop
 *
 * @author Phillip Beauvoir
 * @author Paul Sharples
 * @version 2003-07-01
 */
public abstract class ReloadDragTree
extends ReloadTree
implements DragSourceListener, DragGestureListener, DropTargetListener, Autoscroll
{
    /**
    * The previous highlighted node
    */
    protected ReloadTreeNode prevHilitedNode;

    /**
     * The last dragged path
     */
    private TreePath _pathLast;

    /**
     * A Timer for opening hovered over node
     */
    private Timer _timerHover;

    /**
     * Default Constructor
     */
    public ReloadDragTree() {
        super();

        // Set up Timer for hover action
        _timerHover = new Timer(1000, new ActionListener() {
            public void actionPerformed(ActionEvent e)
            {
                if(isRootPath(_pathLast)) return;
                if(isCollapsed(_pathLast)) expandPath(_pathLast);
                //else collapsePath(_pathLast);
            }
        });
        _timerHover.setRepeats(false);	// Set timer to one-shot mode
    }

    /**
     * Highlight a node on a tree as we drag over it.  It's up to the Tree Renderer
     * to actually draw the highlight.
     * @param node  The node we are highlighting
     * @param hilite True or false
     */
    protected void hiliteNode(ReloadTreeNode node, boolean hilite) {
        if(node == null) return;
        if(prevHilitedNode != null && node != prevHilitedNode) {
            prevHilitedNode.isHiLited = false;
        }
        if(node.isHiLited != hilite) {
            node.isHiLited = hilite;
            repaint();
            prevHilitedNode = node;
        }
    }

    /**
     * Scroll support for Drag and Drop
     * @return
     */
    public Insets getAutoscrollInsets() {
        Rectangle r = getVisibleRect();
        Dimension s = getSize();
        Insets insets = new Insets(r.y + 50,
            r.x,
            s.height - r.y - r.height + 50,
            s.width - r.x - r.width);
        //System.out.println(insets);
        return insets;
    }

    /**
     * Scroll support for Drag and Drop
     * @param cursorLocn
     */
    public void autoscroll(Point cursorLocn) {
        //System.out.println("called autoscroll @ " + cursorLocn.x + ", " + cursorLocn.y);
        Rectangle v = getVisibleRect();
        int y = cursorLocn.y < (v.y + 50) ? cursorLocn.y - 40 : cursorLocn.y;
        Rectangle r = new Rectangle(0, y, getVisibleRect().width, 40);
        scrollRectToVisible(r);
    }

    /**
    * Return the tree node that has been picked up and dragged
    */
    protected ReloadTreeNode getDragSourceTreeNode(DragGestureEvent event) {
        ReloadTreeNode node = null;
        Point location = event.getDragOrigin();
        TreePath dragPath = getPathForLocation(location.x, location.y);
        if(dragPath != null && isPathSelected(dragPath)) {
            node = (ReloadTreeNode)dragPath.getLastPathComponent();
        }
        return node;
    }

    /**
    * Returns the node that is being dragged over
    */
    protected ReloadTreeNode getDragOverTreeNode(DropTargetDragEvent event) {
        ReloadTreeNode node = null;
        Point location = event.getLocation();
        TreePath dragPath = getPathForLocation(location.x, location.y);
        if(dragPath != null) {
            node = (ReloadTreeNode)dragPath.getLastPathComponent();
        }
        return node;
    }

    /**
     * @return a Drag Image
     */
    public BufferedImage getDragImage(TreePath path, Point ptDragOrigin) {
        BufferedImage imgGhost;
        Point ptOffset = new Point();

        // Work out the offset of the drag point from the TreePath bounding rectangle origin
        Rectangle raPath = getPathBounds(path);
        ptOffset.setLocation(ptDragOrigin.x-raPath.x, ptDragOrigin.y-raPath.y);

        // Get the cell renderer (which is a JLabel) for the path being dragged
        JLabel lbl = (JLabel) getCellRenderer().getTreeCellRendererComponent
        (
            this,                                           // tree
            path.getLastPathComponent(),                    // value
            false,                                          // isSelected (dont want a colored background)
            isExpanded(path),                               // isExpanded
            getModel().isLeaf(path.getLastPathComponent()), // isLeaf
            0,                                              // row (not important for rendering)
            false                                           // hasFocus (dont want a focus rectangle)
        );
        lbl.setSize((int)raPath.getWidth(), (int)raPath.getHeight());

        // Get a buffered image of the selection for dragging a ghost image
        imgGhost = new BufferedImage((int)raPath.getWidth(), (int)raPath.getHeight(), BufferedImage.TYPE_INT_ARGB_PRE);
        Graphics2D g2 = imgGhost.createGraphics();

        // Ask the cell renderer to paint itself into the BufferedImage
        g2.setComposite(AlphaComposite.getInstance(AlphaComposite.SRC, 0.5f));		// Make the image ghostlike
        lbl.paint(g2);

        return imgGhost;
    }



    /**
    * Give feedback on whether it's OK to drop here
    * Over-ride this with something sensible
    */
    public abstract boolean isDropOK(DropTargetDragEvent event);

    // ====================== Over-ride these! =================================

    public void dragExit(DragSourceEvent event) {
        //System.out.println("dragExit source");
        DragSourceContext context = event.getDragSourceContext();
        context.setCursor(DragSource.DefaultCopyNoDrop);
        hiliteNode(prevHilitedNode, false);
    }

    public void dropActionChanged(DragSourceDragEvent event) {
        //System.out.println("dropActionChanged source");
    }

    public void dragOver(DragSourceDragEvent event) {
        //System.out.println("dragOver source");
    }

    public void dragEnter(DragSourceDragEvent event) {
        //System.out.println("dragEnter source");
        DragSourceContext context = event.getDragSourceContext();
        int action = event.getDropAction();
        if((action & DnDConstants.ACTION_COPY) != 0) context.setCursor(DragSource.DefaultCopyDrop);
        else if((action & DnDConstants.ACTION_MOVE) != 0) context.setCursor(DragSource.DefaultMoveDrop);
        else context.setCursor(DragSource.DefaultCopyNoDrop);
    }

    public void dragDropEnd(DragSourceDropEvent event) {
        //System.out.println("dragDropEnd source");
        hiliteNode(prevHilitedNode, false);
    }

    public void dragEnter(DropTargetDragEvent event) {
        //System.out.println("dragEnter target");
        if(isDropOK(event)) event.acceptDrag(getCorrectDropContext(event));
        else event.rejectDrag();
    }

    /**
    * We are being dragged over - select the Node if OK
    * Over-ride and call super if need be
    */
    public void dragOver(DropTargetDragEvent event) {
        //System.out.println("dragOver target");
        boolean ok = isDropOK(event);
        if(ok) {
            event.acceptDrag(getCorrectDropContext(event));
            ReloadTreeNode node = getDragOverTreeNode(event);
            hiliteNode(node, true);
        }
        else {
            event.rejectDrag();
            //hiliteNode(prevHilitedNode, false);
        }

        // Do the Hover over expand/collapse thing
        Point pt = event.getLocation();
        TreePath path = getClosestPathForLocation(pt.x, pt.y);
        if(!(path == _pathLast)) {
            _pathLast = path;
            _timerHover.restart();
        }
    }

    public void dragExit(DropTargetEvent event) {
        //System.out.println("dragExit target");
        hiliteNode(prevHilitedNode, false);
    }

    public void dropActionChanged(DropTargetDragEvent event) {
        //System.out.println("dropActionChanged target");
        if(isDropOK(event)) event.acceptDrag(getCorrectDropContext(event));
        else event.rejectDrag();
    }

    public void drop(DropTargetDropEvent event) {}
    public void dragGestureRecognized(DragGestureEvent event) {}

    /**
     * <h3>Amendment History</h3>
     * <p>Due to Mac specific issues, this method has been added to ensure
     * that the correct drop target event is recorded.  On MacOS X
     * "event.getDropAction()" does not return 1 for move and 2 for copy as
     * it should do (like it does on PC).  This method checks to see
     * if the OS is Mac and then makes sure the correct int value is returned</p>
     *
     * @param event a DropTargetDropEvent
     * @return int representing the type of action - move or copy
     */
    public int getCorrectDropContext(DropTargetDropEvent event){
       int OS = ReloadUtils.getOS();
       if(OS == ReloadUtils.MACINTOSH){
           int result = event.getDropAction();
           switch(result) {
               case 0:
                   return 1;
               case 1:
                   return 2;
               default:
                   return 1;
           }
       }
       return event.getDropAction();
    }

    /**
     * <h3>Amendment History</h3>
     * <p>Due to Mac specific issues, this method has been added to ensure
     * that the correct drop target event is recorded.  On MacOS X
     * "event.getDropAction()" does not return 1 for move and 2 for copy as
     * it should do (like it does on PC).  This method checks to see
     * if the OS is Mac and then makes sure the correct int value is returned</p>
     *
     * @param event a DropTargetDropEvent
     * @return int representing the type of action - move or copy
     */
    public int getCorrectDropContext(DropTargetDragEvent event){
       int OS = ReloadUtils.getOS();
       if(OS == ReloadUtils.MACINTOSH){
           int result = event.getDropAction();
           switch(result) {
               case 0:
                   return 1;
               case 1:
                   return 2;
               default:
                   return 1;
           }
       }
       return event.getDropAction();
    }

}